﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;

namespace DockSample
{
    public class Coupling : GenericLoadComponent, ILoad
    {
        private new CouplingBrand _brand;
        private CouplingType _subType;

        private Geometry _couplingGeometry = new Geometry();
        public event EventHandler<RefreshPropertyGridEventArgs> RefreshPropertyGridEvent;

        protected MechInterface _outType;
        protected double _outMinDia;
        protected double _outMaxDia;
        protected double _outRadFrc;
        protected double _outAxFrc;

        public Coupling() {}

        public Coupling(string name)
        {
            _name = name;
            _mainType = MainType.Coupling;
            _inpType = MechInterface.SmoothHollow;
            _outType = MechInterface.SmoothHollow;
            
            _eff = 100;

            _couplingGeometry.PropertyChanged += new PropertyChangedEventHandler(_geometry_PropertyChanged);
        }

        private void _geometry_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (CouplingGeometry.ManualInertiaCalc)
                CalcInertia();
        }

        [Category("Component"), ReadOnly(false), Description("Brand name displayed on component.")]
        public new CouplingBrand Brand
        {
            get { return _brand; }
            set
            {
                if (value != this._brand)
                {
                    this._brand = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Brand");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Coupling type.")]
        public CouplingType SubType
        {
            get { return _subType; }
            set
            {
                if (value != this._subType)
                {
                    this._subType = value;
                    if (!AutoSelect)
                    OnPropertyChanged("SubType");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Geometry of rotating part.")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Geometry CouplingGeometry
        {
            get { return _couplingGeometry; }
            set
            {
                if (value != this._couplingGeometry)
                {
                    this._couplingGeometry = value;
                    OnPropertyChanged("CouplingGeometry");
                }
            }
        }

        private void CalcInertia()
        {
            double density = 0;

            switch (CouplingGeometry.Material)
            {
                case Material.Aluminum:
                    density = 2700;
                    break;

                case Material.Plastic:
                    density = 1500;
                    break;

                default:
                    density = 7800; //Material.Steel
                    break;
            }
            
            //Cylinder volume = PI * radius * radius * cylinderheight

            double totalVolume = Math.PI * Math.Pow(CouplingGeometry.OuterDiameter / 2000, 2) * CouplingGeometry.CylinderHeight / 1000;
            double hollowVolume = Math.PI * Math.Pow(CouplingGeometry.InnerDiameter / 2000, 2) * CouplingGeometry.CylinderHeight / 1000;

            double realVolume = totalVolume - hollowVolume;
            double mass = realVolume * density;

            CouplingGeometry.PartMass = Math.Round(mass, 3);

            CouplingGeometry.PartInertia = mass * (Math.Pow(CouplingGeometry.OuterDiameter / 2000, 2) + Math.Pow(CouplingGeometry.InnerDiameter / 2000, 2)) * 0.5;

            Inertia = CouplingGeometry.PartInertia;

            RaiseRefreshPropertyGridEvent();
        }

        private void RaiseRefreshPropertyGridEvent()
        {
            EventHandler<RefreshPropertyGridEventArgs> handler = RefreshPropertyGridEvent;
            if (handler != null)
                handler(null, new RefreshPropertyGridEventArgs());
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Mechanical output type.")]
        public MechInterface OutType
        {
            get { return _outType; }
            set
            {
                if (value != this._outType)
                {
                    this._outType = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutType");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Minimum output diameter in millimeters (mm).")]
        public double OutMinDia
        {
            get { return _outMinDia; }
            set
            {
                if (value != this._outMinDia)
                {
                    this._outMinDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutMinDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum output diameter in millimeters (mm).")]
        public double OutMaxDia
        {
            get { return _outMaxDia; }
            set
            {
                if (value != this._outMaxDia)
                {
                    this._outMaxDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutMaxDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum radial force in newtons (N) on the mechanical output.")]
        public double OutRadFrc
        {
            get { return _outRadFrc; }
            set
            {
                if (value != this._outRadFrc)
                {
                    this._outRadFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutRadFrc");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum axial force in newtons (N) on the mechanical output.")]
        public double OutAxFrc
        {
            get { return _outAxFrc; }
            set
            {
                if (value != this._outAxFrc)
                {
                    this._outAxFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutAxFrc");
                }
            }
        }
    }
}
