﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;
using System.Reflection.Emit;

namespace DockSample
{
    public sealed class Settings : NotifyPropertyChangedBase
    {
        //General
        private static bool _mechChkOnNonAutoSelCmp;
        private static Condition _conditionFilter;

        //Units
        private static int _resolution;
        private static TimeBase _mainTimeBase;
        
        //Calculation margins
        private static int _speedMargin;
        private static int _torqueMargin;
        private static int _allowedLiftTorque; 
        private static int _inertiaMargin;        
        
        //Motor preferences
        private static Criteria _useBrakedMotor;
        private static MotorFamily _motorFamilyFilter;

        //Gearbox preferences
        private static UseGearBoxCriteria _useGearBox;
        private static GearBoxBrand _gearBoxBrandFilter;
        private static GearBoxType _gearBoxTypeFilter;


        //private static List<string> _gearBoxFamilyFilter = new List<string>()
        private static string _gearBoxFamilyFilter;
        
        private static int _gearBoxOverLoadMargin;

        //Regenerative data
        private static int _maxOverloadFactor;
        private static int _maxDuration;
        private static int _inverseEfficiency;
        private static int _capacitorCharge;

        static readonly Settings _mainSettings = new Settings();

        public static Settings MainSettings
        {
            get { return _mainSettings; }
        }

        /// <summary>
        /// This is a private constructor, meaning no outsiders have access.
        /// </summary>
        private Settings()
        {
            _mechChkOnNonAutoSelCmp = false;
            _conditionFilter = Condition.OnlyNew;            

            _mainTimeBase = TimeBase.MilliSecond;
            _resolution = 1000;
            _torqueMargin = 15;
            _allowedLiftTorque = 70;
            
            _useBrakedMotor = Criteria.Auto;
            _motorFamilyFilter = MotorFamily.Undefined;
            _useGearBox = UseGearBoxCriteria.Auto;
            _gearBoxBrandFilter = GearBoxBrand.LSIS;
            _gearBoxTypeFilter = GearBoxType.Undefined;
            _gearBoxFamilyFilter = "Undefined";

           _maxOverloadFactor = 5;
           _maxDuration = 5000;
           _inverseEfficiency = 80;
           _capacitorCharge = 10;
        }

        [Category("General"), ReadOnly(false), Description("Check mechanical fit on non auto selected components.")]
        public bool MechChkOnNonAutoSelCmp
        {
            get { return _mechChkOnNonAutoSelCmp; }
            set { _mechChkOnNonAutoSelCmp = value; }
        }

        [Category("General"), ReadOnly(false), Description("Include condition code components in search?")]
        public Condition ConditionFilter
        {
            get { return _conditionFilter; }
            set { _conditionFilter = value; }
        }

        [Category("Units"), ReadOnly(false), Description("Sample rate for calculation of load curves. A low sample rate may cause data loss and incorrent results. A high sample rate will degrade the performance of the system and may cause out of memory alarms.")]
        // [DisplayName(,,,, 3)]
        public TimeBase MainTimeBase
        {
            get { return _mainTimeBase; }
            set
            {
                _mainTimeBase = value;
                switch (_mainTimeBase)
                {
                    case TimeBase.MilliSecond:
                        _resolution = 1000;
                        break;

                    case TimeBase.HundredMicroSeconds:
                        _resolution = 10000;
                        break;

                    case TimeBase.TenMicroSeconds:
                        _resolution = 100000;
                        break;

                    case TimeBase.MicroSecond:
                        _resolution = 1000000;
                        break;
                }
            }
        }

        [Browsable(false)]
        public int Resolution
        {
            get { return _resolution; }
            set
            {
                _resolution = value;
            }
        }

        [Category("CalculationMargins"), ReadOnly(false), Description("Smallest allowed speed margin in 0 - 100 percent when auto selecting a component.")]
        public int SpeedMargin
        {
            get { return _speedMargin; }
            set
            {
                if (value > 100)
                    value = 100;
                if (value < 0)
                    value = 0;

                _speedMargin = value;
            }
        }

        [Category("CalculationMargins"), ReadOnly(false), Description("Smallest allowed torque margin in 0 - 100 percent when auto selecting a component.")]
        public int TorqueMargin
        {
            get { return _torqueMargin; }
            set
            {
                if (value > 100)
                    value = 100;
                if (value < 0)
                    value = 0;

                _torqueMargin = value;
            }
        }        

        [Category("CalculationMargins"), ReadOnly(false), Description("Highest allowed lifting torque at standstill (for vertical applications) in 0 - 100 percent when auto selecting a component.")]
        public int AllowedLiftTorque
        {
            get { return _allowedLiftTorque; }
            set
            {
                if (value > 100)
                    value = 100;
                if (value < 0)
                    value = 0;

                _allowedLiftTorque = value;
            }
        }
        [Category("CalculationMargins"), ReadOnly(false), Description("Smallest allowed inertia ratio margin in 0 - 100 percent when auto selecting a motor.")]
        public int InertiaMargin
        {
            get { return _inertiaMargin; }
            set
            {
                if (value > 100)
                    value = 100;
                if (value < 0)
                    value = 0;

                _inertiaMargin = value;
            }
        }

        [Category("MotorPreferences"), ReadOnly(false), Description("Use motor with brake. Always, Never, Auto (program decides).")]
        public Criteria UseBrakedMotor
        {
            get { return _useBrakedMotor; }
            set { _useBrakedMotor = value; }
        }

        [Category("MotorPreferences"), ReadOnly(false), Description("Search motors of this family.")]
        public MotorFamily MotorFamilyFilter
        {
            get { return _motorFamilyFilter; }
            set { _motorFamilyFilter = value; }
        }

        [Category("GearBoxPreferences"), ReadOnly(false), Description("Use gearbox. Always, Auto (program decides).")]
        public UseGearBoxCriteria UseGearBox
        {
            get { return _useGearBox; }
            set { _useGearBox = value; }
        }

        [Category("GearBoxPreferences"), ReadOnly(false), Description("Search gearboxes with this brand.")]
        public GearBoxBrand GearBoxBrandFilter
        {
            get { return _gearBoxBrandFilter; }
            set 
            { 
                _gearBoxBrandFilter = value;
                PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["GearBoxFamilyFilter"];
                ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
                FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);

                //If the brand is changed the family filter must be updated.
                _gearBoxFamilyFilter = "Undefined";

                if (GearBoxBrandFilter == GearBoxBrand.Undefined)
                    FieldIsReadOnly.SetValue(attribute, true);
                else
                    FieldIsReadOnly.SetValue(attribute, false);
            }
        }

        [Category("GearBoxPreferences"), ReadOnly(false), Description("Search gearboxes of this type.")]
        public GearBoxType GearBoxTypeFilter
        {
            get { return _gearBoxTypeFilter; }
            set { _gearBoxTypeFilter = value; }
        }

        //The gearboxfamily is a string that by default will not be displayed as a dropdown. Use "TypeConverter" to force dropdown style.
        [Category("GearBoxPreferences"), ReadOnly(false), Description("Search gearboxes of this family."), TypeConverter(typeof(MyTypeConverter))]
        public string GearBoxFamilyFilter
        {
            get { return _gearBoxFamilyFilter; }
            set { _gearBoxFamilyFilter = value; }
        }

        //Regenerative data
        [Category("Regenerative"), ReadOnly(false), Description("Brake resistor maximum allowed relationship between nominal and peak power as an integer factor.")]
        public int MaxOverloadFactor
        {
            get { return _maxOverloadFactor; }
            set { _maxOverloadFactor = value; }
        }

        [Category("Regenerative"), ReadOnly(false), Description("Brake resistor maximum allowed peak power duration in milliseconds [ms].")]
        public int MaxDuration
        {
            get { return _maxDuration; }
            set { _maxDuration = value; }
        }

        [Category("Regenerative"), ReadOnly(false), Description("Motor natural regenerative consumption due to friction etc in percent [100% = no consumption]")]
        public int InverseEfficiency
        {
            get { return _inverseEfficiency; }
            set { _inverseEfficiency = value; }
        }

        [Category("Regenerative"), ReadOnly(false), Description("Typical regenerative energy in Joules [J] that can be consumed by the DC-link capacitor(s) in the drive.")]
        public int CapacitorCharge
        {
            get { return _capacitorCharge; }
            set { _capacitorCharge = value; }
        }
    }


    public class MyTypeConverter : TypeConverter
    {
        string[] GearBoxFamilies = { "Undefined", "SSO", "HSR"};
        
        public override bool GetStandardValuesSupported( ITypeDescriptorContext context )
        {
            return true;
        }

        public override StandardValuesCollection GetStandardValues( ITypeDescriptorContext context )
        {
            if (Settings.MainSettings.GearBoxBrandFilter == GearBoxBrand.LSIS)
            {
                GearBoxFamilies[1] = "SSO";
                GearBoxFamilies[2] = "HSR";
            }
            else
            {
                GearBoxFamilies[1] = "CP";
                GearBoxFamilies[2] = "LP";
            }
             
             
            return new StandardValuesCollection(GearBoxFamilies);
         }
    }
}
