﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DockSample
{
    public class ToothedBelt : GenericLoadComponent, ILoad
    {
        protected MechInterface _outType;
        protected double _outMinDia;
        protected double _outMaxDia;
        protected double _outRadFrc;
        protected double _outAxFrc;

        protected new string _brand;
        protected double _movePerRev;
        protected double _angle;
        
        private Geometry _driveWheelGeometry = new Geometry();
        //protected double _gearing = 1;
        private Geometry _loadWheelGeometry = new Geometry();

        public event EventHandler<RefreshPropertyGridEventArgs> RefreshPropertyGridEvent;

        public ToothedBelt() { }

        public ToothedBelt(string name)
        {
            _name = name;
            _mainType = MainType.ToothedBelt;
            _brand = "Undefined"; 

            _driveWheelGeometry.PropertyChanged += new PropertyChangedEventHandler(_driveWheelGeometry_PropertyChanged);
            _loadWheelGeometry.PropertyChanged += new PropertyChangedEventHandler(_loadWheelGeometry_PropertyChanged);
        }

        void _driveWheelGeometry_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (DriveWheelGeometry.ManualInertiaCalc)
                CalcInertiaAndGearing();
        }

        void _loadWheelGeometry_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (LoadWheelGeometry.ManualInertiaCalc)
                CalcInertiaAndGearing();
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Mechanical output type.")]
        public MechInterface OutType
        {
            get { return _outType; }
            set
            {
                if (value != this._outType)
                {
                    this._outType = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutType");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Minimum output diameter in millimeters (mm).")]
        public double OutMinDia
        {
            get { return _outMinDia; }
            set
            {
                if (value != this._outMinDia)
                {
                    this._outMinDia = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutMinDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum output diameter in millimeters (mm).")]
        public double OutMaxDia
        {
            get { return _outMaxDia; }
            set
            {
                if (value != this._outMaxDia)
                {
                    this._outMaxDia = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutMaxDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum radial force in newtons (N) on the mechanical output.")]
        public double OutRadFrc
        {
            get { return _outRadFrc; }
            set
            {
                if (value != this._outRadFrc)
                {
                    this._outRadFrc = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutRadFrc");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum axial force in newtons (N) on the mechanical output.")]
        public double OutAxFrc
        {
            get { return _outAxFrc; }
            set
            {
                if (value != this._outAxFrc)
                {
                    this._outAxFrc = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutAxFrc");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Brand name displayed on component.")]
        public new string Brand
        {
            get { return _brand; }
            set
            {
                if (value != this._brand)
                {
                    this._brand = value;
                    //if (!AutoSelect)
                    OnPropertyChanged("Brand");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false),
        DescriptionAttribute("Movement amount in meters (m) per motor revolution.")]
        public double MovePerRev
        {
            get { return _movePerRev; }
            set
            {
                if (value != this._movePerRev)
                {
                    this._movePerRev = value;
                    //if (!AutoSelect)
                    OnPropertyChanged("MovePerRev");
                }
            }
        }

        [Category("Main"), Description("Inclination between 90 and -90 degrees (°). 0 = Horizontal.")]
        public double Angle
        {
            get { return _angle; }
            set
            {
                if (value != this._angle)
                {
                    if (value > 90)
                        value = 90;

                    if (value < -90)
                        value = -90;

                    this._angle = value;

                    //if (!AutoSelect)
                    OnPropertyChanged("Angle");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Geometry of rotating part.")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Geometry DriveWheelGeometry
        {
            get { return _driveWheelGeometry; }
            set
            {
                if (value != this._driveWheelGeometry)
                {
                    this._driveWheelGeometry = value;
                    OnPropertyChanged("DriveWheelGeometry");
                }
            }
        }

        /*[Category("Main"), ReadOnly(false), Description("Total gearing (no unit).")]
        public double Gearing
        {
            get { return _gearing; }
            set
            {
                if (value != this._gearing)
                {
                    this._gearing = value;
                    if (!AutoSelect)
                        OnPropertyChanged("Gearing");
                }
            }
        }*/

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Geometry of rotating part.")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Geometry LoadWheelGeometry
        {
            get { return _loadWheelGeometry; }
            set
            {
                if (value != this._loadWheelGeometry)
                {
                    this._loadWheelGeometry = value;
                    OnPropertyChanged("LoadWheelGeometry");
                }
            }
        }

        private void CalcInertiaAndGearing()
        {
            double drDensity = 0;

            switch (DriveWheelGeometry.Material)
            {
                case Material.Aluminum:
                    drDensity = 2700;
                    break;

                case Material.Plastic:
                    drDensity = 1500;
                    break;

                default:
                    drDensity = 7800; //Material.Steel
                    break;
            }

            double ldDensity = 0;

            switch (LoadWheelGeometry.Material)
            {
                case Material.Aluminum:
                    ldDensity = 2700;
                    break;

                case Material.Plastic:
                    ldDensity = 1500;
                    break;

                default:
                    ldDensity = 7800; //Material.Steel
                    break;
            }

            //Cylinder volume = PI * radius * radius * cylinderheight

            //Drive Wheel
            double drTotalVolume = Math.PI * Math.Pow(DriveWheelGeometry.OuterDiameter / 2000, 2) * DriveWheelGeometry.CylinderHeight / 1000;
            double drHollowVolume = Math.PI * Math.Pow(DriveWheelGeometry.InnerDiameter / 2000, 2) * DriveWheelGeometry.CylinderHeight / 1000;

            double drRealVolume = drTotalVolume - drHollowVolume;

            DriveWheelGeometry.PartMass = drRealVolume * drDensity;

            DriveWheelGeometry.PartInertia = DriveWheelGeometry.PartMass * (Math.Pow(DriveWheelGeometry.OuterDiameter / 2000, 2) + Math.Pow(DriveWheelGeometry.InnerDiameter / 2000, 2)) * 0.5;

            //Load Wheel
            double ldTotalVolume = Math.PI * Math.Pow(LoadWheelGeometry.OuterDiameter / 2000, 2) * LoadWheelGeometry.CylinderHeight / 1000;
            double ldHollowVolume = Math.PI * Math.Pow(LoadWheelGeometry.InnerDiameter / 2000, 2) * LoadWheelGeometry.CylinderHeight / 1000;

            double ldRealVolume = ldTotalVolume - ldHollowVolume;

            LoadWheelGeometry.PartMass = ldRealVolume * ldDensity;

            LoadWheelGeometry.PartInertia = LoadWheelGeometry.PartMass * (Math.Pow(LoadWheelGeometry.OuterDiameter / 2000, 2) + Math.Pow(LoadWheelGeometry.InnerDiameter / 2000, 2)) * 0.5;

            //Calculate gearing and total inertia
            //Gearing = LoadWheelGeometry.OuterDiameter / DriveWheelGeometry.OuterDiameter;

            Inertia = (DriveWheelGeometry.PartInertia + LoadWheelGeometry.PartInertia) / 10000;

            //Update the property grid
            RaiseRefreshPropertyGridEvent();
        }

        private void RaiseRefreshPropertyGridEvent()
        {
            EventHandler<RefreshPropertyGridEventArgs> handler = RefreshPropertyGridEvent;
            if (handler != null)
                handler(null, new RefreshPropertyGridEventArgs());
        }
    }
}
