﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DockSample
{
    public class TurnTable : GenericLoadComponent, ILoad
    {
        protected new string _brand;
        protected double _movePerRev;
        protected double _angle;

        private Geometry _turnTableGeometry = new Geometry();
        public event EventHandler<RefreshPropertyGridEventArgs> RefreshPropertyGridEvent;

        public TurnTable() { }

        public TurnTable(string name)
        {
            _name = name;
            _mainType = MainType.TurnTable;
            _brand = "Undefined";

            _turnTableGeometry.PropertyChanged += new PropertyChangedEventHandler(_geometry_PropertyChanged);
        }

        void _geometry_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (TurnTableGeometry.ManualInertiaCalc)
                CalcInertia();
        }

        [Category("Component"), ReadOnly(false), Description("Brand name displayed on component.")]
        public new string Brand
        {
            get { return _brand; }
            set
            {
                if (value != this._brand)
                {
                    this._brand = value;
                    //if (!AutoSelect)
                    OnPropertyChanged("Brand");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false),
        DescriptionAttribute("Movement amount in meters (m) per motor revolution.")]
        public double MovePerRev
        {
            get { return _movePerRev; }
            set
            {
                if (value != this._movePerRev)
                {
                    this._movePerRev = value;
                    //if (!AutoSelect)
                    OnPropertyChanged("MovePerRev");
                }
            }
        }

        [Category("Main"), Description("Inclination between 0 and 90 degrees (°). 0 = Horizontal.")]
        public double Angle
        {
            get { return _angle; }
            set
            {
                if (value != this._angle)
                {
                    if (value > 90)
                        value = 90;

                    if (value < 0)
                        value = 0;

                    this._angle = value;

                    //if (!AutoSelect)
                    OnPropertyChanged("Angle");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Mechanical measurements of rotating part.")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Geometry TurnTableGeometry
        {
            get { return _turnTableGeometry; }
            set
            {
                if (value != this._turnTableGeometry)
                {
                    this._turnTableGeometry = value;
                    OnPropertyChanged("TurnTableGeometry");
                }
            }
        }

        private void CalcInertia()
        {
            double density = 0;

            switch (TurnTableGeometry.Material)
            {
                case Material.Aluminum:
                    density = 2700;
                    break;

                case Material.Plastic:
                    density = 1500;
                    break;

                default:
                    density = 7800; //Material.Steel
                    break;
            }

            //Cylinder volume = PI * radius * radius * cylinderheight

            double totalVolume = Math.PI * Math.Pow(TurnTableGeometry.OuterDiameter / 2000, 2) * TurnTableGeometry.CylinderHeight / 1000;
            double hollowVolume = Math.PI * Math.Pow(TurnTableGeometry.InnerDiameter / 2000, 2) * TurnTableGeometry.CylinderHeight / 1000;

            double realVolume = totalVolume - hollowVolume;
            double mass = realVolume * density;

            TurnTableGeometry.PartMass = Math.Round(mass, 3);

            TurnTableGeometry.PartInertia = mass * (Math.Pow(TurnTableGeometry.OuterDiameter / 2000, 2) + Math.Pow(TurnTableGeometry.InnerDiameter / 2000, 2)) * 0.5;

            Inertia = TurnTableGeometry.PartInertia;

            RaiseRefreshPropertyGridEvent();
        }

        private void RaiseRefreshPropertyGridEvent()
        {
            EventHandler<RefreshPropertyGridEventArgs> handler = RefreshPropertyGridEvent;
            if (handler != null)
                handler(null, new RefreshPropertyGridEventArgs());
        }
    }
}

