﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.IO;
using System.Linq;
using System.Data.OleDb;
using DGVPrinterHelper;
using System.Drawing.Printing;

namespace DockSample
{
    public partial class FrmResults : DockContent
    {
        private Moves _moves;
        private SortedList<int, ILoad> _iLoads;
        private Motor _motor;
        private ApplicationData _applicationData;

        private SortedList <int, Motor> _dBMotors;
        private SortedList <int, GearBox> _dBGearBoxes;

        //Flags to indicate what was selected
        private bool selectMotorOnly;
        private bool selectMotorAndGearBox;
        private int minGearing = 0;
        private int maxGearing = 999;

        private int checkedCombinations;

        private int systemGearBoxNr;

        private BackgroundWorker backgroundWorker;
        private DlgSearchProgress dlgSearchProgress;

        public FrmResults(Moves moves, SortedList<int, ILoad> callingLoads, Motor motor, ApplicationData applicationData)
        {
            _moves = moves as Moves;
            _iLoads = callingLoads as SortedList<int, ILoad>;
            _motor = motor as Motor;
            _applicationData = applicationData as ApplicationData;

            _dBMotors = new SortedList<int,Motor>();
            _dBGearBoxes = new SortedList<int,GearBox>();

            checkedCombinations = 0;
            systemGearBoxNr = -1;

            backgroundWorker = new BackgroundWorker();
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.WorkerSupportsCancellation = true;

            backgroundWorker.DoWork +=new DoWorkEventHandler(backgroundWorker_DoWork);
            backgroundWorker.ProgressChanged +=new ProgressChangedEventHandler(backgroundWorker_ProgressChanged);
            backgroundWorker.RunWorkerCompleted +=new RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);

            InitializeComponent();
        }

        #region Events

        private void FrmResults_Activated(object sender, EventArgs e)
        {            
            textBox_Status.Text = "";
            listBox_PreferredGearing.SelectedIndex = 0;
            _moves.AddAllMotionProfiles(_moves.MovesList);

            if (_iLoads.Count > 0)
            {
                UpdateMovesTable();
                _moves.Refresh = true;
            }
            else
            {
                dataGridView_Moves.DataSource = null;
                dataGridView_LoadResultsTable.DataSource = null;
            }
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            minGearing = 0;
            maxGearing = 999;

            if (listBox_PreferredGearing.SelectedIndex != 0)
            {
                minGearing = listBox_PreferredGearing.SelectedIndex * 10 - 9;
                maxGearing = listBox_PreferredGearing.SelectedIndex * 10;
            }

            //Check if there is a system gearbox in the loads list
            systemGearBoxNr = FindSystemGearBoxNr();

            //If nothing is selected do not start search 
            if ((systemGearBoxNr < 0 && _motor.AutoSelect == false) ||
                (_iLoads.Count == 0))
                textBox_Status.Text = "Nothing to do";
            else
            {
                if (backgroundWorker.IsBusy != true)
                {
                    //Use a modal dialog to display the search progress and to lock user operations
                    dlgSearchProgress = new DlgSearchProgress();

                    // event handler for the Cancel button in AlertForm
                    dlgSearchProgress.Cancelled += new EventHandler<EventArgs>(buttonCancel_Click);

                    // Start the asynchronous operation.
                    backgroundWorker.RunWorkerAsync();

                    dlgSearchProgress.StartPosition = FormStartPosition.CenterParent;
                    dlgSearchProgress.ShowDialog();
                }
            }
        }

        // This event handler cancels the backgroundworker, fired from Cancel button in AlertForm.
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            if (backgroundWorker.WorkerSupportsCancellation == true)
            {
                // Cancel the asynchronous operation.
                backgroundWorker.CancelAsync();
                // Close the AlertForm
                dlgSearchProgress.Close();
            }
        }

        // This event handler is where the time consuming work is done.
        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;

            //Search component(s) and report progress
            FindComponents(worker);

            //End search on user request
            if (worker.CancellationPending == true)
            {
                e.Cancel = true;
                return;
            }

            //Delay to allow for the progressbar to update
           // System.Threading.Thread.Sleep(500);
        }

        // This event handler updates the progress.
        private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Pass the progress to AlertForm label and progressbar
            dlgSearchProgress.Message = "% completed of " + (_dBMotors.Count * _dBGearBoxes.Count + _dBMotors.Count).ToString() + " possible combinations";
            dlgSearchProgress.ProgressValue = e.ProgressPercentage;
        }

        // This event handler deals with the results of the background operation.
        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled == true)
            {
                textBox_Status.Text = "Cancelled!";
                ResetSystemMotor();
                ResetSystemGearBox(systemGearBoxNr);
            }
            else if (e.Error != null)
            {
                textBox_Status.Text = "Error: " + e.Error.Message;
            }
            else
            {
                if (selectMotorOnly)
                    textBox_Status.Text = "Motor only selected.";
                else
                    if (selectMotorAndGearBox)
                        textBox_Status.Text = "Motor and gearbox selected.";
                    else
                        textBox_Status.Text = "Could not find motor and/or gearbox.";

                CalcMotorResults(_motor);
                CalculateMovesResults();
                DisplayMovesResults();
                _moves.Refresh = true;
            }
            // Close the Search dialog
            dlgSearchProgress.Close();
        }

        #endregion

        #region Methods

        public void UpdateMovesTable()
        {
            if (dataGridView_Moves.RowCount > 0)
                dataGridView_Moves.Rows.Clear();

            dataGridView_Moves.DataSource = null;

            //Define variable to check if the initial- and finalspeed matches
            double oldFinalSpeed = 0;

            double totalTime = 0;
            double totalDistance = 0;

            //Add data to the Sectors table
            foreach (var m in _moves.MovesList)
            {
                Move move = m.Value;

                double absAverageSpeed = 0;
                double absPeakSpeed = 0;
                double absPeakAcc = 0;

                foreach (Sector s in move.SectorsList)
                {
                    absAverageSpeed += Math.Abs(s.AverageSpeed) * s.Time;

                    if (Math.Abs(s.FinalSpeed) > Math.Abs(absPeakSpeed))
                        absPeakSpeed = Math.Abs(s.FinalSpeed);

                    if (Math.Abs(s.PeakAcc) > Math.Abs(absPeakAcc))
                        absPeakAcc = Math.Abs(s.PeakAcc);
                }

                absAverageSpeed /= move.Time;

                dataGridView_Moves.Rows.Add(move.Name, move.MoveType,
                FormatValue(move.Time),
                FormatValue(totalTime + move.Time),
                FormatValue(move.Distance),
                FormatValue(move.Degrees),
                FormatValue(totalDistance + Math.Abs(move.Distance)),
                FormatValue(move.InitialSpeed),
                FormatValue(move.FinalSpeed),
                FormatValue(absAverageSpeed),
                FormatValue(absPeakSpeed),
                FormatValue(absPeakAcc));

                //Get the handle of the last added row
                DataGridViewRow row = new DataGridViewRow();
                row = dataGridView_Moves.Rows[dataGridView_Moves.RowCount - 1];

                if (move.MoveType == DockSample.MoveType.PreDefined || move.TimeLocked)
                    row.Cells[2].Style.BackColor = Color.BurlyWood;

                if (move.MoveType == DockSample.MoveType.PreDefined || move.DistanceLocked)
                    if (!move.DistanceInDegrees)
                        row.Cells[4].Style.BackColor = Color.BurlyWood;
                    else
                        row.Cells[5].Style.BackColor = Color.BurlyWood;

                if (move.FinalSpeedLocked)
                    row.Cells[8].Style.BackColor = Color.BurlyWood;

                if (move.AverageAccLocked)
                    row.Cells[9].Style.BackColor = Color.BurlyWood;

                //Set a red background if the initial speed of this sector does
                //not match the speed of the previous sector
                if (FormatValue(move.InitialSpeed) != oldFinalSpeed)
                    row.Cells[7].Style.BackColor = Color.LightSalmon;

                oldFinalSpeed = FormatValue(move.FinalSpeed);
                totalTime += move.Time;
                totalDistance += Math.Abs(move.Distance);
            }
        }

        public void FindComponents(BackgroundWorker worker)
        {
            //Flag to follow the search
            checkedCombinations = 0;

            //Begin by clearing the motor and gearbox lists
            _dBMotors.Clear();
            _dBGearBoxes.Clear();

            //Reset component pointers
            int motorOnlyNr = -1;
            int motorWithGearBoxNr = -1;
            string gearBoxArticleNr = null;
            double gearBoxPrice = 0;

            //Search result flags
            bool motorOnlyFound = false;
            bool motorAndGearBoxFound = false;

            //Clear the gearbox checked for auto selection
            if (systemGearBoxNr >= 0)
                ResetSystemGearBox(systemGearBoxNr);

            // - - -  Read motors from database - - -

            //Check if the motor is allowed to be selected by the program
            if (_motor.AutoSelect)
                //Create a list with all motors that fulfills the search criteria(s)
                ReadMotorsToList();
            else
                //Add the defined motor to the list
                _dBMotors.Add(0, _motor);

            //End search on user request
            if (worker.CancellationPending == true)
                return;

            // - - -  Read gearboxes from database - - -

            //Check if the gearbox is allowed to be selected by the program
            //if (systemGearBoxNr >= 0)
                //Create a list with all gearboxes that fulfills the search criteria(s)
                //ReadGearBoxesToList();

            //End search on user request
            if (worker.CancellationPending == true)
                return;
            
            // - - -  Check motor only - - - 

            //Find the smallest motor that can handle the load
            if (Settings.MainSettings.UseGearBox == UseGearBoxCriteria.Auto && _motor.AutoSelect)
                if (FindMotorOnly(worker, ref motorOnlyNr, systemGearBoxNr))
                    motorOnlyFound = true;

            //End search on user request
            if (worker.CancellationPending == true)
                return;

            // - - -  Check gearbox and motor - - -

            //Check if the gearbox is allowed to be selected by the program
            if (systemGearBoxNr >= 0)
                //Find the best combination of motor and gearbox based on the selection criteria
                if (FindMotorAndGearBox(worker, ref motorWithGearBoxNr, ref gearBoxArticleNr, ref gearBoxPrice, systemGearBoxNr))
                    motorAndGearBoxFound = true;

            //End search on user request
            if (worker.CancellationPending == true)
                return;

            //Check the search results and select the cheapest alternative.
            selectMotorOnly = false;
            selectMotorAndGearBox = false;

            if (motorOnlyFound || motorAndGearBoxFound)
                if (motorOnlyFound && motorAndGearBoxFound)
                    if ((_dBMotors[motorOnlyNr].Price + _dBMotors[motorOnlyNr].DrivePrice) < 
                        (_dBMotors[motorWithGearBoxNr].Price + _dBMotors[motorWithGearBoxNr].DrivePrice) + gearBoxPrice) //MotorOnly cheapest
                        selectMotorOnly = true;
                    else
                        selectMotorAndGearBox = true;
                else
                    if (motorOnlyNr >= 0)
                        selectMotorOnly = true;
                    else
                        if (motorWithGearBoxNr >= 0)
                            selectMotorAndGearBox = true;

            if (selectMotorOnly)
            {
                if (_motor.AutoSelect)
                    //Read each property manually from the database to keep the reference to the main _motor-instance
                    ReadMotorFromdB(_motor, _dBMotors[motorOnlyNr].ArticleNo);
                
                if (systemGearBoxNr >= 0)
                    //The system gearbox will not be used. Reset it.
                    ResetSystemGearBox(systemGearBoxNr);
            }
            else
                if (selectMotorAndGearBox)
                {
                    if (_motor.AutoSelect)
                        //Read each property manually from the database to keep the reference to the main _motor-instance
                        ReadMotorFromdB(_motor, _dBMotors[motorWithGearBoxNr].ArticleNo);

                    //Read the parameters of the selected gearbox into the _iLoad-object in the loads list
                    ReadGearBoxFromdB((GearBox)_iLoads[systemGearBoxNr], gearBoxArticleNr);
                }
                else
                {
                    //No component found. Reset all values
                    if (systemGearBoxNr >= 0)
                        ResetSystemGearBox(systemGearBoxNr);

                    if (_motor.AutoSelect)
                        ResetSystemMotor();
                }
        }

        private int FindSystemGearBoxNr()
        {
            foreach (var iLoad in _iLoads)
            {
                //Try to cast the iLoad as a GearBox object
                GearBox gearBox = iLoad.Value as GearBox;
                int loadNr = iLoad.Key;

                //Check if it was a gearBox object
                if (gearBox != null)
                {
                    //Check if the program is allowed to define this gearbox
                    if (gearBox.AutoSelect == true)
                    {
                        return iLoad.Key;
                    }
                }
            }
            return -1;
        }

        private void ResetSystemGearBox(int loadNr)
        {
            //Try to cast the iLoad as a GearBox object
            GearBox gearBox = _iLoads[loadNr] as GearBox;
            {
                #region ClearGearBoxData

                //Component data
                gearBox.ArticleNo = "";

                gearBox.Brand = GearBoxBrand.Undefined;
                gearBox.SubType = GearBoxType.Undefined;
                gearBox.Description = "Not selected";
                gearBox.Price = 0;

                //Main data
                gearBox.Gearing = 1;
                gearBox.IdleTrq = 0;
                gearBox.Inertia = 0;
                gearBox.Eff = 100;
                gearBox.Backlash = 0;

                //Mechanical input
                gearBox.InpType = MechInterface.Undefined;
                gearBox.InpMinDia = 0;
                gearBox.InpMaxDia = 0;
                gearBox.InpRadFrc = 0;
                gearBox.InpAxFrc = 0;

                //Mechanical output
                gearBox.OutType = MechInterface.Undefined;
                gearBox.OutMinDia = 0;
                gearBox.OutMaxDia = 0;
                gearBox.OutRadFrc = 0;
                gearBox.OutAxFrc = 0;

                //Torque data
                gearBox.MaxTrq = 0;
                gearBox.MaxTrqDur = 0;
                gearBox.TrqED = 0;
                gearBox.NomTrq = 0;

                //Speed data
                gearBox.MaxSpd = 0;
                gearBox.MaxSpdDur = 0;
                gearBox.SpdED = 0;
                gearBox.NomSpd = 0;

                #endregion
            }
        }

        private void ResetSystemMotor()
        {
            //Clear all motor properties
            #region ClearMotorData
            _motor.ArticleNo = "";

            _motor.Brand = Brand.BSD;

            _motor.Description = "Not selected";
            // ToDo: motor. = dBMotor.Type;
            _motor.Price = 0;

            //Main data
            _motor.Resolution = 0;
            _motor.InertiaRatio = 0;
            _motor.Inertia = 0;
            _motor.Eff = 100;
            _motor.FldWeakSpd = 0;

            //Mechanical output
            _motor.OutType = MechInterface.Undefined;

            _motor.OutMinDia = 0;
            _motor.OutMaxDia = 0;
            _motor.OutRadFrc = 0;
            _motor.OutAxFrc = 0;

            //Torque data
            _motor.MaxTrq = 0;
            _motor.MaxTrqDur = 0;
            _motor.TrqED = 0;
            _motor.NomTrq = 0;

            //Speed data
            _motor.MaxSpd = 0;
            _motor.MaxSpdDur = 0;
            _motor.SpdED = 0;
            _motor.NomSpd = 0;
            #endregion
        }

        private void ReadMotorsToList()
        {
            //Read all motors that fulfills the search criteria(s) into a list

            //Go through the loads to check if a brake is needed or not.
            bool useBrake = false;

            if (Settings.MainSettings.UseBrakedMotor == Criteria.Auto)
            {
                foreach (var m in _moves.MovesList)
                {
                    Move move = m.Value;

                    move.CalcLoadResults(_iLoads);

                    foreach (var lR in m.Value.LoadResultsList)

                        if (lR.Value.Active)

                            switch (lR.Value.MainType)
                            {
                                case MainType.Force:
                                    var force = _iLoads[lR.Key] as LoadForce;

                                    if (force.Force != 0)
                                        useBrake = true;

                                    break;

                                case MainType.Torque:
                                    var torque = _iLoads[lR.Key] as LoadTorque;

                                    if (torque.Torque != 0)
                                        useBrake = true;

                                    break;

                                case MainType.BallScrew:
                                    var ballScrew = _iLoads[lR.Key] as BallScrew;

                                    if (ballScrew.Angle != 0)
                                        useBrake = true;

                                    break;

                                case MainType.RackAndPinion:
                                    var rackAndPinion = _iLoads[lR.Key] as RackAndPinion;

                                    if (rackAndPinion.Angle != 0)
                                        useBrake = true;

                                    break;

                                case MainType.TurnTable:
                                    var turnTable = _iLoads[lR.Key] as TurnTable;

                                    if (turnTable.Angle != 0)
                                        useBrake = true;

                                    break;

                                case MainType.ToothedBelt:
                                    var toothedBelt = _iLoads[lR.Key] as ToothedBelt;

                                    if (toothedBelt.Angle != 0)
                                        useBrake = true;

                                    break;
                            }

                    if (useBrake)
                        break;
                }
            }
            else
                if (Settings.MainSettings.UseBrakedMotor == Criteria.Always)
                    useBrake = true;

            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read the data from the Motors table
                    var qryMotors =
                        from tblMotors myMotors in dbDriveSystemsContext.tblMotors
                        orderby myMotors.Price, myMotors.NomTrq
                        select myMotors;

                    foreach (var dBMotor in qryMotors)
                    {
                        Motor motor = new Motor();

                        //Check if the motor fulfills the search criteria(s)
                        if ((dBMotor.Braked == "Yes" && useBrake == true ||
                            dBMotor.Braked == "No" && useBrake == false) &&
                            (Settings.MainSettings.MotorFamilyFilter == MotorFamily.Undefined ||
                            Settings.MainSettings.MotorFamilyFilter.ToString() == dBMotor.Family) &&
                            ((Settings.MainSettings.ConditionFilter == Condition.OnlyNew && !dBMotor.Campaign) ||
                            (Settings.MainSettings.ConditionFilter == Condition.NewAndCampaign && !dBMotor.Sold) ||
                            (Settings.MainSettings.ConditionFilter == Condition.OnlyCampaign && dBMotor.Campaign && !dBMotor.Sold)))
                        {

                            #region ReadMotorData

                            motor.ArticleNo = dBMotor.ArticleNo;

                            motor.Campaign = dBMotor.Campaign;
                            motor.Sold = dBMotor.Sold;

                            motor.Brand = Brand.BSD; 

                            motor.Description = dBMotor.Description;

                            switch (dBMotor.SubType)
                            {
                                case ("ServoMotor"):
                                    motor.SubType = MotorType.ServoMotor;
                                    break;

                                default:
                                    motor.SubType = MotorType.Undefined;
                                    break;
                            }
                            
                            //Query all drives in the database that can be used with this motor
                            var qryDrivesPerMotor =
                                        from tblDrives myDrives in dbDriveSystemsContext.tblDrives
                                        join tblDriveAndMotorCombinations myDriveAndMotorCombinations
                                            in dbDriveSystemsContext.tblDriveAndMotorCombinations
                                        on myDrives.ArticleNo equals myDriveAndMotorCombinations.DriveArticleNo
                                        where myDriveAndMotorCombinations.MotorArticleNo == motor.ArticleNo
                                        orderby myDrives.ArticleNo
                                        select myDrives;

                            //Check the price of the cheapest usable drive
                            motor.DrivePrice = 999999;

                            foreach (var drive in qryDrivesPerMotor)
                                //Check if the drive fulfills the search criteria(s)
                                if (((Settings.MainSettings.ConditionFilter == Condition.OnlyNew && !drive.Campaign) ||
                                    (Settings.MainSettings.ConditionFilter == Condition.NewAndCampaign && !drive.Sold) ||
                                    (Settings.MainSettings.ConditionFilter == Condition.OnlyCampaign && !drive.Sold)))
                                {
                                    if (drive.Price < motor.DrivePrice)
                                        motor.DrivePrice = drive.Price;
                                }
                            
                            motor.Price = dBMotor.Price;

                            switch (dBMotor.Braked)
                            {
                                case ("Yes"):
                                    motor.Braked = true;
                                    break;
                            }

                            //Main data
                            motor.Resolution = dBMotor.Resolution;
                            motor.InertiaRatio = dBMotor.InertiaRatio;
                            motor.Inertia = dBMotor.Inertia / 10000;
                            motor.Eff = dBMotor.Eff;
                            motor.FldWeakSpd = dBMotor.FldWeakSpd;
                            motor.InvEff = dBMotor.InverseEfficiency;

                            //Mechanical output
                            switch (dBMotor.OutType)
                            {
                                case "Flange":
                                    motor.OutType = MechInterface.Flange;
                                    break;

                                case "SmoothHollow":
                                    motor.OutType = MechInterface.SmoothHollow;
                                    break;

                                case "KeyedHollow":
                                    motor.OutType = MechInterface.KeyedHollow;
                                    break;

                                case "KeyedShaft":
                                    motor.OutType = MechInterface.KeyedShaft;
                                    break;

                                case "SmoothShaft":
                                    motor.OutType = MechInterface.SmoothShaft;
                                    break;
                            }

                            motor.OutMinDia = dBMotor.OutMinDia;
                            motor.OutMaxDia = dBMotor.OutMaxDia;
                            motor.OutRadFrc = dBMotor.OutRadFrc;
                            motor.OutAxFrc = dBMotor.OutAxFrc;

                            //Torque data
                            motor.MaxTrq = dBMotor.MaxTrq;
                            motor.MaxTrqDur = dBMotor.MaxTrqDur;
                            motor.TrqED = dBMotor.TrqED;
                            motor.NomTrq = dBMotor.NomTrq;

                            //Speed data
                            motor.MaxSpd = dBMotor.MaxSpd;
                            motor.MaxSpdDur = dBMotor.MaxSpdDur;
                            motor.SpdED = dBMotor.SpdED;
                            motor.NomSpd = dBMotor.NomSpd;
                            #endregion

                            //If true then add the motor to the list                            
                            _dBMotors.Add(_dBMotors.Count, motor);
                        }
                    }
                }
            }
        }

        
        private void ReadGearBoxesToList(Motor motor)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            _dBGearBoxes.Clear();

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read data from the GearBox table
              //      var qryGearBoxes =
              //          from tblGearBoxes myGearBoxes in dbDriveSystemsContext.tblGearBoxes
              //          orderby myGearBoxes.Price, myGearBoxes.Gearing, myGearBoxes.InpMaxDia
              //          select myGearBoxes; 

                                                    //Query all gearboxes in the database that can be used with this motor
                                var qryGearBoxes =
                                    from tblGearBoxes myGearBoxes in dbDriveSystemsContext.tblGearBoxes
                                    join tblMotorAndGearBoxCombinations myMotorAndGearBoxCombinations
                                        in dbDriveSystemsContext.tblMotorAndGearBoxCombinations
                                    on myGearBoxes.ArticleNo equals myMotorAndGearBoxCombinations.GearBoxArticleNo
                                    where myMotorAndGearBoxCombinations.MotorArticleNo == motor.ArticleNo
                                    & myGearBoxes.Gearing >= minGearing
                                    & myGearBoxes.Gearing <= maxGearing
                                    orderby myGearBoxes.Price, myGearBoxes.Gearing, myGearBoxes.InpMaxDia
                                    select myGearBoxes;

                    //Iterate through the gearboxes to find those that fits the selected motor
                    foreach (var dBGearBox in qryGearBoxes)
                    {
                        //Check that the gearbox fulfills the search criteria(s)
                        if ((Settings.MainSettings.GearBoxBrandFilter == GearBoxBrand.Undefined ||
                            Settings.MainSettings.GearBoxBrandFilter.ToString() == dBGearBox.Brand) &&
                             (Settings.MainSettings.GearBoxTypeFilter == GearBoxType.Undefined ||
                               Settings.MainSettings.GearBoxTypeFilter.ToString() == dBGearBox.SubType) &&
                             (Settings.MainSettings.GearBoxFamilyFilter == "Undefined" || //GearBoxFamily.Undefined ||
                               Settings.MainSettings.GearBoxFamilyFilter == dBGearBox.Family) &&
                            ((Settings.MainSettings.ConditionFilter == Condition.OnlyNew && !dBGearBox.Campaign) ||
                            (Settings.MainSettings.ConditionFilter == Condition.NewAndCampaign && !dBGearBox.Sold) ||
                            (Settings.MainSettings.ConditionFilter == Condition.OnlyCampaign && dBGearBox.Campaign && !dBGearBox.Sold)))
                        {
                            GearBox gearBox = new GearBox();

                            #region ReadGearBoxData

                            gearBox.ArticleNo = dBGearBox.ArticleNo;

                            gearBox.Campaign = dBGearBox.Campaign;
                            gearBox.Sold = dBGearBox.Sold;

                            switch (dBGearBox.Brand)
                            {
                                case ("LSIS"):
                                    gearBox.Brand = GearBoxBrand.LSIS;
                                    break;

                                case ("Wittenstein"):
                                    gearBox.Brand = GearBoxBrand.Wittenstein;
                                    break;

                                default:
                                    gearBox.Brand = GearBoxBrand.Undefined;
                                    break;
                            }

                            switch (dBGearBox.SubType)
                            {
                                case ("PlanetaryStraight"):
                                    gearBox.SubType = GearBoxType.PlanetaryStraight;
                                    break;

                                case ("PlanetaryRightAngle"):
                                    gearBox.SubType = GearBoxType.PlanetaryRightAngle;
                                    break;

                                case ("WormGear"):
                                    gearBox.SubType = GearBoxType.WormGear;
                                    break;

                                default:
                                    gearBox.SubType = GearBoxType.Undefined;
                                    break;
                            }

                            gearBox.Description = dBGearBox.Description;
                            gearBox.Price = dBGearBox.Price;

                            //Main data
                            gearBox.Gearing = dBGearBox.Gearing;
                            gearBox.IdleTrq = dBGearBox.IdleTrq;
                            gearBox.Inertia = dBGearBox.Inertia / 10000;
                            gearBox.Eff = dBGearBox.Eff;
                            gearBox.Backlash = dBGearBox.Backlash;

                            //Mechanical input
                            switch (dBGearBox.InpType)
                            {
                                case "Flange":
                                    gearBox.InpType = MechInterface.Flange;
                                    break;

                                case "SmoothHollow":
                                    gearBox.InpType = MechInterface.SmoothHollow;
                                    break;

                                case "KeyedHollow":
                                    gearBox.InpType = MechInterface.KeyedHollow;
                                    break;

                                case "KeyedShaft":
                                    gearBox.InpType = MechInterface.KeyedShaft;
                                    break;

                                case "SmoothShaft":
                                    gearBox.InpType = MechInterface.SmoothShaft;
                                    break;

                                default:
                                    gearBox.InpType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.InpMinDia = dBGearBox.InpMinDia;
                            gearBox.InpMaxDia = dBGearBox.InpMaxDia;
                            gearBox.InpRadFrc = dBGearBox.InpRadFrc;
                            gearBox.InpAxFrc = dBGearBox.InpAxFrc;

                            //Mechanical output
                            switch (dBGearBox.OutType)
                            {
                                case "Flange":
                                    gearBox.OutType = MechInterface.Flange;
                                    break;

                                case "SmoothHollow":
                                    gearBox.OutType = MechInterface.SmoothHollow;
                                    break;

                                case "KeyedHollow":
                                    gearBox.OutType = MechInterface.KeyedHollow;
                                    break;

                                case "KeyedShaft":
                                    gearBox.OutType = MechInterface.KeyedShaft;
                                    break;

                                case "SmoothShaft":
                                    gearBox.OutType = MechInterface.SmoothShaft;
                                    break;

                                default:
                                    gearBox.OutType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.OutMinDia = dBGearBox.OutMinDia;
                            gearBox.OutMaxDia = dBGearBox.OutMaxDia;
                            gearBox.OutRadFrc = dBGearBox.OutRadFrc;
                            gearBox.OutAxFrc = dBGearBox.OutAxFrc;

                            //Torque data
                            gearBox.MaxTrq = dBGearBox.MaxTrq;
                            gearBox.MaxTrqDur = dBGearBox.MaxTrqDur;
                            gearBox.TrqED = dBGearBox.TrqED;
                            gearBox.NomTrq = dBGearBox.NomTrq;

                            //Speed data
                            gearBox.MaxSpd = dBGearBox.MaxSpd;
                            gearBox.MaxSpdDur = dBGearBox.MaxSpdDur;
                            gearBox.SpdED = dBGearBox.SpdED;
                            gearBox.NomSpd = dBGearBox.NomSpd;

                            #endregion

                            //If true then add the gearbox to the list
                            _dBGearBoxes.Add(_dBGearBoxes.Count(), gearBox);
                        }
                    }
                }
            }
        }


        private void ReadGearBoxesToList()
        {
            //Denna funktion används ej!!!!!!!!!!!!!!!!!!!!!

            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            _dBGearBoxes.Clear();

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read data from the GearBox table
                    var qryGearBoxes =
                        from tblGearBoxes myGearBoxes in dbDriveSystemsContext.tblGearBoxes
                        orderby myGearBoxes.Price, myGearBoxes.Gearing, myGearBoxes.InpMaxDia
                        select myGearBoxes;

                  
                    //Iterate through the gearboxes to find those that fits the selected motor
                    foreach (var dBGearBox in qryGearBoxes)
                    {
                        //Check that the gearbox fulfills the search criteria(s)
                        if ((Settings.MainSettings.GearBoxBrandFilter == GearBoxBrand.Undefined ||
                            Settings.MainSettings.GearBoxBrandFilter.ToString() == dBGearBox.Brand) &&
                             (Settings.MainSettings.GearBoxTypeFilter == GearBoxType.Undefined ||
                               Settings.MainSettings.GearBoxTypeFilter.ToString() == dBGearBox.SubType) &&
                             (Settings.MainSettings.GearBoxFamilyFilter == "Undefined" ||
                               Settings.MainSettings.GearBoxFamilyFilter.ToString() == dBGearBox.Family) &&
                            ((Settings.MainSettings.ConditionFilter == Condition.OnlyNew && !dBGearBox.Campaign) ||
                            (Settings.MainSettings.ConditionFilter == Condition.NewAndCampaign && !dBGearBox.Sold) ||
                            (Settings.MainSettings.ConditionFilter == Condition.OnlyCampaign && dBGearBox.Campaign && !dBGearBox.Sold)))
                        {
                            GearBox gearBox = new GearBox();

                            #region ReadGearBoxData

                            gearBox.ArticleNo = dBGearBox.ArticleNo;

                            gearBox.Campaign = dBGearBox.Campaign;
                            gearBox.Sold = dBGearBox.Sold;

                            switch (dBGearBox.Brand)
                            {
                                case ("LSIS"):
                                    gearBox.Brand = GearBoxBrand.LSIS;
                                    break;
                                
                                case ("Wittenstein"):
                                    gearBox.Brand = GearBoxBrand.Wittenstein;
                                    break;

                                default:
                                    gearBox.Brand = GearBoxBrand.Undefined;
                                    break;
                            }

                            switch (dBGearBox.SubType)
                            {
                                case ("PlanetaryStraight"):
                                    gearBox.SubType = GearBoxType.PlanetaryStraight;
                                    break;

                                case ("PlanetaryRightAngle"):
                                    gearBox.SubType = GearBoxType.PlanetaryRightAngle;
                                    break;

                                case ("WormGear"):
                                    gearBox.SubType = GearBoxType.WormGear;
                                    break;

                                default:
                                    gearBox.SubType = GearBoxType.Undefined;
                                    break;
                            }

                            gearBox.Description = dBGearBox.Description;
                            gearBox.Price = dBGearBox.Price;

                            //Main data
                            gearBox.Gearing = dBGearBox.Gearing;
                            gearBox.IdleTrq = dBGearBox.IdleTrq;
                            gearBox.Inertia = dBGearBox.Inertia / 10000;
                            gearBox.Eff = dBGearBox.Eff;
                            gearBox.Backlash = dBGearBox.Backlash;

                            //Mechanical input
                            switch (dBGearBox.InpType)
                            {
                                case "Flange":
                                    gearBox.InpType = MechInterface.Flange;
                                    break;

                                case "SmoothHollow":
                                    gearBox.InpType = MechInterface.SmoothHollow;
                                    break;

                                case "KeyedHollow":
                                    gearBox.InpType = MechInterface.KeyedHollow;
                                    break;

                                case "KeyedShaft":
                                    gearBox.InpType = MechInterface.KeyedShaft;
                                    break;

                                case "SmoothShaft":
                                    gearBox.InpType = MechInterface.SmoothShaft;
                                    break;

                                default:
                                    gearBox.InpType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.InpMinDia = dBGearBox.InpMinDia;
                            gearBox.InpMaxDia = dBGearBox.InpMaxDia;
                            gearBox.InpRadFrc = dBGearBox.InpRadFrc;
                            gearBox.InpAxFrc = dBGearBox.InpAxFrc;

                            //Mechanical output
                            switch (dBGearBox.OutType)
                            {
                                case "Flange":
                                    gearBox.OutType = MechInterface.Flange;
                                    break;

                                case "SmoothHollow":
                                    gearBox.OutType = MechInterface.SmoothHollow;
                                    break;

                                case "KeyedHollow":
                                    gearBox.OutType = MechInterface.KeyedHollow;
                                    break;

                                case "KeyedShaft":
                                    gearBox.OutType = MechInterface.KeyedShaft;
                                    break;

                                case "SmoothShaft":
                                    gearBox.OutType = MechInterface.SmoothShaft;
                                    break;

                                default:
                                    gearBox.OutType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.OutMinDia = dBGearBox.OutMinDia;
                            gearBox.OutMaxDia = dBGearBox.OutMaxDia;
                            gearBox.OutRadFrc = dBGearBox.OutRadFrc;
                            gearBox.OutAxFrc = dBGearBox.OutAxFrc;

                            //Torque data
                            gearBox.MaxTrq = dBGearBox.MaxTrq;
                            gearBox.MaxTrqDur = dBGearBox.MaxTrqDur;
                            gearBox.TrqED = dBGearBox.TrqED;
                            gearBox.NomTrq = dBGearBox.NomTrq;

                            //Speed data
                            gearBox.MaxSpd = dBGearBox.MaxSpd;
                            gearBox.MaxSpdDur = dBGearBox.MaxSpdDur;
                            gearBox.SpdED = dBGearBox.SpdED;
                            gearBox.NomSpd = dBGearBox.NomSpd;

                            #endregion

                            //If true then add the gearbox to the list
                            _dBGearBoxes.Add(_dBGearBoxes.Count(), gearBox);
                        }
                    }
                }
            }
        }

        private bool FindMotorOnly(BackgroundWorker worker, ref int MotorOnlyNr, int systemGearBoxNr)
        {
            double inertia = 0;
            double peakTorque = 0;
            double peakSpeed = 0;
            double liftTorque = 0;

            //Recalculate the loads for each move with the system gearbox values set to zero
            foreach (var m in _moves.MovesList)
            {
                Move move = m.Value;
                move.CalcLoadResults(_iLoads);

                if (move.TotalLoadResults.Inertia > inertia)
                    inertia = move.TotalLoadResults.Inertia;

                if (move.TotalLoadResults.PeakTrq > peakTorque)
                    peakTorque = move.TotalLoadResults.PeakTrq;

                if (move.TotalLoadResults.PeakSpd > peakSpeed)
                    peakSpeed = move.TotalLoadResults.PeakSpd;

                if (move.TotalLoadResults.LiftTorque > liftTorque)
                    liftTorque = move.TotalLoadResults.LiftTorque;
            }

            //Check the mechanical interface the motor should connect to
            MechInterface inpType = MechInterface.Undefined;
            double inpMinDia = 0;
            double inpMaxDia = 0;

            bool interfaceCheckDone = false;

            if (Settings.MainSettings.MechChkOnNonAutoSelCmp == true)
                foreach (var iL in _iLoads)
                {
                    ILoad iLoad = iL.Value;

                    //The check has already been done, interface defined
                    if (!interfaceCheckDone)
                        //Only attach to mechanical components
                        if (iLoad.Detachable == false)
                            //A systemGearBox will not be used in a MotorOnly solution 
                            if (iL.Key != systemGearBoxNr)
                                switch (iLoad.MainType)
                                {
                                    case MainType.GearBox:
                                        var gearBox = iLoad as GearBox;

                                        inpType = gearBox.InpType;
                                        inpMinDia = gearBox.InpMinDia;
                                        inpMaxDia = gearBox.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;

                                    case MainType.Coupling:
                                        var coupling = iLoad as Coupling;

                                        inpType = coupling.InpType;
                                        inpMinDia = coupling.InpMinDia;
                                        inpMaxDia = coupling.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;

                                    case MainType.BallScrew:
                                        var ballScrew = iLoad as BallScrew;

                                        inpType = ballScrew.InpType;
                                        inpMinDia = ballScrew.InpMinDia;
                                        inpMaxDia = ballScrew.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;

                                    case MainType.RackAndPinion:
                                        var rackAndPinion = iLoad as RackAndPinion;

                                        inpType = rackAndPinion.InpType;
                                        inpMinDia = rackAndPinion.InpMinDia;
                                        inpMaxDia = rackAndPinion.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;

                                    case MainType.TurnTable:
                                        var turnTable = iLoad as TurnTable;

                                        inpType = turnTable.InpType;
                                        inpMinDia = turnTable.InpMinDia;
                                        inpMaxDia = turnTable.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;

                                    case MainType.ToothedBelt:
                                        var toothedBelt = iLoad as ToothedBelt;

                                        inpType = toothedBelt.InpType;
                                        inpMinDia = toothedBelt.InpMinDia;
                                        inpMaxDia = toothedBelt.InpMaxDia;
                                        interfaceCheckDone = true;

                                        break;
                                }

                    if (interfaceCheckDone)
                        break;
                }

            foreach (var dBMotor in _dBMotors)
            {
                //End search on user request
                if (worker.CancellationPending == true)
                    break;

                //Report how far the check has come
                checkedCombinations++;

                double progress = ((double)checkedCombinations / (double)(_dBGearBoxes.Count * _dBMotors.Count + _dBMotors.Count)) * 100.0;

                if (_dBGearBoxes.Count == 0)
                    progress = ((double)checkedCombinations / (double)_dBMotors.Count) * 100.0;

                //Initialize the motor object to be checked
                Motor motor = dBMotor.Value;
                int index = dBMotor.Key;

                //Check the inertia ratio between the motor and the defined load
                double inertiaRatio = inertia / motor.Inertia;

                //If the mechanical test is active then check that the motor fits to the attached component
                if (Settings.MainSettings.MechChkOnNonAutoSelCmp == false ||
                    (motor.OutType == MechInterface.SmoothShaft && inpType == MechInterface.SmoothHollow ||
                    motor.OutType == MechInterface.SmoothShaft && inpType == MechInterface.KeyedHollow ||
                    motor.OutType == MechInterface.KeyedShaft && inpType == MechInterface.SmoothHollow ||
                    motor.OutType == MechInterface.KeyedShaft && inpType == MechInterface.KeyedHollow ||
                    motor.OutType == MechInterface.Flange && inpType == MechInterface.Flange) &&
                    (motor.OutMaxDia <= inpMaxDia && motor.OutMinDia >= inpMinDia))

                    //Check that the motor can handle the inertia, peak speed and peak torque. 
                    if (motor.InertiaRatio > inertiaRatio &&
                        (motor.NomTrq * (Settings.MainSettings.AllowedLiftTorque / 100.0)) > liftTorque * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) &&
                        motor.MaxTrq > peakTorque * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) &&
                        motor.MaxSpd > peakSpeed * (1.0 + Settings.MainSettings.SpeedMargin / 100.0))
                        if (CalcMotorResults(motor))
                        {
                            //Success
                            MotorOnlyNr = index;
                            return true;
                        }
            }

            //No motor found
            return false;
        }

        private bool FindMotorAndGearBox(BackgroundWorker worker, ref int MotorNr, ref string GearBoxArticleNr, ref double GearBoxPrice, int systemGearBoxNr)
        {
            //Check the mechanical interface for the gearbox output side
            #region GearBoxInterfaceCheckOutputSide

            MechInterface mechTypeOnGbOutput = MechInterface.Undefined;
            double minDiaOnGbOutput = 0;
            double maxDiaOnGbOutput = 0;
            bool autoSelectOnOutput = false;
            double totalComponentPrice = 0;

            bool mechCheckDoneOnOutput = false;

            //Should this be checked at all?
            if (Settings.MainSettings.MechChkOnNonAutoSelCmp)
                foreach (var iL in _iLoads)
                {
                    //Only check the mechanical interface of components after this load
                    if (iL.Key > systemGearBoxNr)
                    {
                        ILoad iLoad = iL.Value;

                        if (iLoad.Detachable == false)
                            switch (iLoad.MainType)
                            {
                                case MainType.GearBox:

                                    if (iLoad.AutoSelect)
                                        autoSelectOnOutput = true;

                                    var gearBox = iLoad as GearBox;

                                    mechTypeOnGbOutput = gearBox.InpType;
                                    minDiaOnGbOutput = gearBox.InpMinDia;
                                    maxDiaOnGbOutput = gearBox.InpMaxDia;

                                    mechCheckDoneOnOutput = true;

                                    break;

                                case MainType.Coupling:

                                    if (iLoad.AutoSelect)
                                        autoSelectOnOutput = true;

                                    var coupling = iLoad as Coupling;

                                    mechTypeOnGbOutput = coupling.InpType;
                                    minDiaOnGbOutput = coupling.InpMinDia;
                                    maxDiaOnGbOutput = coupling.InpMaxDia;
                                    mechCheckDoneOnOutput = true;

                                    break;

                                case MainType.BallScrew:

                                    if (iLoad.AutoSelect)
                                        autoSelectOnOutput = true;

                                    var ballScrew = iLoad as BallScrew;

                                    mechTypeOnGbOutput = ballScrew.InpType;
                                    minDiaOnGbOutput = ballScrew.InpMinDia;
                                    maxDiaOnGbOutput = ballScrew.InpMaxDia;
                                    mechCheckDoneOnOutput = true;

                                    break;
                            }
                    }

                    if (mechCheckDoneOnOutput)
                        break;
                }

            #endregion

            //Go through all listed motors
            foreach (var dBMotor in _dBMotors)
            {
                //End search on user request
                if (worker.CancellationPending == true)
                    break;

                //Initialize the motor to be checked
                Motor motor = dBMotor.Value;
                int motorIndex = dBMotor.Key;

                double gearingLimit = 0;

                //Check the mechanical interface for the gearbox input side
                #region GearBoxInterfaceCheckInputSide

                //GearBox input side
                MechInterface mechTypeOnGbInput = MechInterface.Undefined;
                double minDiaOnGbInput = 0;
                double maxDiaOnGbInput = 0;
                bool autoSelectOnInput = false;

                bool mechCheckDoneOnInput = false;

                //If the gearbox is not the first load then check the mechanical interface of the previous load
                if (systemGearBoxNr > 0)
                {
                    foreach (var iL in _iLoads)
                    {
                        if (iL.Key < systemGearBoxNr)
                        {
                            ILoad iLoad = iL.Value;

                            if (iLoad.Detachable == false)
                                switch (iLoad.MainType)
                                {
                                    case MainType.GearBox:

                                        if (iLoad.AutoSelect)
                                            autoSelectOnInput = true;

                                        var gearBox = iLoad as GearBox;

                                        mechTypeOnGbInput = gearBox.OutType;
                                        minDiaOnGbInput = gearBox.OutMinDia;
                                        maxDiaOnGbInput = gearBox.OutMaxDia;
                                        mechCheckDoneOnInput = true;

                                        break;

                                    case MainType.Coupling:

                                        if (iLoad.AutoSelect)
                                            autoSelectOnInput = true;

                                        var coupling = iLoad as Coupling;

                                        mechTypeOnGbInput = coupling.OutType;
                                        minDiaOnGbInput = coupling.OutMinDia;
                                        maxDiaOnGbInput = coupling.OutMaxDia;
                                        mechCheckDoneOnInput = true;

                                        break;
                                }
                        }
                    }
                }

                if (!mechCheckDoneOnInput)
                {
                    if (systemGearBoxNr == 0)
                    {
                        mechTypeOnGbInput = motor.OutType;
                        minDiaOnGbInput = motor.OutMinDia;
                        maxDiaOnGbInput = motor.OutMaxDia;
                        autoSelectOnInput = true;
                    }
                }

                #endregion

                //Go through all listed gearboxes

                ReadGearBoxesToList(motor);

                if (_dBGearBoxes.Count > 0)
                {
                    foreach (var dBGearBox in _dBGearBoxes)
                    {
                        //End search on user request
                        if (worker.CancellationPending == true)
                            break;



                        //Initialize the gearbox to be checked
                        GearBox gearBox = dBGearBox.Value;
                        int gearBoxIndex = dBGearBox.Key;

                        //Report how far the check has come
                        checkedCombinations++;
                        //double progress = ((double)checkedCombinations / ((double)(_dBMotors.Count * _dBGearBoxes.Count + _dBMotors.Count))) * 100.0;
                        double progress = (double)checkedCombinations;                        
                        if (progress < 100)
                            worker.ReportProgress((int)progress);
                        else
                        {
                        worker.ReportProgress(99);
                            checkedCombinations = 0;
                        }

                        //If a combination has already been selected then check if the current combination is cheaper
                        double totalComponentPriceNextCombination = motor.Price + motor.DrivePrice + gearBox.Price;

                        if ((MotorNr == -1 && GearBoxArticleNr == null) || (totalComponentPriceNextCombination < totalComponentPrice))
                        {
                            //Check that gear ratio does not result in overspeed
                            if (gearBox.Gearing < gearingLimit || gearingLimit == 0)
                            {
                                //If the mechanical test is active then check that the gearBox fits to the attached component

                                //Input side

                                if ((Settings.MainSettings.MechChkOnNonAutoSelCmp == false && !autoSelectOnInput) ||
                                    (((mechTypeOnGbInput == MechInterface.SmoothShaft && gearBox.InpType == MechInterface.SmoothHollow) ||
                                      (mechTypeOnGbInput == MechInterface.SmoothShaft && gearBox.InpType == MechInterface.KeyedHollow) ||
                                      (mechTypeOnGbInput == MechInterface.KeyedShaft && gearBox.InpType == MechInterface.SmoothHollow) ||
                                      (mechTypeOnGbInput == MechInterface.KeyedShaft && gearBox.InpType == MechInterface.KeyedHollow) ||
                                      (mechTypeOnGbInput == MechInterface.Flange && gearBox.InpType == MechInterface.Flange)) &&
                                        (maxDiaOnGbInput <= gearBox.InpMaxDia && minDiaOnGbInput >= gearBox.InpMinDia)))

                                    //Output side

                                    if ((Settings.MainSettings.MechChkOnNonAutoSelCmp == false && !autoSelectOnOutput) ||
                                        (((mechTypeOnGbOutput == MechInterface.SmoothHollow && gearBox.OutType == MechInterface.SmoothShaft) ||
                                        (mechTypeOnGbOutput == MechInterface.KeyedHollow && gearBox.OutType == MechInterface.SmoothShaft) ||
                                        (mechTypeOnGbOutput == MechInterface.SmoothHollow && gearBox.OutType == MechInterface.KeyedShaft) ||
                                        (mechTypeOnGbOutput == MechInterface.KeyedHollow && gearBox.OutType == MechInterface.KeyedShaft) ||
                                        (mechTypeOnGbOutput == MechInterface.Flange && gearBox.OutType == MechInterface.Flange)) &&
                                        (maxDiaOnGbOutput >= gearBox.OutMaxDia && minDiaOnGbOutput <= gearBox.OutMinDia)))
                                    {
                                        //Read the parameters of the selected gearbox into the _iLoad-object in the loads list
                                        ReadGearBoxFromList((GearBox)_iLoads[systemGearBoxNr], gearBoxIndex);

                                        bool gearBoxTestOK = false;
                                        double totalTime = 0;
                                        double rmsTrqSum = 0;
                                        double rmsSpdSum = 0;
                                        double gearBoxRmsTrq = 0;
                                        double gearBoxRmsSpd = 0;

                                        //Check if the selected gearBox can handle the load
                                        foreach (var m in _moves.MovesList)
                                        {
                                            //All moves must pass the test. If this is not the first move then check that the previous move passed the test
                                            if (m.Key == 0 || gearBoxTestOK)
                                            {
                                                //Reset the flag before the next test
                                                gearBoxTestOK = false;

                                                //This calculation is cheating since it does not return the true rms values. However the duty cycles for the
                                                //mechanical components are often very long (> 600s) which means that a accurate calculation is normally not needed.
                                                Move move = m.Value;

                                                move.CalcLoadResults(_iLoads);

                                                motor.Description = motor.Description;
                                                gearBox.Description = gearBox.Description;

                                                if (gearBox.ArticleNo == "100-6087")
                                                    gearBox.ArticleNo = gearBox.ArticleNo;


                                                //Check if the gearbox and motor passes the test based on the nominal and maximum speed
                                                if ((move.LoadResultsList[systemGearBoxNr].PeakSpd * (1.0 + Settings.MainSettings.SpeedMargin / 100.0) < gearBox.MaxSpd) &&
                                                     (move.TotalLoadResults.PeakSpd * (1.0 + Settings.MainSettings.SpeedMargin / 100.0) < motor.MaxSpd))
                                                    {
                                                        //Check if the gearbox and motor passes the test based on the nominal and maximum torque                  
                                                        if (move.LoadResultsList[systemGearBoxNr].PeakTrq * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) < gearBox.MaxTrq / gearBox.Gearing)
                                                            if (move.LoadResultsList[systemGearBoxNr].RmsTrqED * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) < gearBox.NomTrq / gearBox.Gearing)
                                                        if (move.TotalLoadResults.PeakTrq * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) < motor.MaxTrq)
                                                            if (move.TotalLoadResults.Inertia / motor.Inertia * (1.0 + Settings.MainSettings.InertiaMargin / 100.0) < motor.InertiaRatio)
                                                                //Check that the motor passes the test based on the lifting torque
                                                                if (move.TotalLoadResults.LiftTorque * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) < ((Settings.MainSettings.AllowedLiftTorque / 100.0) * motor.NomTrq))
                                                                    //Check that the gearbox can handle the peak motor torque
                                                                    if (move.TotalLoadResults.PeakTrq * move.LoadResultsList[systemGearBoxNr].AccOutGearing < gearBox.MaxTrq) // * (1.0 + Settings.MainSettings.GearBoxOverLoad / 100.0))
                                                                    {
                                                                        //Add the rms torque and speed to a variable containing the total sum. This is used at the last move for calculating the rms value for all moves
                                                                        rmsTrqSum += Math.Pow(move.TotalLoadResults.RmsTrqED, 2) * move.Time;
                                                                        rmsSpdSum += Math.Pow(move.TotalLoadResults.RmsSpdED, 2) * move.Time;

                                                                        totalTime += move.Time;

                                                                        if (m.Key == _moves.MovesList.Count - 1)
                                                                        {
                                                                            gearBoxRmsSpd = Math.Sqrt(rmsSpdSum / totalTime);
                                                                            gearBoxRmsTrq = Math.Sqrt(rmsTrqSum / totalTime);

                                                                            if (gearBoxRmsTrq * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) < gearBox.NomTrq)
                                                                                if (gearBoxRmsSpd * (1.0 + Settings.MainSettings.SpeedMargin / 100.0) < gearBox.NomSpd)
                                                                                    gearBoxTestOK = true;
                                                                        }
                                                                        else
                                                                            //GearBox test for this move was succesful
                                                                            gearBoxTestOK = true;
                                                                    }
                                                    }
                                                    else
                                                    {
                                                        //This gearbox results in overspeed. Save the gearbox gearing.
                                                        //All further gearboxes checked with this motor must have a lower gearing.
                                                        gearingLimit = gearBox.Gearing;
                                                    }
                                            }
                                        }

                                        if (gearBoxTestOK)
                                        {
                                            //Make an in depth check of the motor

                                            //Check the mechanical interface the motor should connect to
                                            #region MotorInterfaceCheck

                                            MechInterface mechTypeOnMtrOutput = MechInterface.Undefined;
                                            double minDiaOnMtrOutput = 0;
                                            double maxDiaOnMtrOutput = 0;
                                            bool autoSelectOnMtrOutput = false;

                                            bool mechCheckDoneMotor = false;

                                            if (Settings.MainSettings.MechChkOnNonAutoSelCmp == true &&
                                                systemGearBoxNr != 0)
                                            {
                                                foreach (var iL in _iLoads)
                                                {
                                                    ILoad iLoad = iL.Value;

                                                    //Only attach to mechanical components
                                                    if (iLoad.Detachable == false)
                                                        switch (iLoad.MainType)
                                                        {
                                                            case MainType.GearBox:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var tmpGearBox = iLoad as GearBox;

                                                                mechTypeOnMtrOutput = tmpGearBox.InpType;
                                                                minDiaOnMtrOutput = tmpGearBox.InpMinDia;
                                                                maxDiaOnMtrOutput = tmpGearBox.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;

                                                            case MainType.Coupling:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var coupling = iLoad as Coupling;

                                                                mechTypeOnMtrOutput = coupling.InpType;
                                                                minDiaOnMtrOutput = coupling.InpMinDia;
                                                                maxDiaOnMtrOutput = coupling.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;

                                                            case MainType.BallScrew:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var ballScrew = iLoad as BallScrew;

                                                                mechTypeOnMtrOutput = ballScrew.InpType;
                                                                minDiaOnMtrOutput = ballScrew.InpMinDia;
                                                                maxDiaOnMtrOutput = ballScrew.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;

                                                            case MainType.RackAndPinion:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var rackAndPinion = iLoad as RackAndPinion;

                                                                mechTypeOnMtrOutput = rackAndPinion.InpType;
                                                                minDiaOnMtrOutput = rackAndPinion.InpMinDia;
                                                                maxDiaOnMtrOutput = rackAndPinion.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;

                                                            case MainType.TurnTable:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var turnTable = iLoad as TurnTable;

                                                                mechTypeOnMtrOutput = turnTable.InpType;
                                                                minDiaOnMtrOutput = turnTable.InpMinDia;
                                                                maxDiaOnMtrOutput = turnTable.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;

                                                            case MainType.ToothedBelt:

                                                                if (iLoad.AutoSelect)
                                                                    autoSelectOnMtrOutput = true;

                                                                var toothedBelt = iLoad as ToothedBelt;

                                                                mechTypeOnMtrOutput = toothedBelt.InpType;
                                                                minDiaOnMtrOutput = toothedBelt.InpMinDia;
                                                                maxDiaOnMtrOutput = toothedBelt.InpMaxDia;
                                                                mechCheckDoneMotor = true;

                                                                break;
                                                        }

                                                    if (mechCheckDoneMotor)
                                                        break;
                                                }
                                            }
                                            else
                                            {
                                                mechTypeOnMtrOutput = _dBGearBoxes[systemGearBoxNr].InpType;
                                                minDiaOnMtrOutput = _dBGearBoxes[systemGearBoxNr].InpMinDia;
                                                maxDiaOnMtrOutput = _dBGearBoxes[systemGearBoxNr].InpMaxDia;
                                            }

                                            # endregion

                                            //If mechanical test is active then check that the motor fits to the attached component
                                            if ((Settings.MainSettings.MechChkOnNonAutoSelCmp == false && !autoSelectOnMtrOutput) ||
                                                (((motor.OutType == MechInterface.SmoothShaft && mechTypeOnMtrOutput == MechInterface.SmoothHollow) ||
                                                (motor.OutType == MechInterface.SmoothShaft && mechTypeOnMtrOutput == MechInterface.KeyedHollow) ||
                                                (motor.OutType == MechInterface.KeyedShaft && mechTypeOnMtrOutput == MechInterface.SmoothHollow) ||
                                                (motor.OutType == MechInterface.KeyedShaft && mechTypeOnMtrOutput == MechInterface.KeyedHollow) ||
                                                (motor.OutType == MechInterface.Flange && mechTypeOnMtrOutput == MechInterface.Flange)) &&
                                                (motor.OutMinDia <= maxDiaOnMtrOutput && motor.OutMinDia >= minDiaOnMtrOutput)))

                                                //Run the calculation
                                                if (CalcMotorResults(motor))
                                                {
                                                    //Success

                                                    //Return the motor and gearbox number
                                                    MotorNr = motorIndex;
                                                    GearBoxArticleNr = gearBox.ArticleNo;//gearBoxIndex;
                                                    GearBoxPrice = gearBox.Price;

                                                    totalComponentPrice = motor.Price + motor.DrivePrice + gearBox.Price;
                                                    motor.Description = motor.Description;
                                                    gearBox.Description = gearBox.Description;
                                                    return true;
                                                }
                                        }
                                    }
                            }
                        }
                    }
                }
            }

            if (MotorNr == -1 && GearBoxArticleNr == null)
                //No motor and gearbox combination found
                return false;
            else
                //Return a flag that the search was succesful
                return true;
        }

        private void ReadGearBoxFromList(GearBox gearBox, int gearBoxNr)
        {
            GearBox dBGearBox = _dBGearBoxes[gearBoxNr];

            #region ReadGearBoxData

            gearBox.ArticleNo = dBGearBox.ArticleNo;

            gearBox.Campaign = dBGearBox.Campaign;
            gearBox.Sold = dBGearBox.Sold;

            switch (dBGearBox.Brand.ToString())
            {
                case ("LSIS"):
                    gearBox.Brand = GearBoxBrand.LSIS;
                    break;
                
                case ("Wittenstein"):
                    gearBox.Brand = GearBoxBrand.Wittenstein;
                    break;

                default:
                    gearBox.Brand = GearBoxBrand.Undefined;
                    break;
            }

            switch (dBGearBox.SubType.ToString())
            {
                case ("PlanetaryStraight"):
                    gearBox.SubType = GearBoxType.PlanetaryStraight;
                    break;

                case ("PlanetaryRightAngle"):
                    gearBox.SubType = GearBoxType.PlanetaryRightAngle;
                    break;

                case ("WormGear"):
                    gearBox.SubType = GearBoxType.WormGear;
                    break;

                default:
                    gearBox.SubType = GearBoxType.Undefined;
                    break;
            }

            gearBox.Description = dBGearBox.Description;
            gearBox.Price = dBGearBox.Price;

            //Main data
            gearBox.Gearing = dBGearBox.Gearing;
            gearBox.IdleTrq = dBGearBox.IdleTrq;
            gearBox.Inertia = dBGearBox.Inertia;
            gearBox.Eff = dBGearBox.Eff;
            gearBox.Backlash = dBGearBox.Backlash;

            //Mechanical input
            switch (dBGearBox.InpType.ToString())
            {
                case "Flange":
                    gearBox.InpType = MechInterface.Flange;
                    break;

                case "SmoothHollow":
                    gearBox.InpType = MechInterface.SmoothHollow;
                    break;

                case "KeyedHollow":
                    gearBox.InpType = MechInterface.KeyedHollow;
                    break;

                case "KeyedShaft":
                    gearBox.InpType = MechInterface.KeyedShaft;
                    break;

                case "SmoothShaft":
                    gearBox.InpType = MechInterface.SmoothShaft;
                    break;

                default:
                    gearBox.InpType = MechInterface.Undefined;
                    break;
            }

            gearBox.InpMinDia = dBGearBox.InpMinDia;
            gearBox.InpMaxDia = dBGearBox.InpMaxDia;
            gearBox.InpRadFrc = dBGearBox.InpRadFrc;
            gearBox.InpAxFrc = dBGearBox.InpAxFrc;

            //Mechanical output
            switch (dBGearBox.OutType.ToString())
            {
                case "Flange":
                    gearBox.OutType = MechInterface.Flange;
                    break;

                case "SmoothHollow":
                    gearBox.OutType = MechInterface.SmoothHollow;
                    break;

                case "KeyedHollow":
                    gearBox.OutType = MechInterface.KeyedHollow;
                    break;

                case "KeyedShaft":
                    gearBox.OutType = MechInterface.KeyedShaft;
                    break;

                case "SmoothShaft":
                    gearBox.OutType = MechInterface.SmoothShaft;
                    break;

                default:
                    gearBox.OutType = MechInterface.Undefined;
                    break;
            }

            gearBox.OutMinDia = dBGearBox.OutMinDia;
            gearBox.OutMaxDia = dBGearBox.OutMaxDia;
            gearBox.OutRadFrc = dBGearBox.OutRadFrc;
            gearBox.OutAxFrc = dBGearBox.OutAxFrc;

            //Torque data
            gearBox.MaxTrq = dBGearBox.MaxTrq;
            gearBox.MaxTrqDur = dBGearBox.MaxTrqDur;
            gearBox.TrqED = dBGearBox.TrqED;
            gearBox.NomTrq = dBGearBox.NomTrq;

            //Speed data
            gearBox.MaxSpd = dBGearBox.MaxSpd;
            gearBox.MaxSpdDur = dBGearBox.MaxSpdDur;
            gearBox.SpdED = dBGearBox.SpdED;
            gearBox.NomSpd = dBGearBox.NomSpd;

            #endregion

        }

        private void ReadGearBoxFromdB(GearBox gearBox, string gearBoxArticleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Query all gearboxes in the database that can be used with this motor
                    var qryGearBoxes =
                        from tblGearBoxes myGearBoxes in dbDriveSystemsContext.tblGearBoxes
                        where myGearBoxes.ArticleNo == gearBoxArticleNo
                        & myGearBoxes.Gearing >= minGearing
                        & myGearBoxes.Gearing <= maxGearing
                        select myGearBoxes;

                    foreach (var dBGearBox in qryGearBoxes) //Should only be one item
                    {


                        #region ReadGearBoxData

                        gearBox.ArticleNo = dBGearBox.ArticleNo;

                        gearBox.Campaign = dBGearBox.Campaign;
                        gearBox.Sold = dBGearBox.Sold;

                        switch (dBGearBox.Brand.ToString())
                        {
                            case ("LSIS"):
                                gearBox.Brand = GearBoxBrand.LSIS;
                                break;
                            
                            case ("Wittenstein"):
                                gearBox.Brand = GearBoxBrand.Wittenstein;
                                break;

                            default:
                                gearBox.Brand = GearBoxBrand.Undefined;
                                break;
                        }

                        switch (dBGearBox.SubType.ToString())
                        {
                            case ("PlanetaryStraight"):
                                gearBox.SubType = GearBoxType.PlanetaryStraight;
                                break;

                            case ("PlanetaryRightAngle"):
                                gearBox.SubType = GearBoxType.PlanetaryRightAngle;
                                break;

                            case ("WormGear"):
                                gearBox.SubType = GearBoxType.WormGear;
                                break;

                            default:
                                gearBox.SubType = GearBoxType.Undefined;
                                break;
                        }

                        gearBox.Description = dBGearBox.Description;
                        gearBox.Price = dBGearBox.Price;

                        //Main data
                        gearBox.Gearing = dBGearBox.Gearing;
                        gearBox.IdleTrq = dBGearBox.IdleTrq;
                        gearBox.Inertia = dBGearBox.Inertia / 10000;
                        gearBox.Eff = dBGearBox.Eff;
                        gearBox.Backlash = dBGearBox.Backlash;

                        //Mechanical input
                        switch (dBGearBox.InpType.ToString())
                        {
                            case "Flange":
                                gearBox.InpType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                gearBox.InpType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                gearBox.InpType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                gearBox.InpType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                gearBox.InpType = MechInterface.SmoothShaft;
                                break;

                            default:
                                gearBox.InpType = MechInterface.Undefined;
                                break;
                        }

                        gearBox.InpMinDia = dBGearBox.InpMinDia;
                        gearBox.InpMaxDia = dBGearBox.InpMaxDia;
                        gearBox.InpRadFrc = dBGearBox.InpRadFrc;
                        gearBox.InpAxFrc = dBGearBox.InpAxFrc;

                        //Mechanical output
                        switch (dBGearBox.OutType.ToString())
                        {
                            case "Flange":
                                gearBox.OutType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                gearBox.OutType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                gearBox.OutType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                gearBox.OutType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                gearBox.OutType = MechInterface.SmoothShaft;
                                break;

                            default:
                                gearBox.OutType = MechInterface.Undefined;
                                break;
                        }

                        gearBox.OutMinDia = dBGearBox.OutMinDia;
                        gearBox.OutMaxDia = dBGearBox.OutMaxDia;
                        gearBox.OutRadFrc = dBGearBox.OutRadFrc;
                        gearBox.OutAxFrc = dBGearBox.OutAxFrc;

                        //Torque data
                        gearBox.MaxTrq = dBGearBox.MaxTrq;
                        gearBox.MaxTrqDur = dBGearBox.MaxTrqDur;
                        gearBox.TrqED = dBGearBox.TrqED;
                        gearBox.NomTrq = dBGearBox.NomTrq;

                        //Speed data
                        gearBox.MaxSpd = dBGearBox.MaxSpd;
                        gearBox.MaxSpdDur = dBGearBox.MaxSpdDur;
                        gearBox.SpdED = dBGearBox.SpdED;
                        gearBox.NomSpd = dBGearBox.NomSpd;

                        #endregion

                    }
                }
            }
        }


        private void ReadMotorFromdB(Motor motor, string articleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read the data from the Motors table
                    var qryMotor =
                        from tblMotors myMotors in dbDriveSystemsContext.tblMotors
                        where myMotors.ArticleNo == articleNo
                        select myMotors;

                    //Load the motor object with its data
                    foreach (var dBMotor in qryMotor) //Should only be one item
                    {
                        motor.ArticleNo = dBMotor.ArticleNo;

                        motor.Campaign = dBMotor.Campaign;
                        motor.Sold = dBMotor.Sold;

                        motor.Brand = Brand.BSD; 

                        motor.Description = dBMotor.Description;

                        if (dBMotor.Braked == "Yes")
                            motor.Braked = true;
                        else
                            motor.Braked = false;

                        // ToDo: motor. = dBMotor.Type;
                        motor.Price = dBMotor.Price;

                        //Main data
                        motor.Resolution = dBMotor.Resolution;
                        motor.InertiaRatio = dBMotor.InertiaRatio;
                        motor.Inertia = dBMotor.Inertia / 10000;
                        motor.Eff = dBMotor.Eff;
                        motor.FldWeakSpd = dBMotor.FldWeakSpd;
                        motor.InvEff = dBMotor.InverseEfficiency;

                        //Mechanical output
                        switch (dBMotor.OutType)
                        {
                            case "Flange":
                                motor.OutType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                motor.OutType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                motor.OutType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                motor.OutType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                motor.OutType = MechInterface.SmoothShaft;
                                break;

                            default:
                                motor.OutType = MechInterface.Undefined;
                                break;
                        }

                        motor.OutMinDia = dBMotor.OutMinDia;
                        motor.OutMaxDia = dBMotor.OutMaxDia;
                        motor.OutRadFrc = dBMotor.OutRadFrc;
                        motor.OutAxFrc = dBMotor.OutAxFrc;

                        //Torque data
                        motor.MaxTrq = dBMotor.MaxTrq;
                        motor.MaxTrqDur = dBMotor.MaxTrqDur;
                        motor.TrqED = dBMotor.TrqED;
                        motor.NomTrq = dBMotor.NomTrq;

                        //Speed data
                        motor.MaxSpd = dBMotor.MaxSpd;
                        motor.MaxSpdDur = dBMotor.MaxSpdDur;
                        motor.SpdED = dBMotor.SpdED;
                        motor.NomSpd = dBMotor.NomSpd;
                    }
                }
            }
        }

        public bool CalcMotorResults(Motor motor)
        {
            //Calculate the motor results based on the selected motor (without accessing the database)

            //If the torque ED-factor is not defined then set a conservative value
            if (motor.MaxTrqDur <= 0)
                motor.MaxTrqDur = 1;

            //If the speed ED-factor is not defined then set a conservative value
            if (motor.MaxSpdDur <= 0)
                motor.MaxSpdDur = 1;

            //If the field weakening speed is not defined then set conservative values
            if (motor.FldWeakSpd == 0)
                motor.FldWeakSpd = motor.NomSpd * 0.75;
            //frmOutput... send message

            bool fieldWeakening = false;

            //Clear the motor results before calculating
            _moves.MotorResults = null;
            _moves.MotorResults = new MotorResult();

            Queue<double> _debug = new Queue<double>();

            //the rms-values are based on the duty cycle. During this cycle the rms-value must not exceed the nominal value.
            double dutyCycleTrq = (motor.MaxTrqDur / (motor.TrqED / 100)) * Settings.MainSettings.Resolution;
            double dutyCycleSpd = (motor.MaxSpdDur / (motor.SpdED / 100)) * Settings.MainSettings.Resolution;

            //Go through the motionProfile to check the motor rms- and maximum torque.
            foreach (var m in _moves.MovesList)
            {
                Move move = m.Value;

                //Save the peak inertia
                if (motor.Inertia + move.TotalLoadResults.Inertia > _moves.MotorResults.Inertia) //????????????
                    _moves.MotorResults.Inertia = motor.Inertia + move.TotalLoadResults.Inertia;

                foreach (Sector sector in move.SectorsList)
                {
                    //Check the movement direction
                    bool forwardOrUp = (sector.Distance >= 0);

                    foreach (SectorElement mp in sector.SectorElementsList)
                    {
                        //Check the available motor torque in relation to the motor speed. If the motorspeed is
                        //above the field weakening point the available torque is assumed to drop at a linear rate.

                        //Recalculate the velocity from m/s to rpm
                        double spdRpm = 60 * (mp.Vel / move.TotalLoadResults.InpMovePerRev);
                        if (spdRpm > 4000)
                            fieldWeakening = fieldWeakening;

                        if (spdRpm > motor.FldWeakSpd)
                            mp.MotorTorqueLimit = motor.MaxTrq * motor.FldWeakSpd / spdRpm;
                        else
                            mp.MotorTorqueLimit = motor.MaxTrq;

                        //torque
                        //Calculate the total torque profile. This profile is offset with the lifting torque
                        mp.Torque = move.TotalLoadResults.LiftTorque + move.TotalLoadResults.ThrustTorque +
                            (move.TotalLoadResults.Inertia + motor.Inertia) * (mp.Acc * 2 * Math.PI) /
                            (move.TotalLoadResults.InpMovePerRev * (motor.Eff / 100));

                        if (sector.Distance != 0)
                            if (forwardOrUp)
                                mp.Torque += move.TotalLoadResults.IdleTorque;
                            else
                                mp.Torque -= move.TotalLoadResults.IdleTorque;

                        //if the available motor torque is below the application torque then leave the loop
                        if (Math.Abs(mp.Torque + move.TotalLoadResults.ConstTorque) > Math.Abs(mp.MotorTorqueLimit)) //???????????????
                            fieldWeakening = true;

                        //Save the peak acc torque value
                        //  if (Math.Abs(accTorque) > Math.Abs(_moves.MotorResults.AccTorque))
                        //    _moves.MotorResults.AccTorque = accTorque;

                        //Save the peak load torque value
                        if (mp.Torque > _moves.MotorResults.PeakTrq)
                            _moves.MotorResults.PeakTrq = mp.Torque;

                        //Calculate the square of the momentary torque value
                        double temp = Math.Pow(mp.Torque, 2);

                        //Add the current torque value to the FIFO
                        _moves.MotorResults.QueueRmsTrq.Enqueue(temp);

                        //Add the result to a sum for all rms-values for the given duration
                        _moves.MotorResults.RmsTrqSum += temp;

                        //Check if the number of items in the FIFO exceeds the time duration
                        if (_moves.MotorResults.QueueRmsTrq.Count > dutyCycleTrq)
                        {
                            //Save the rms value if it exceeds the previously stored value
                            if (_moves.MotorResults.RmsTrqSum / _moves.MotorResults.QueueRmsTrq.Count > _moves.MotorResults.RmsTrqED)
                                _moves.MotorResults.RmsTrqED =
                                    Math.Sqrt(_moves.MotorResults.RmsTrqSum / _moves.MotorResults.QueueRmsTrq.Count);

                            //Remove the oldest value from the FIFO
                            _moves.MotorResults.RmsTrqSum -= _moves.MotorResults.QueueRmsTrq.Peek();
                            _moves.MotorResults.QueueRmsTrq.Dequeue();
                        }

                        //speed         
                        if (Math.Abs(spdRpm) > Math.Abs(_moves.MotorResults.PeakSpd))
                            _moves.MotorResults.PeakSpd = spdRpm;

                        spdRpm = Math.Pow(spdRpm, 2);

                        //Add the current speed value to the FIFO
                        _moves.MotorResults.QueueRmsSpd.Enqueue(spdRpm);

                        //Add the result to a sum for all rms-values for the given duration
                        _moves.MotorResults.RmsSpdSum += spdRpm;

                        //Check if the number of items in the FIFO exceeds the time duration
                        if (_moves.MotorResults.QueueRmsSpd.Count > dutyCycleSpd)
                        {
                            if (Math.Sqrt(_moves.MotorResults.RmsSpdSum / dutyCycleSpd) > _moves.MotorResults.RmsSpdED)
                                _moves.MotorResults.RmsSpdED = Math.Sqrt(_moves.MotorResults.RmsSpdSum / dutyCycleSpd);

                            //Remove the oldest value from the FIFO
                            _moves.MotorResults.RmsSpdSum -= _moves.MotorResults.QueueRmsSpd.Peek();
                            _moves.MotorResults.QueueRmsSpd.Dequeue();
                        }

                        //Regenerative energy including motor
                        if ((forwardOrUp && mp.Torque < 0) || (!forwardOrUp && mp.Torque > 0))
                            mp.RegenerativePower = Math.Abs(mp.Torque * (mp.Vel / (move.TotalLoadResults.InpMovePerRev)));
                        else
                            mp.RegenerativePower = 0;
                    }
                }
            }

            //If the total duration for the moves did not exceed the allowed ED duration then 
            //calculate the rms-values based on the number of samples
            if (_moves.MotorResults.RmsTrqED == 0)
                _moves.MotorResults.RmsTrqED = Math.Sqrt(_moves.MotorResults.RmsTrqSum / _moves.MotorResults.QueueRmsTrq.Count);

           // _moves.MotorResults.PeakTrq = Math.Abs(_moves.MotorResults.AccTorque); // +_moves.MotorResults.ConstTorque;

            //If the total duration for the move did not exceed the allowed ED duration then 
            //calculate the rms-values based on the number of samples
            if (_moves.MotorResults.RmsSpdED == 0)
                _moves.MotorResults.RmsSpdED = Math.Sqrt(_moves.MotorResults.RmsSpdSum / _moves.MotorResults.QueueRmsSpd.Count);

            //Check if the motor passed the check
            if (_moves.MotorResults.PeakTrq * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) <= motor.MaxTrq &&
                _moves.MotorResults.RmsTrqED * (1.0 + Settings.MainSettings.TorqueMargin / 100.0) <= motor.NomTrq &&
                _moves.MotorResults.PeakSpd * (1.0 + Settings.MainSettings.SpeedMargin / 100.0) <= motor.MaxSpd &&
                _moves.MotorResults.RmsSpdED * (1.0 + Settings.MainSettings.SpeedMargin / 100.0) <= motor.NomSpd &&
                motor.InertiaRatio > (_moves.MotorResults.Inertia / motor.Inertia) * (1.0 + Settings.MainSettings.InertiaMargin / 100.0) &&
                fieldWeakening == false)

                return true;
            else
                return false;
        }

        private double FormatValue(double raw)
        {
            //Hm........?
            if (Math.Abs(raw) < 10)
                return Math.Round(raw, 2);

            if (Math.Abs(raw) < 100)
                return Math.Round(raw, 1);
            else
                return Math.Round(raw, 0);
        }

        public void CalculateMovesResults()
        {
            //Recalculate the moves
            foreach (var m in _moves.MovesList)
                m.Value.CalcLoadResults(_iLoads);

            //Calculate the results for each load

            //Clear the LoadResult list
            _moves.LoadResultsList.Clear();

            foreach (var l in _iLoads)
            {
                int loadNr = l.Key;
                ILoad iL = l.Value;

                double totalTime = 0;

                LoadResult loadResult = new LoadResult(iL.Name, iL.MainType, iL.AutoSelect);

                _moves.LoadResultsList.Add(loadNr, loadResult);

                foreach (var m in _moves.MovesList)
                {
                    //This calculation is cheating since it does not return the true rms values. However the duty cycles for the
                    //mechanical components are often very long (> 600s) which means that a accurate calculation is normally not needed.
                    //ToDo!!!

                    Move move = m.Value;

                    _moves.LoadResultsList[loadNr].AccOutGearing = move.LoadResultsList[loadNr].AccOutGearing;
                    _moves.LoadResultsList[loadNr].AccInpGearing = move.LoadResultsList[loadNr].AccInpGearing;


                    if (Math.Abs(move.LoadResultsList[loadNr].PeakSpd) > Math.Abs(_moves.LoadResultsList[loadNr].PeakSpd))
                        _moves.LoadResultsList[loadNr].PeakSpd = Math.Abs(move.LoadResultsList[loadNr].PeakSpd);

                    _moves.LoadResultsList[loadNr].RmsSpdSum += Math.Pow(move.LoadResultsList[loadNr].RmsSpdED, 2) * move.Time;

                    if (Math.Abs(move.LoadResultsList[loadNr].PeakTrq) > Math.Abs(_moves.LoadResultsList[loadNr].PeakTrq))
                        _moves.LoadResultsList[loadNr].PeakTrq = Math.Abs(move.LoadResultsList[loadNr].PeakTrq);

                    _moves.LoadResultsList[loadNr].RmsTrqSum += Math.Pow(move.LoadResultsList[loadNr].RmsTrqED, 2) * move.Time;

                    totalTime += move.Time;
                }

                _moves.LoadResultsList[loadNr].RmsSpdED = Math.Sqrt(_moves.LoadResultsList[loadNr].RmsSpdSum / totalTime);
                _moves.LoadResultsList[loadNr].RmsTrqED = Math.Sqrt(_moves.LoadResultsList[loadNr].RmsTrqSum / totalTime);
            }

            //Calculate the motor results
            CalcMotorResults(_motor);
        }

        public void DisplayMovesResults()
        {
            //Display the results for each load.

            //Clear the loadResults table
            if (dataGridView_LoadResultsTable.RowCount > 0)
                dataGridView_LoadResultsTable.Rows.Clear();

            dataGridView_LoadResultsTable.DataSource = null;

            //Display the motor results
            double maxTrqPercent = Math.Abs(Math.Round((_moves.MotorResults.PeakTrq / _motor.MaxTrq) * 100, 0));
            double nomTrqPercent = Math.Abs(Math.Round((_moves.MotorResults.RmsTrqED / _motor.NomTrq) * 100, 0));

            double maxSpdPercent = Math.Abs(Math.Round((_moves.MotorResults.PeakSpd / _motor.MaxSpd) * 100, 0));
            double nomSpdPercent = Math.Abs(Math.Round((_moves.MotorResults.RmsSpdED / _motor.NomSpd) * 100, 0));

            //Add the motor results to the table
            dataGridView_LoadResultsTable.Rows.Add(_motor.Name, _motor.Description,
                FormatValue(_moves.MotorResults.PeakSpd) + " (" + maxSpdPercent + "%)",
                FormatValue(_moves.MotorResults.RmsSpdED) + " (" + nomSpdPercent + "%)",

                FormatValue(_moves.MotorResults.PeakTrq) + " (" + maxTrqPercent + "%)",
                FormatValue(_moves.MotorResults.RmsTrqED) + " (" + nomTrqPercent + "%)",
                "-", "-", "-", "-", _motor.OutType, _motor.OutMinDia, _motor.OutMaxDia);

            //Get the handle of the last added row
            DataGridViewRow row = new DataGridViewRow();
            row = dataGridView_LoadResultsTable.Rows[dataGridView_LoadResultsTable.RowCount - 1];

            //Change the background color if the motor is user defined
            if (_motor.AutoSelect == true)
                row.DefaultCellStyle.BackColor = Color.PowderBlue;
            else
                row.DefaultCellStyle.BackColor = Color.White;

            //Check if the result is above treshold values
            if (maxSpdPercent > 80 || maxSpdPercent == 0)
                if (maxSpdPercent > 100)
                    row.Cells[2].Style.BackColor = Color.Coral;
                else
                    row.Cells[2].Style.BackColor = Color.Gold;

            if (nomSpdPercent > 80 || nomSpdPercent == 0)
                if (nomSpdPercent > 100)
                    row.Cells[3].Style.BackColor = Color.Coral;
                else
                    row.Cells[3].Style.BackColor = Color.Gold;

            if (maxTrqPercent > 80 || maxTrqPercent == 0)
                if (maxTrqPercent > 100)
                    row.Cells[4].Style.BackColor = Color.Coral;
                else
                    row.Cells[4].Style.BackColor = Color.Gold;

            if (nomTrqPercent > 80 || nomTrqPercent == 0)
                if (nomTrqPercent > 100)
                    row.Cells[5].Style.BackColor = Color.Coral;
                else
                    row.Cells[5].Style.BackColor = Color.Gold;
            
            MechInterface prevOutType = _motor.OutType;
            double prevOutMinDia = _motor.OutMinDia;
            double prevOutMaxDia = _motor.OutMaxDia;

            //Display the load results                
            foreach (var lR in _iLoads)
            {
                ILoad iL = lR.Value;
                int loadNr = lR.Key;

                string description = "";

                MechInterface inpType = MechInterface.Undefined;
                double inpMinDia = 0;
                double inpMaxDia = 0;

                MechInterface outType = MechInterface.Undefined;
                double outMinDia = 0;
                double outMaxDia = 0;

                double maxSpd = 0;
                double maxTrq = 0;
                double nomSpd = 0;
                double nomTrq = 0;

                double gearing = 1;

                double loadRatio = 0;

                switch (iL.MainType)
                {
                    case MainType.GearBox:
                        var gearBox = iL as GearBox;
                        _moves.LoadResultsList[loadNr].Name = gearBox.Name;
                        description = gearBox.Description;

                        maxSpd = gearBox.MaxSpd;
                        maxTrq = gearBox.MaxTrq;
                        nomSpd = gearBox.NomSpd;
                        nomTrq = gearBox.NomTrq;

                        gearing = gearBox.Gearing;

                        inpType = gearBox.InpType;
                        inpMinDia = gearBox.InpMinDia;
                        inpMaxDia = gearBox.InpMaxDia;

                        outType = gearBox.OutType;
                        outMinDia = gearBox.OutMinDia;
                        outMaxDia = gearBox.OutMaxDia;

                        //Motor torque limit
                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100; //ToDo!!!

                        double motorTorqueLimitPercent = FormatValue(300 / loadRatio) * 100;

                        if (motorTorqueLimitPercent > 300)
                            motorTorqueLimitPercent = 300;

                        double motorTorqueLimit = FormatValue(motorTorqueLimitPercent / 300 * _motor.MaxTrq);
                        
                        textBox_MotorTorqueLimit.Text = motorTorqueLimitPercent.ToString() + "%" + " (" + motorTorqueLimit.ToString() + "Nm)";

                        if (motorTorqueLimitPercent < 20)
                            if (motorTorqueLimitPercent < 5)
                                textBox_MotorTorqueLimit.BackColor = Color.Salmon;
                            else
                                textBox_MotorTorqueLimit.BackColor = Color.Gold;
                        else
                            textBox_MotorTorqueLimit.BackColor = SystemColors.Window;
                        
                        break;

                    case MainType.Coupling:
                        var coupling = iL as Coupling;
                        _moves.LoadResultsList[loadNr].Name = coupling.Name;
                        description = coupling.Description;
                        maxSpd = coupling.MaxSpd;
                        maxTrq = coupling.MaxTrq;
                        nomSpd = coupling.NomSpd;
                        nomTrq = coupling.NomTrq;

                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100;

                        inpType = coupling.InpType;
                        inpMinDia = coupling.InpMinDia;
                        inpMaxDia = coupling.InpMaxDia;

                        outType = coupling.OutType;
                        outMinDia = coupling.OutMinDia;
                        outMaxDia = coupling.OutMaxDia;
                        break;

                    case MainType.BallScrew:
                        var ballScrew = iL as BallScrew;
                        _moves.LoadResultsList[loadNr].Name = ballScrew.Name;
                        description = ballScrew.Description;
                        maxSpd = ballScrew.MaxSpd;
                        maxTrq = ballScrew.MaxTrq;
                        nomSpd = ballScrew.NomSpd;
                        nomTrq = ballScrew.NomTrq;

                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100;

                        inpType = ballScrew.InpType;
                        inpMinDia = ballScrew.InpMinDia;
                        inpMaxDia = ballScrew.InpMaxDia;

                        outType = MechInterface.Undefined;
                        outMinDia = 0;
                        outMaxDia = 0;
                        break;

                    case MainType.RackAndPinion:
                        var rackAndPinion = iL as RackAndPinion;
                        _moves.LoadResultsList[loadNr].Name = rackAndPinion.Name;
                        description = rackAndPinion.Description;
                        maxSpd = rackAndPinion.MaxSpd;
                        maxTrq = rackAndPinion.MaxTrq;
                        nomSpd = rackAndPinion.NomSpd;
                        nomTrq = rackAndPinion.NomTrq;

                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100;

                        inpType = rackAndPinion.InpType;
                        inpMinDia = rackAndPinion.InpMinDia;
                        inpMaxDia = rackAndPinion.InpMaxDia;

                        outType = MechInterface.Undefined;
                        outMinDia = 0;
                        outMaxDia = 0;
                        break;

                    case MainType.TurnTable:
                        var turnTable = iL as TurnTable;
                        _moves.LoadResultsList[loadNr].Name = turnTable.Name;
                        description = turnTable.Description;
                        maxSpd = turnTable.MaxSpd;
                        maxTrq = turnTable.MaxTrq;
                        nomSpd = turnTable.NomSpd;
                        nomTrq = turnTable.NomTrq;

                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100;

                        inpType = turnTable.InpType;
                        inpMinDia = turnTable.InpMinDia;
                        inpMaxDia = turnTable.InpMaxDia;

                        outType = MechInterface.Undefined;
                        outMinDia = 0;
                        outMaxDia = 0;
                        break;

                    case MainType.ToothedBelt:
                        var toothedBelt = iL as ToothedBelt;
                        _moves.LoadResultsList[loadNr].Name = toothedBelt.Name;
                        description = toothedBelt.Description;
                        maxSpd = toothedBelt.MaxSpd;
                        maxTrq = toothedBelt.MaxTrq;
                        nomSpd = toothedBelt.NomSpd;
                        nomTrq = toothedBelt.NomTrq;

                        loadRatio = FormatValue((_motor.MaxTrq * _moves.LoadResultsList[loadNr].AccOutGearing) / maxTrq) * 100;

                        inpType = toothedBelt.InpType;
                        inpMinDia = toothedBelt.InpMinDia;
                        inpMaxDia = toothedBelt.InpMaxDia;

                        outType = MechInterface.Undefined;
                        outMinDia = 0;
                        outMaxDia = 0;
                        break;
                }

                //Add data to the LoadResults table
                maxSpdPercent = Math.Abs(Math.Round((_moves.LoadResultsList[loadNr].PeakSpd / maxSpd) * 100, 0));
                nomSpdPercent = Math.Abs(Math.Round((_moves.LoadResultsList[loadNr].RmsSpdED / nomSpd) * 100, 0));

                maxTrqPercent = Math.Abs(Math.Round((_moves.LoadResultsList[loadNr].PeakTrq * gearing / maxTrq) * 100, 0));
                nomTrqPercent = Math.Abs(Math.Round((_moves.LoadResultsList[loadNr].RmsTrqED * gearing / nomTrq) * 100, 0));

                if (maxSpd == 0)
                {
                    maxSpdPercent = 0;
                    nomSpdPercent = 0;

                    maxTrqPercent = 0;
                    nomTrqPercent = 0;
                }

                dataGridView_LoadResultsTable.Rows.Add(_moves.LoadResultsList[loadNr].Name, description,
                    FormatValue(_moves.LoadResultsList[loadNr].PeakSpd) + " (" + maxSpdPercent + "%)",
                    FormatValue(_moves.LoadResultsList[loadNr].RmsSpdED) + " (" + nomSpdPercent + "%)",

                    FormatValue(_moves.LoadResultsList[loadNr].PeakTrq) + " (" + maxTrqPercent + "%)",
                    FormatValue(_moves.LoadResultsList[loadNr].RmsTrqED) + " (" + nomTrqPercent + "%)",
                loadRatio + " %",
                inpType, inpMinDia, inpMaxDia, outType, outMinDia, outMaxDia);

                //Get the handle of the last added row
                row = dataGridView_LoadResultsTable.Rows[dataGridView_LoadResultsTable.RowCount - 1];

                //Change the background color if the load is user defined
                if (iL.AutoSelect == true)
                    row.DefaultCellStyle.BackColor = Color.PowderBlue;
                else
                    row.DefaultCellStyle.BackColor = Color.White;

                //Check if the result is above treshold values
                if (maxSpdPercent > 80 || maxSpdPercent == 0)
                    if (maxSpdPercent > 100)
                        row.Cells[2].Style.BackColor = Color.Coral;
                    else
                        row.Cells[2].Style.BackColor = Color.Gold;

                if (nomSpdPercent > 80 || nomSpdPercent == 0)
                    if (nomSpdPercent > 100)
                        row.Cells[3].Style.BackColor = Color.Coral;
                    else
                        row.Cells[3].Style.BackColor = Color.Gold;

                if (maxTrqPercent > 80 || maxTrqPercent == 0)
                    if (maxTrqPercent > 100)
                        row.Cells[4].Style.BackColor = Color.Coral;
                    else
                        row.Cells[4].Style.BackColor = Color.Gold;

                if (nomTrqPercent > 80 || nomTrqPercent == 0)
                    if (nomTrqPercent > 100)
                        row.Cells[5].Style.BackColor = Color.Coral;
                    else
                        row.Cells[5].Style.BackColor = Color.Gold;

                if (loadRatio > 100 || loadRatio == 0)
                    if (loadRatio > 200)
                        row.Cells[6].Style.BackColor = Color.Coral;
                    else
                        row.Cells[6].Style.BackColor = Color.Gold;

                //Check the mechanical match
                if (prevOutType == MechInterface.SmoothShaft && inpType == MechInterface.SmoothShaft ||
                prevOutType == MechInterface.SmoothShaft && inpType == MechInterface.KeyedShaft ||
                prevOutType == MechInterface.KeyedShaft && inpType == MechInterface.SmoothShaft ||
                prevOutType == MechInterface.KeyedShaft && inpType == MechInterface.KeyedShaft ||

                prevOutType == MechInterface.SmoothHollow && inpType == MechInterface.SmoothHollow ||
                prevOutType == MechInterface.SmoothHollow && inpType == MechInterface.KeyedHollow ||
                prevOutType == MechInterface.KeyedHollow && inpType == MechInterface.SmoothShaft ||
                prevOutType == MechInterface.KeyedHollow && inpType == MechInterface.KeyedHollow ||

                prevOutType == MechInterface.Flange && inpType != MechInterface.Flange)
                    row.Cells[7].Style.BackColor = Color.Salmon;

                if (prevOutMaxDia > inpMaxDia)
                    row.Cells[9].Style.BackColor = Color.Salmon;

                if (prevOutMinDia < inpMinDia)
                    row.Cells[8].Style.BackColor = Color.Salmon;

                prevOutType = outType;
                prevOutMinDia = outMinDia;
                prevOutMaxDia = outMaxDia;

            }

            //Display additional result in the form textboxes
            double inertiaPercent = Math.Abs(Math.Round((_moves.MotorResults.Inertia - _motor.Inertia) / (_motor.Inertia * _motor.InertiaRatio) * 100, 0));

            double inertiaRatio = FormatValue((_moves.MotorResults.Inertia - _motor.Inertia) / _motor.Inertia);
            textBox_InertiaRatio.Text = inertiaRatio.ToString() + " (" + inertiaPercent.ToString() + "%)";

            if (inertiaPercent > 80 || inertiaPercent == 0)
                if (inertiaPercent > 100)
                    textBox_InertiaRatio.BackColor = Color.Salmon;
                else
                    textBox_InertiaRatio.BackColor = Color.Gold;
            else
                textBox_InertiaRatio.BackColor = SystemColors.Window;          
        }

        public void ClearTables()
        {
            //Clear the loadResults table
            if (dataGridView_Moves.RowCount > 0)
                dataGridView_Moves.Rows.Clear();

            dataGridView_Moves.DataSource = null;

            //Clear the loadResults table
            if (dataGridView_LoadResultsTable.RowCount > 0)
                dataGridView_LoadResultsTable.Rows.Clear();

            dataGridView_LoadResultsTable.DataSource = null;
        }            

        #region PrintMethods
        //Return the requested object to the main form for printing

        public DataGridView GetDGVMoves()
        {
            return dataGridView_Moves;
        }

        public DataGridView GetDGVComponents()
        {
            return dataGridView_LoadResultsTable;
        }

        public TextBox GetInertiaRatioTextBox()
        {
            return textBox_InertiaRatio;
        }

        public TextBox GetTorqueLimitTextBox()
        {
            return textBox_MotorTorqueLimit;
        }

        #endregion PrintMethods


        #endregion Methods
    }
}

 

          

        