using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Reflection;
using System.Windows.Forms;
using System.IO;
using WeifenLuo.WinFormsUI.Docking;
using DockSample.Customization;
using System.Xml;
using System.Globalization;
using System.Data.OleDb;
using System.Linq;
using DGVPrinterHelper;
using System.Drawing.Printing;
using System.Windows.Forms.DataVisualization.Charting;
using System.Windows.Forms.PropertyGridInternal;
using Pitman.Printing;

namespace DockSample
{
    #region BSDDriveSizer

    public partial class MainForm : Form
    {
        public struct columnData
        {
            public string columnName;
            public int columnWidth;

            public columnData(string str, int wdt)
            {
                columnName = str;
                columnWidth = wdt;
            }
        }

        private bool m_bSaveLayout = true;
        private DeserializeDockContent m_deserializeDockContent;

        //Create the default application forms
        //Left side
        private FrmSplash frmSplash;

        private FrmNavigator frmNavigator;
        private FrmSettings frmSettings;
        private FrmProperties frmProperties;

        //Main
        private FrmOverview frmOverview;
        private FrmAccessories frmAccessories;
        private FrmResults frmResults;

        //Right side
        private FrmGraph frmGraph;
        private FrmTable frmTable;
        private DummyToolbox frmToolbox;

        //Bottom
        private DummyDoc frmOutput;
        private DummyDoc frmUnitConversion;
        private DummyDoc frmHelp;

        //Create the default application objects
        private ApplicationData applicationData = new ApplicationData();
        private Motor motor = new Motor();
        private Moves moves = new Moves();

        private XML xml;

        public SortedList<int, ILoad> iLoads = new SortedList<int, ILoad>();

        public SortedList<int, IAccessory> iAccessories = new SortedList<int, IAccessory>();

        //Check if a load or move should be focused when the
        //corresponding node is selected in the TreeView
        bool leftMouseButton = false;

        //Check if unsaved changes exists
        private bool unsavedData = false;

        //Database password
        //public string dBPassWord = "BBSD";

        //Main file path
        string mainPath = @"C:\";

        //File name
        string mainFileName;

        //Disable calculations while values are being read from database
        private bool disableUpdate = false;

        //Create an instance of the datagridprinter class
        private DGVPrinter myDGVPrinter;

        //Page pointer when printing
        private int printIndex;

        public MainForm()
        {
            m_deserializeDockContent = new DeserializeDockContent(GetContentFromPersistString);

            InitializeComponent();
        }

        private void MainForm_Load(object sender, System.EventArgs e)
        {
            frmSplash = new FrmSplash();

            frmSplash.Show();
            
            applicationData.PropertyChanged += new PropertyChangedEventHandler(ApplicationData_PropertyChanged);
            moves.PropertyChanged += new PropertyChangedEventHandler(Moves_PropertyChanged);
            motor.PropertyChanged += new PropertyChangedEventHandler(Motor_PropertyChanged);

            //Create the accessories list
            CreateAccessoriesList();

            xml = new XML();

            //Create the default forms that can be displayed at startup
            frmNavigator = new FrmNavigator();
            frmSettings = new FrmSettings();
            frmProperties = new FrmProperties(iLoads, iAccessories, motor);

            frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
            frmAccessories = new FrmAccessories(motor, iAccessories, moves);
            frmResults = new FrmResults(moves, iLoads, motor, applicationData);

            frmGraph = new FrmGraph();
            frmTable = new FrmTable();
            frmToolbox = new DummyToolbox();

            frmOutput = CreateNewDocument("Output");
            frmUnitConversion = CreateNewDocument("Unit conversion");
            frmHelp = CreateNewHelpDocument("Help");

            //Event handlers to catch user requests to add/insert a new load or move
            frmNavigator.AddInsertLoadEvent += new EventHandler<AddInsertLoadEventArgs>(MainForm_AddInsertLoadEvent);
            frmNavigator.AddInsertMoveEvent += new EventHandler<AddInsertMoveEventArgs>(MainForm_AddInsertMoveEvent);

            //Event handlers to catch user requests to move or remove an existing load or move
            frmNavigator.MoveRemoveLoadEvent += new EventHandler<MoveRemoveLoadEventArgs>(MainForm_MoveRemoveLoadEvent);
            frmNavigator.MoveRemoveMoveEvent += new EventHandler<MoveRemoveMoveEventArgs>(MainForm_MoveRemoveMoveEvent);

            //Event handlers to set focus to the form that corresponds to the selected node
            frmNavigator.treeView_Navigator.AfterSelect += new TreeViewEventHandler(treeView_Navigator_AfterSelect);
            frmNavigator.treeView_Navigator.MouseDown += new MouseEventHandler(treeView_Navigator_MouseDown);
            
            ////Load layout data if an existing form configuration exists
            string configFile = Path.Combine(Path.GetDirectoryName(AppDomain.CurrentDomain.BaseDirectory), "DockPanel.config"); //Application.ExecutablePath

            if (File.Exists(configFile) == true)
            {
                try
                {
                    dockPanel.LoadFromXml(configFile, m_deserializeDockContent);
                }
                finally
                {
                    setDefaultLayout();
                }
            }
            else
                setDefaultLayout();

            //Create the navigator treeview 
            frmNavigator.CreateTreeView();

            //Add the application data node to the treeview 
            frmNavigator.AddApplicationDataNode(applicationData);

            //Allow the user to drag or reorder windows
            dockPanel.AllowEndUserDocking = true;

            //Initialize the printer object
            myDGVPrinter = new DGVPrinter();
        }

        void CreateAccessoriesList()
        {
            //Quick and dirty solution. Create one object for each accessory type. 
            //If the object contains a valid article number it will be displayed and saved

            Cable _encoderCable = new Cable(MainType.EncoderCable);
            iAccessories.Add(0, _encoderCable);

            Cable _motorCable = new Cable(MainType.MotorCable);
            iAccessories.Add(1, _motorCable); 
            
            Cable _brakeCable = new Cable(MainType.BrakeCable);
            iAccessories.Add(2, _brakeCable);

            Drive _drive = new Drive();
            iAccessories.Add(3, _drive);
            
            BrakeResistor _brakeResistor = new BrakeResistor();
            iAccessories.Add(4, _brakeResistor);

            Filter _EMCFilter = new Filter(MainType.EMCFilter);
            iAccessories.Add(5, _EMCFilter);

            Filter _DCFilter = new Filter(MainType.DCFilter);
            iAccessories.Add(6, _DCFilter);

            Option _battery = new Option(MainType.Battery);
            iAccessories.Add(7, _battery);

            Option _safety = new Option(MainType.Safety);
            iAccessories.Add(8, _safety);

            Option _IOCard = new Option(MainType.IOCard);
            iAccessories.Add(9, _IOCard);

            Option _IOCable = new Option(MainType.IOCable);
            iAccessories.Add(10, _IOCable);

            Option _busCable = new Option(MainType.BusCable);
            iAccessories.Add(11, _busCable);

            Option _controller = new Option(MainType.Controller);
            iAccessories.Add(12, _controller);

            Option _extensionCard = new Option(MainType.ExtensionCard);
            iAccessories.Add(13, _extensionCard);

            Option _extensionCable = new Option(MainType.ExtensionCable);
            iAccessories.Add(14, _extensionCable);

            Option _extensionTerminal = new Option(MainType.ExtensionTerminal);
            iAccessories.Add(15, _extensionTerminal);
        }

        void MainForm_AddInsertLoadEvent(object sender, AddInsertLoadEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            //Create a variable to store the list key
            int index = -1;

            //Check if it is an insert or add operation
            if (e.NodeName != "Loads") //= Insert
            {
                //Find the selected node in the sorted list 
                //= insertion point for the new load
                foreach (KeyValuePair<int, ILoad> pair in iLoads)
                {
                    if (pair.Value.Name == e.NodeName)
                        index = pair.Key;
                }

                //Display an error if the load was not found
                if ((index == -1))
                {
                    MessageBox.Show(
                        "Load not found.", "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation,
                    MessageBoxDefaultButton.Button1);

                    //Exit the method.
                    return;
                }

                //Make room for the load to be inserted

                //Create a temporary iLoad variable
                ILoad tempLoad = null;

                //If more loads exists then move these down one step
                //beginning with the last load
                for (int i = iLoads.Count - 1; i >= index; i--)
                {
                    //Save the data before manipulating the list
                    iLoads.TryGetValue(i, out tempLoad);

                    //Remove the load from the list
                    iLoads.Remove(i);

                    //Add the load to the list again with the new index
                    iLoads.Add(i + 1, tempLoad);
                }
            }
            else
                //Add the new load at the last position
                index = iLoads.Count;

            //Create a name based on the component type and a counter.
            //Start with the last number and check that the name does not exist.
            int count = iLoads.Count;

            count = FindNextLoadName(count, e.MainType.ToString());

            string name = e.MainType.ToString() + count.ToString();

            //Add the new load to the iLoads-list
            switch (e.MainType)
            {
                case MainType.BallScrew:
                    iLoads.Add(index, new BallScrew(name));
                    break;

                case MainType.RackAndPinion:
                    iLoads.Add(index, new RackAndPinion(name));
                    break;

                case MainType.ToothedBelt:
                    iLoads.Add(index, new ToothedBelt(name));
                    break;

                case MainType.TurnTable:
                    iLoads.Add(index, new TurnTable(name));
                    break;

                case MainType.Coupling:
                    iLoads.Add(index, new Coupling(name));
                    break;

                case MainType.GearBox:
                    iLoads.Add(index, new GearBox(name));
                    break;

                case MainType.Mass:
                    iLoads.Add(index, new LoadMass(name));
                    break;

                case MainType.Force:
                    iLoads.Add(index, new LoadForce(name));
                    break;

                case MainType.Torque:
                    iLoads.Add(index, new LoadTorque(name));
                    break;

                case MainType.Inertia:
                    iLoads.Add(index, new LoadInertia(name));
                    break;
            }

            //Attach load events and add the load to the navigator
            InitializeLoad(iLoads[index], index);

            //Update the ComboBox on the LoadProperties form
            frmProperties.ComboBox_ReLoad();

            //Set the added load as selected
            frmProperties.ComboBox_SelectLoad(iLoads[index].Name);

            //Recalculate each move based on the new data
            foreach (var m in moves.MovesList)
            {
                Move move = m.Value;

                //Update the active loads list on each move form
                move.UpdateLoadResults(iLoads);

                //Activate the inserted load on each move form
                move.ActivateLoadResult(index);

                //Update the LoadResults if this move form is the active tab (= visible to the user)
                FrmMove frm = (FrmMove)FindDocument(move.Name);

                if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                    //Recalculate all moves
                    move.Refresh = true;
            }
        }

        private void MainForm_MoveRemoveLoadEvent(object sender, MoveRemoveLoadEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            //Create a variable to store the list key
            int index = -1;

            //Create temporary move variables
            ILoad tempLoad1 = null;
            ILoad tempLoad2 = null;

            //Find the node in the sorted list
            foreach (KeyValuePair<int, ILoad> pair in iLoads)
                if (pair.Value.Name == e.NodeName)
                    index = pair.Key;


            if ((index == -1))
            {
                MessageBox.Show(
                    "Load not found", "Error",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button1);

                //Exit the method.
                return;
            }

            //Move node up
            if (e.MoveRemoveEvent == MoveRemoveEvent.MoveUp)
            {
                //Check if it is possible to move up)
                if (index > 0)
                {
                    //Save the data before manipulating the list
                    iLoads.TryGetValue(index - 1, out tempLoad1);
                    iLoads.TryGetValue(index, out tempLoad2);

                    //Remove the loads from the list
                    iLoads.Remove(index - 1);
                    iLoads.Remove(index);

                    //Remove the loads from the navigator
                    frmNavigator.RemoveLoadNode(tempLoad1.Name);
                    frmNavigator.RemoveLoadNode(tempLoad2.Name);

                    //Add the loads to the list again with the swapped data
                    iLoads.Add(index - 1, tempLoad2);
                    iLoads.Add(index, tempLoad1);

                    //Add the loads to the navigator
                    frmNavigator.AddLoadNode(index - 1, tempLoad1, tempLoad1.Name);
                    frmNavigator.AddLoadNode(index - 1, tempLoad2, tempLoad2.Name);

                    //Update the combobox on the LoadProperties form
                    frmProperties.ComboBox_ReLoad();

                    frmProperties.ComboBox_SelectLoad(tempLoad2.Name);

                    //Recalculate each move based on the new data
                    foreach (var m in moves.MovesList)
                    {
                        Move move = m.Value;

                        //Swap the active loads list on each move form
                        move.SwapLoadResults(index, index - 1);
                    }
                }
            }

            //Move node down
            if (e.MoveRemoveEvent == MoveRemoveEvent.MoveDown)
            {
                //Check if it is possible to move down)
                if (index < iLoads.Count - 1)
                {
                    //Save the data before manipulating the list
                    iLoads.TryGetValue(index, out tempLoad1);
                    iLoads.TryGetValue(index + 1, out tempLoad2);

                    //Remove the loads from the list
                    iLoads.Remove(index);
                    iLoads.Remove(index + 1);

                    //Remove the loads from the navigator
                    frmNavigator.RemoveLoadNode(tempLoad1.Name);
                    frmNavigator.RemoveLoadNode(tempLoad2.Name);

                    //Add the loads to the list again with the swapped data
                    iLoads.Add(index, tempLoad2);
                    iLoads.Add(index + 1, tempLoad1);

                    //Add the loads to the navigator
                    frmNavigator.AddLoadNode(index, tempLoad2, tempLoad2.Name);
                    frmNavigator.AddLoadNode(index + 1, tempLoad1, tempLoad1.Name);

                    //Update the ComboBox on the LoadProperties form
                    frmProperties.ComboBox_ReLoad();

                    frmProperties.ComboBox_SelectLoad(tempLoad1.Name);

                    //Recalculate each load based on the new data
                    foreach (var m in moves.MovesList)
                    {
                        Move move = m.Value;

                        //Swap the active loads list on each move form
                        move.SwapLoadResults(index, index + 1);
                    }
                }
            }

            //Remove node
            if (e.MoveRemoveEvent == MoveRemoveEvent.Remove)
            {
                //Remove the event handlers
                switch (iLoads[index].MainType)
                {
                    case (MainType.BallScrew):
                        BallScrew ballScrew = iLoads[index] as BallScrew;
                        ballScrew.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.RackAndPinion):
                        RackAndPinion rackAndPinion = iLoads[index] as RackAndPinion;
                        rackAndPinion.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.ToothedBelt):
                        ToothedBelt toothedBelt = iLoads[index] as ToothedBelt;
                        toothedBelt.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.TurnTable):
                        TurnTable turnTable = iLoads[index] as TurnTable;
                        turnTable.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.Coupling):
                        Coupling coupling = iLoads[index] as Coupling;
                        coupling.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.GearBox):
                        GearBox gearBox = iLoads[index] as GearBox;
                        gearBox.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.Mass):
                        LoadMass mass = iLoads[index] as LoadMass;
                        mass.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.Torque):
                        LoadTorque torque = iLoads[index] as LoadTorque;
                        torque.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.Inertia):
                        LoadInertia inertia = iLoads[index] as LoadInertia;
                        inertia.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;
                }

                //MainForm_RemoveILoad(index);
                iLoads.Remove(index);

                //Remove the load from the navigator
                frmNavigator.RemoveLoadNode(e.NodeName);

                //If more loads exist after then move these up one step
                //Check if the removed load was the last
                if (index < iLoads.Count)
                {
                    for (int i = index + 1; i < iLoads.Count + 1; i++)
                    {
                        //Save the data before manipulating the list
                        iLoads.TryGetValue(i, out tempLoad1);

                        //Remove the load from the list
                        iLoads.Remove(i);

                        //Add the load to the list again with the new index
                        iLoads.Add(i - 1, tempLoad1);
                    }
                }
            }

            //Recalculate each move based on the new data
            foreach (var m in moves.MovesList)
            {
                Move move = m.Value;

                //Update the active loads list on each move form
                move.UpdateLoadResults(iLoads);

                //Activate the inserted load on each move form
                move.ActivateLoadResult(index);

                //Update the LoadResults if this move form is the active tab (= visible to the user)
                FrmMove frm = (FrmMove)FindDocument(move.Name);

                if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                    //Recalculate all moves
                    move.Refresh = true;
            }

            //Update the combobox on the LoadProperties form
            frmProperties.ComboBox_ReLoad();

            if (e.MoveRemoveEvent == MoveRemoveEvent.Remove)
            {
                if (iLoads.Count > 0)
                    frmProperties.ComboBox_SelectLoad(iLoads[0].Name);
                else
                    frmProperties.Clear();
            }
            else
                frmProperties.ComboBox_SelectLoad(iLoads[index].Name);

            //Select the tree node that corresponds to the selected load
            if (iLoads.Count > 0)
                frmNavigator.SelectLoadNode(iLoads[0].Name);
        }

        private int FindNextLoadName(int count, string name)
        {
            string text;

            foreach (KeyValuePair<int, ILoad> pair in iLoads)
            {
                text = name + count.ToString();
                if (pair.Value.Name == text)
                {
                    count++;
                    count = FindNextLoadName(count, name);
                }
            }

            return count;
        }

        private void MainForm_AddInsertMoveEvent(object sender, AddInsertMoveEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            //Create a variable to store the list key
            int index = -1;

            //Create a variable for the form of the selected node
            FrmMove frmNode = null;

            //Check if it is an insert or add operation
            if (e.NodeName != "Moves") //= Insert
            {
                //Find the selected node in the sorted list 
                //= insertion point for the new load
                foreach (KeyValuePair<int, Move> pair in moves.MovesList)
                {
                    if (pair.Value.Name == e.NodeName)
                        index = pair.Key;
                }

                //Display an error if the move was not found
                if ((index == -1))
                {
                    MessageBox.Show(
                        "Move not found.", "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation,
                    MessageBoxDefaultButton.Button1);

                    //Exit the method.
                    return;
                }

                //Find the form of the selected move
                frmNode = (FrmMove)FindDocument(moves.MovesList[index].Name);

                //Make room for the move to be inserted

                //Create a temporary Move variable
                Move tempMove = null;

                //If more loads exists then move these down one step
                //beginning with the last move
                for (int i = moves.MovesList.Count - 1; i >= index; i--)
                {
                    //Save the data before manipulating the list
                    moves.MovesList.TryGetValue(i, out tempMove);

                    //Remove the move from the list
                    moves.MovesList.Remove(i);

                    //Add the move to the list again with the new index
                    moves.MovesList.Add(i + 1, tempMove);
                }
            }
            else
                //Add the new move at the last position
                index = moves.MovesList.Count;

            //Create a name based on movetype and a counter.
            //Start with the last number and check that the name does not exist.
            int count = moves.MovesList.Count;
            count = FindNextMoveName(count, e.MoveType.ToString());
            string name = e.MoveType.ToString() + count.ToString();

            Profile profile = Profile.Undefined;

            //Set Symmetric profile as default if MoveType = PreDefined
            if (e.MoveType == MoveType.PreDefined)
                profile = Profile.Symmetric;

            //Add a new move to the moves list
            moves.MovesList.Add(index, new Move(name, e.MoveType, profile));

            //Attach event handler, create treenode...
            initializeMove(moves.MovesList[index], index);
        }

        private void MainForm_MoveRemoveMoveEvent(object sender, MoveRemoveMoveEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            //Create a variable to store the list key
            int index = -1;

            //Create temporary move variables
            Move tempMove1 = null;
            Move tempMove2 = null;

            //Find the node in the sorted list
            foreach (KeyValuePair<int, Move> pair in moves.MovesList)
            {
                if (pair.Value.Name == e.NodeName)
                    index = pair.Key;
            }

            if ((index == -1))
            {
                MessageBox.Show(
                    "Move not found", "Error",
                MessageBoxButtons.OK,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button1);

                //Exit the method.
                return;
            }

            //Move node up
            if (e.MoveRemoveEvent == MoveRemoveEvent.MoveUp)
            {
                //Check if it is possible to move up)
                if (index > 0)
                {
                    //Save the data before manipulating the list
                    moves.MovesList.TryGetValue(index - 1, out tempMove1);
                    moves.MovesList.TryGetValue(index, out tempMove2);

                    //Remove the loads from the list
                    moves.MovesList.Remove(index - 1);
                    moves.MovesList.Remove(index);

                    //Remove the loads from the navigator
                    frmNavigator.RemoveMoveNode(tempMove1.Name);
                    frmNavigator.RemoveMoveNode(tempMove2.Name);

                    //Add the moves to the list again with the swapped data
                    moves.MovesList.Add(index - 1, tempMove2);
                    moves.MovesList.Add(index, tempMove1);

                    //Add the moves to the navigator
                    frmNavigator.AddMoveNode(index - 1, tempMove2, tempMove2.Name);

                    //Add the sector nodes to the application tree
                    foreach (Sector sector in tempMove2.SectorsList)
                        frmNavigator.AddSectorNode(sector, sector.Parent, sector.Name, sector.Index);

                    frmNavigator.AddMoveNode(index, tempMove1, tempMove1.Name);

                    //Add the sector nodes to the application tree
                    foreach (Sector sector in tempMove1.SectorsList)
                        frmNavigator.AddSectorNode(sector, sector.Parent, sector.Name, sector.Index);
                }
            }

            //Move node down
            if (e.MoveRemoveEvent == MoveRemoveEvent.MoveDown)
            {
                //Check if it is possible to move down)
                if (index < moves.MovesList.Count - 1)
                {
                    //Save the data before manipulating the list
                    moves.MovesList.TryGetValue(index, out tempMove1);
                    moves.MovesList.TryGetValue(index + 1, out tempMove2);

                    //Remove the loads from the list
                    moves.MovesList.Remove(index);
                    moves.MovesList.Remove(index + 1);

                    //Remove the loads from the navigator
                    frmNavigator.RemoveMoveNode(tempMove1.Name);
                    frmNavigator.RemoveMoveNode(tempMove2.Name);

                    //Add the moves to the list again with the swapped data
                    moves.MovesList.Add(index, tempMove2);
                    moves.MovesList.Add(index + 1, tempMove1);

                    //Add the moves to the navigator
                    frmNavigator.AddMoveNode(index, tempMove2, tempMove2.Name);

                    //Add the sector nodes to the application tree
                    foreach (Sector sector in tempMove2.SectorsList)
                        frmNavigator.AddSectorNode(sector, sector.Parent, sector.Name, sector.Index);

                    frmNavigator.AddMoveNode(index + 1, tempMove1, tempMove1.Name);

                    //Add the sector nodes to the application tree
                    foreach (Sector sector in tempMove1.SectorsList)
                        frmNavigator.AddSectorNode(sector, sector.Parent, sector.Name, sector.Index);
                }
            }

            //Remove node
            if (e.MoveRemoveEvent == MoveRemoveEvent.Remove)
            {
                //Find the form of the selected move
                FrmMove frmMove = (FrmMove)FindDocument(moves.MovesList[index].Name);

                //Close the form if it exists
                if (frmMove != null)
                    frmMove.Close();

                //Remove the event handlers
                moves.MovesList[index].PropertyChanged -= new PropertyChangedEventHandler(Move_PropertyChanged);
                moves.MovesList[index].SectorsList.CollectionChanged -= new NotifyCollectionChangedEventHandler(SectorsList_CollectionChanged);
                moves.MovesList[index].SectorPropertyChanged -= new PropertyChangedEventHandler(Sector_PropertyChanged);

                //Remove the load from the navigator
                frmNavigator.RemoveMoveNode(e.NodeName);

                //Remove the move from the MovesList
                moves.MovesList.Remove(index);

                //If more loads exist after then move these up one step
                //Check if the removed load was the last
                if (index < moves.MovesList.Count)
                {
                    for (int i = index + 1; i < moves.MovesList.Count + 1; i++)
                    {
                        //Save the data before manipulating the list
                        moves.MovesList.TryGetValue(i, out tempMove1);

                        //Remove the load from the list
                        moves.MovesList.Remove(i);

                        //Add the moves to the list again with the new index
                        moves.MovesList.Add(i - 1, tempMove1);
                    }
                }
            }

            //Clear all results if no moves exists
            if (moves.MovesList.Count == 0)
            {
                frmGraph.ClearJerkAccAndVelDistGraph();
                frmGraph.ClearTorqueAndRegenPwrGraph();
                frmTable.ClearTable();
            }
        }

        private int FindNextMoveName(int count, string name)
        {
            string text;

            foreach (KeyValuePair<int, Move> pair in moves.MovesList)
            {
                text = name + count.ToString();
                if (pair.Value.Name == text)
                {
                    count++;
                    count = FindNextMoveName(count, name);
                }
            }

            return count;
        }

        private void treeView_Navigator_MouseDown(object sender, MouseEventArgs e)
        {
            //Detect if the left mouse button is pressed and use the result in the treeView_AfterSelect-
            //event as a condition to highlight the selected item.
            //This event is needed to make the context menus work correctly.

            if (e.Button == MouseButtons.Left)
                leftMouseButton = true;
        }

        private void treeView_Navigator_AfterSelect(Object sender, TreeViewEventArgs e)
        {
            if (leftMouseButton == true)
            {
                if (e.Node.Name != null)
                {
                    IDockContent frm = null;

                    //Application data
                    if (e.Node.Name == "OverView")
                    {
                        frm = FindDocument("OverView");

                        if (frm != null)
                            frmOverview.Show();
                        else
                        {
                            frmOverview = null;
                            frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
                            frmOverview.Show(dockPanel, DockState.Document);
                        }
                    }

                    //Accessories
                    if (e.Node.Name == "Accessories")
                    {
                        frm = FindDocument("Accessories");

                        if (frm != null)
                            frmAccessories.Show();
                        else
                        {
                            frmAccessories = null;
                            frmAccessories = new FrmAccessories(motor, iAccessories, moves);
                            frmAccessories.Show(dockPanel, DockState.Document);
                        }

                        //Update the ComboBox on the LoadProperties form
                        frmProperties.ComboBox_ReLoad();

                        //Set the added load as selected
                        frmProperties.ComboBox_SelectLoad(iAccessories[0].Name);
                    }

                    //Results
                    if (e.Node.Name == "Results")
                    {
                        {
                            frm = FindDocument("Results");

                            if (frm != null)
                                frmResults.Show();
                            else
                            {
                                frmResults = null;
                                frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                                frmResults.Show(dockPanel, DockState.Document);
                            }
                        }
                    }

                    //Move
                    foreach (KeyValuePair<int, Move> m in moves.MovesList)
                    {
                        if (m.Value.Name == e.Node.Name)
                        {
                            frm = FindDocument(e.Node.Name);

                            if (frm != null)
                                frm.DockHandler.Show();
                            else
                            {
                                Move move = m.Value as Move;

                                FrmMove frmMove = new FrmMove(move, iLoads, motor);
                                frmMove.Text = e.Node.Name;
                                frmMove.Show(dockPanel, DockState.Document);

                                //Update the sector table and graph window
                                move.Refresh = true;
                            }
                        }
                    }

                    //Load
                    foreach (KeyValuePair<int, ILoad> pair in iLoads)
                    {
                        if (pair.Value.Name == e.Node.Name)
                            if (frmProperties.IsHidden == false)
                                frmProperties.ComboBox_SelectLoad(e.Node.Name);
                            else
                                frmProperties.Show(dockPanel);
                    }

                    //Motor
                    if (e.Node.Name == "Motor")
                        frmProperties.ComboBox_SelectLoad(e.Node.Name);
                }
            }

            //Set mouse click flag to false to enable a new check
            leftMouseButton = false;
        }

        private void ApplicationData_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            //Find the parent node for all _applicationData properties
            TreeNode parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "ApplicationData");

            switch (e.PropertyName)
            {
                case "AppName":
                    TreeNode childNode = frmNavigator.FindNode(parentNode, "AppName");
                    childNode.Text = "AppName: " + applicationData.AppName;
                    break;

                case "Date":
                    childNode = frmNavigator.FindNode(parentNode, "Date");
                    childNode.Text = "Date: " + applicationData.Date;
                    break;

                case "Customer":
                    childNode = frmNavigator.FindNode(parentNode, "Customer");
                    childNode.Text = "Customer: " + applicationData.Customer;
                    break;

                case "OurRef":
                    childNode = frmNavigator.FindNode(parentNode, "OurRef");
                    childNode.Text = "OurRef: " + applicationData.OurRef;
                    break;

                case "YourRef":
                    childNode = frmNavigator.FindNode(parentNode, "YourRef");
                    childNode.Text = "YourRef: " + applicationData.YourRef;
                    break;
            }
        }

        private void Load_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            ILoad iLoad = (ILoad)sender;

            //Find the parent node for all _load properties.
            //Note: The treeView is only called when renaming loads so it finds the OLD node name
            TreeNode parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "Loads");
            parentNode = frmNavigator.FindNode(parentNode, iLoad.OldName);

            switch (e.PropertyName)
            {
                case "Name":
                    parentNode.Name = iLoad.Name;
                    parentNode.Text = iLoad.Name;
                    break;

                case "AutoSelect":
                    if (iLoad.MainType == MainType.GearBox)
                    {
                        GearBox gearBox = iLoad as GearBox;
                        gearBox.SetOrResetReadOnlyAttributes(gearBox);

                        if (gearBox.AutoSelect && !disableUpdate)
                        {
                            FrmResults frmResults = (FrmResults)FindDocument("Results");

                            if (frmResults == dockPanel.ActiveDocumentPane.ActiveContent)
                            {
                                //If any move results exists then update the graph and results table               
                                if (iLoads.Count > 0)
                                {
                                    frmResults.CalculateMovesResults();
                                    frmResults.DisplayMovesResults();
                                }
                            }
                            else
                                foreach (var m in moves.MovesList)
                                {
                                    Move move = m.Value;

                                    //Update the LoadResults if this move form is the active tab (= visible to the user)
                                    FrmMove frm = (FrmMove)FindDocument(move.Name);

                                    if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                                        if (iLoads.Count > 0)
                                            frm.CreateLoadResultsTable();
                                }
                        }
                    }
                    break;

                case "ArticleNo":
                    if (iLoad.MainType == MainType.GearBox)
                    {
                        GearBox gearBox = iLoad as GearBox;
                        ReadGearBoxesFromdB(gearBox, gearBox.ArticleNo);
                    }
                    break;
            }

            //Do not recalculate automatically if the load is system defined and is changed while searching
            if (!iLoad.AutoSelect && !disableUpdate)
            {
                foreach (var m in moves.MovesList)
                {
                    Move move = m.Value;

                    //Update the LoadResults if this move form is the active tab (= visible to the user)
                    FrmMove frm = (FrmMove)FindDocument(move.Name);

                    if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                    {
                        if (iLoads.Count > 0)
                        {
                            move.CalcLoadResults(iLoads);
                            frm.CalcMotorResults();
                            frm.CreateLoadResultsTable();

                            bool validTorqueResults = false;

                            if (move.TotalLoadResults.PeakTrq != 0)
                                validTorqueResults = true;

                            if (validTorqueResults == true)
                            {
                                double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                                frmGraph.AddMotorSpeedToVelGraph(move.GetAllSectorElements(), scale, motor.NomSpd, motor.MaxSpd);
                            }
                        }

                        frmGraph.CreateTorqueGraph(move.GetAllSectorElements(), motor.NomTrq, motor.MaxTrq);
                        frmGraph.CreateTorquePerComponentGraph(move.MotorResults, move.LoadResultsList);

                        frmTable.CreateTable(move.GetAllSectorElements());
                    }
                }

                FrmResults frmResults = (FrmResults)FindDocument("Results");

                if (frmResults == dockPanel.ActiveDocumentPane.ActiveContent)
                {
                    //If any move results exists then update the graph and results table               
                    if (iLoads.Count > 0)
                    {
                        frmResults.CalculateMovesResults();
                        frmResults.DisplayMovesResults();

                        moves.AddAllMotionProfiles(moves.MovesList);

                        frmGraph.CreateJerkAccAndVelDistGraphs(moves.ActiveSectorElementsList);

                        bool validTorqueResults = false;

                        if (moves.LoadResultsList[0].PeakTrq != 0) //ToDo
                            validTorqueResults = true;

                        if (validTorqueResults == true)
                        {
                            double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                            frmGraph.AddMotorSpeedToVelGraph(moves.ActiveSectorElementsList, scale, motor.NomSpd, motor.MaxSpd);
                        }
                    }

                    frmGraph.CreateTorqueGraph(moves.ActiveSectorElementsList, motor.NomTrq, motor.MaxTrq);
                    frmGraph.CreateTorquePerComponentGraph(moves.MotorResults, moves.LoadResultsList);

                    frmTable.CreateTable(moves.ActiveSectorElementsList);
                }
            }
        }

        private void Move_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            Move move = (Move)sender;

            //Find the parent node for all _move properties
            TreeNode parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "Moves");
            parentNode = frmNavigator.FindNode(parentNode, move.Name);

            switch (e.PropertyName)
            {
                case "Name":
                    //Special case: When renaming, find the OLD node name 
                    parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "Moves");
                    parentNode = frmNavigator.FindNode(parentNode, move.OldName);

                    parentNode.Name = move.Name;
                    parentNode.Text = move.Name;

                    //Find the form of the selected move
                    FrmMove frmMove = (FrmMove)FindDocument(move.OldName);

                    //Rename it if it exists
                    if (frmMove != null)
                        frmMove.Text = move.Name;

                    return;

                case "MoveType":
                    TreeNode childNode = frmNavigator.FindNode(parentNode, "MoveType");
                    childNode.Text = "MoveType: " + move.MoveType.ToString();
                    break;

                case "Profile":
                    childNode = frmNavigator.FindNode(parentNode, "Profile");
                    childNode.Text = "Profile: " + move.Profile;
                    break;

                case "Time":
                    childNode = frmNavigator.FindNode(parentNode, "Time");
                    childNode.Text = "Time: " + move.Time;
                    break;

                case "Distance":
                    childNode = frmNavigator.FindNode(parentNode, "Distance");
                    childNode.Text = "Distance: " + move.Distance;
                    break;

                case "LoadResultsCalculated":
                    //These nodenames must be updated!!!
                    parentNode = frmNavigator.FindNode(parentNode, "LoadResults");

                    childNode = frmNavigator.FindNode(parentNode, "TotalGearing");
                    childNode.Text = "TotalGearing: " + move.TotalLoadResults.AccOutGearing.ToString() + " :1";

                    childNode = frmNavigator.FindNode(parentNode, "PeakSpeed");
                    childNode.Text = "PeakSpeed: " + move.TotalLoadResults.PeakSpd.ToString() + " m/s";

                    childNode = frmNavigator.FindNode(parentNode, "RmsSpeed");
                    childNode.Text = "RmsSpeed: " + move.TotalLoadResults.RmsSpdED.ToString() + " m/s";

                    string movePerRevUnit = " mm";

                    if (move.TotalLoadResults.InpMovePerRevUnitmm == false)
                        movePerRevUnit = " ";

                    childNode = frmNavigator.FindNode(parentNode, "MovePerRev");
                    childNode.Text = "MovePerRev: " + move.TotalLoadResults.InpMovePerRev.ToString() + movePerRevUnit;

                    childNode = frmNavigator.FindNode(parentNode, "Backlash");
                    childNode.Text = "TotalBacklash: " + move.TotalLoadResults.Backlash.ToString() + " arcmin";

                    childNode = frmNavigator.FindNode(parentNode, "TotalEfficiency");
                    childNode.Text = "TotalEfficiency: " + move.TotalLoadResults.Efficiency.ToString() + " %";

                    childNode = frmNavigator.FindNode(parentNode, "TotalInertia");
                    childNode.Text = "TotalInertia: " + move.TotalLoadResults.Inertia.ToString() + " kgcm";

                    childNode = frmNavigator.FindNode(parentNode, "TotalConstTorque");
                    childNode.Text = "TotalConstTorque: " + move.TotalLoadResults.ConstTorque.ToString() + " Nm";

                    childNode = frmNavigator.FindNode(parentNode, "TotalAccTorque");
                    childNode.Text = "TotalAccTorque: " + move.TotalLoadResults.AccTorque.ToString() + " Nm";

                    childNode = frmNavigator.FindNode(parentNode, "TotalDecTorque");
                    childNode.Text = "TotalDecTorque: " + move.TotalLoadResults.DecTorque.ToString() + " Nm";

                    childNode = frmNavigator.FindNode(parentNode, "TotalPeakTorque");
                    childNode.Text = "TotalPeakTorque: " + move.TotalLoadResults.PeakTrq.ToString() + " Nm";

                    childNode = frmNavigator.FindNode(parentNode, "TotalRmsTorque");
                    childNode.Text = "TotalRmsTorque: " + move.TotalLoadResults.RmsTrqED.ToString() + " Nm";
                    break;

                case "Refresh":
                    move.Refresh = false;
                    break;
            }


            //Find the form of the selected move
            FrmMove frm = (FrmMove)FindDocument(move.Name);

            move.CalcMoveResults();

            frm.UpdateMoveResults();

            frmGraph.CreateJerkAccAndVelDistGraphs(move.GetAllSectorElements());

            if (iLoads.Count > 0)
            {
                move.CalcLoadResults(iLoads);
                frm.CalcMotorResults();
                frm.CreateLoadResultsTable();

                bool validTorqueResults = false;

                if (move.TotalLoadResults.PeakTrq != 0)
                    validTorqueResults = true;

                if (validTorqueResults == true)
                {
                    double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                    frmGraph.AddMotorSpeedToVelGraph(move.GetAllSectorElements(), scale, motor.NomSpd, motor.MaxSpd);
                }
            }

            frmGraph.CreateTorqueGraph(move.GetAllSectorElements(), motor.NomTrq, motor.MaxTrq);
            frmGraph.CreateTorquePerComponentGraph(move.MotorResults, move.LoadResultsList);

            frmTable.CreateTable(move.GetAllSectorElements());
        }

        private void Moves_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            Moves moves = (Moves)sender;

            if (moves.Refresh)
                moves.Refresh = false;

            if (!disableUpdate)
            {
                //If any move results exists then update the graph and results table               
                if (iLoads.Count > 0)
                {
                    frmResults.CalculateMovesResults();
                    frmResults.DisplayMovesResults();

                    moves.AddAllMotionProfiles(moves.MovesList);

                    frmGraph.CreateJerkAccAndVelDistGraphs(moves.ActiveSectorElementsList);

                    bool validTorqueResults = false;

                    if (moves.LoadResultsList[0].PeakTrq != 0) //ToDo
                        validTorqueResults = true;

                    if (validTorqueResults == true)
                    {
                        double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                        frmGraph.AddMotorSpeedToVelGraph(moves.ActiveSectorElementsList, scale, motor.NomSpd, motor.MaxSpd);
                    }
                }

                frmGraph.CreateTorqueGraph(moves.ActiveSectorElementsList, motor.NomTrq, motor.MaxTrq);
                frmGraph.CreateTorquePerComponentGraph(moves.MotorResults, moves.LoadResultsList);

                frmTable.CreateTable(moves.ActiveSectorElementsList);
            }
        }

        void SectorsList_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            //Add or remove a sector node in the application tree
            IList<Sector> sectorList = (IList<Sector>)sender;

            if (e.NewItems != null)
            {
                Sector sector = sectorList[e.NewStartingIndex];
                frmNavigator.AddSectorNode(sector, sector.Parent, sector.Name, sector.Index);

                foreach (Sector item in e.NewItems)
                    item.PropertyChanged += Sector_PropertyChanged;
            }

            if (e.OldItems != null)
            {
                Sector sector = sectorList[e.OldStartingIndex];
                frmNavigator.RemoveSectorNode(sector.Parent, e.OldStartingIndex);

                foreach (Sector item in e.OldItems)
                    item.PropertyChanged -= Sector_PropertyChanged;
            }
        }

        private void Sector_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            Sector sector = (Sector)sender;

            //Find the parent node for all _move properties
            TreeNode parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "Moves");

            //Find the parent node for the current sector
            parentNode = frmNavigator.FindNode(parentNode, sector.Parent);

            //Find the current sector
            parentNode = frmNavigator.FindNode(parentNode, sector.Name);

            switch (e.PropertyName)
            {
                case "Name":
                    //Special case: When renaming, find the OLD node name 
                    parentNode = frmNavigator.FindNode(frmNavigator.treeView_Navigator.Nodes[0], "Moves");
                    parentNode = frmNavigator.FindNode(parentNode, sector.Parent);
                    parentNode = frmNavigator.FindNode(parentNode, sector.OldName);

                    parentNode.Name = sector.Name;
                    parentNode.Text = sector.Name;
                    break;

                case "Distance":
                    TreeNode childNode = frmNavigator.FindNode(parentNode, "Distance");
                    childNode.Text = "Distance: " + sector.Distance.ToString() + " m";
                    break;

                case "Time":
                    childNode = frmNavigator.FindNode(parentNode, "Time");
                    childNode.Text = "Time: " + sector.Time.ToString() + " s";
                    break;

                case "AverageAcc":
                    childNode = frmNavigator.FindNode(parentNode, "AverageAcc");
                    childNode.Text = "AverageAcc: " + sector.AverageAcc + " m/s";
                    break;

                case "InitialSpeed":
                    childNode = frmNavigator.FindNode(parentNode, "InitialSpeed");
                    childNode.Text = "InitialSpeed: " + sector.InitialSpeed + " m/s";
                    break;

                case "FinalSpeed":
                    childNode = frmNavigator.FindNode(parentNode, "FinalSpeed");
                    childNode.Text = "FinalSpeed: " + sector.FinalSpeed + " m/s";
                    break;

                case "PercentSineVel":
                    childNode = frmNavigator.FindNode(parentNode, "PercentSineVel");
                    childNode.Text = "PercentSineVel: " + sector.PercentSineVel.ToString() + " %";
                    break;

                case "PercentSineAcc":
                    childNode = frmNavigator.FindNode(parentNode, "PercentSineAcc");
                    childNode.Text = "PercentSineAcc: " + sector.PercentSineAcc.ToString() + " %";
                    break;

                case "AverageSpeed":
                    childNode = frmNavigator.FindNode(parentNode, "AverageSpeed");
                    childNode.Text = "AverageSpeed: " + sector.AverageSpeed.ToString() + " m/s";
                    break;

                case "PeakAcc":
                    childNode = frmNavigator.FindNode(parentNode, "PeakAcc");
                    childNode.Text = "PeakAcc: " + sector.PeakAcc.ToString() + " m/s";
                    break;

                case "ChangingAccTime":
                    childNode = frmNavigator.FindNode(parentNode, "ChangingAccTime");
                    childNode.Text = "ChangingAccTime: " + sector.ChangingAccTime.ToString() + " s";
                    break;

                case "AverageJerk":
                    childNode = frmNavigator.FindNode(parentNode, "AverageJerk");
                    childNode.Text = "AverageJerk: " + sector.AverageJerk.ToString() + " m/s";
                    break;

                case "PeakJerk":
                    childNode = frmNavigator.FindNode(parentNode, "PeakJerk");
                    childNode.Text = "PeakJerk: " + sector.PeakJerk.ToString() + " m/s";
                    break;

                case "ChangingJerkTime":
                    childNode = frmNavigator.FindNode(parentNode, "ChangingJerkTime");
                    childNode.Text = "ChangingJerkTime: " + sector.ChangingJerkTime.ToString() + " s";
                    break;
            }
        }

        private void Motor_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //Set flag to indicate that unsaved changes exists
            unsavedData = true;

            if (e.PropertyName == "ArticleNo")
                ReadMotorFromdB(motor.ArticleNo);

            if (e.PropertyName == "AutoSelect")
            {
                motor.SetOrResetReadOnlyAttributes(motor);

                if (motor.AutoSelect && !disableUpdate)
                {
                    FrmResults frmResults = (FrmResults)FindDocument("Results");

                    if (frmResults == dockPanel.ActiveDocumentPane.ActiveContent)
                    {
                        //If any move results exists then update the graph and results table               
                        if (iLoads.Count > 0)
                        {
                            frmResults.CalculateMovesResults();
                            frmResults.DisplayMovesResults();
                        }
                    }
                    else
                        foreach (var m in moves.MovesList)
                        {
                            Move move = m.Value;

                            //Update the LoadResults if this move form is the active tab (= visible to the user)
                            FrmMove frm = (FrmMove)FindDocument(move.Name);

                            if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                                if (iLoads.Count > 0)
                                    frm.CreateLoadResultsTable();
                        }
                }
            }

            //Do not recalculate automatically if the load is system defined
            //and is changed while searching
            if (!motor.AutoSelect && !disableUpdate)
            {
                foreach (var m in moves.MovesList)
                {
                    Move move = m.Value;

                    //Update the LoadResults if this move form is the active tab (= visible to the user)
                    FrmMove frm = (FrmMove)FindDocument(move.Name);

                    if (frm == dockPanel.ActiveDocumentPane.ActiveContent)
                    {
                        if (iLoads.Count > 0)
                        {
                            move.CalcLoadResults(iLoads);
                            frm.CalcMotorResults();
                            frm.CreateLoadResultsTable();

                            bool validTorqueResults = false;

                            if (move.TotalLoadResults.PeakTrq != 0)
                                validTorqueResults = true;

                            if (validTorqueResults == true)
                            {
                                double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                                frmGraph.AddMotorSpeedToVelGraph(move.GetAllSectorElements(), scale, motor.NomSpd, motor.MaxSpd);
                            }
                        }

                        frmGraph.CreateTorqueGraph(move.GetAllSectorElements(), motor.NomTrq, motor.MaxTrq);
                        frmGraph.CreateTorquePerComponentGraph(move.MotorResults, move.LoadResultsList);

                        frmTable.CreateTable(move.GetAllSectorElements());
                    }
                }

                FrmResults frmResults = (FrmResults)FindDocument("Results");

                if (frmResults == dockPanel.ActiveDocumentPane.ActiveContent)
                {
                    //If any move results exists then update the graph and results table               
                    if (iLoads.Count > 0)
                    {
                        frmResults.CalculateMovesResults();
                        frmResults.DisplayMovesResults();

                        frmGraph.CreateJerkAccAndVelDistGraphs(moves.ActiveSectorElementsList);

                        bool validTorqueResults = false;

                        if (moves.LoadResultsList[0].PeakTrq != 0) //ToDo
                            validTorqueResults = true;

                        if (validTorqueResults == true)
                        {
                            double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                            frmGraph.AddMotorSpeedToVelGraph(moves.ActiveSectorElementsList, scale, motor.NomSpd, motor.MaxSpd);
                        }
                    }

                    frmGraph.CreateTorqueGraph(moves.ActiveSectorElementsList, motor.NomTrq, motor.MaxTrq);
                    frmGraph.CreateTorquePerComponentGraph(moves.MotorResults, moves.LoadResultsList);

                    frmTable.CreateTable(moves.ActiveSectorElementsList);
                }
            }
        }

        private void ReadMotorFromdB(string articleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";  //Path.GetFullPath
            //(@"C:\DriveSystems.mdb");

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read the data from the Motors table
                    var qryMotor =
                        from tblMotors myMotors in dbDriveSystemsContext.tblMotors
                        where myMotors.ArticleNo == articleNo
                        select myMotors;

                    disableUpdate = true;

                    //Load the motor object with its data
                    foreach (var dBMotor in qryMotor) //Should only be one item
                    {
                        motor.ArticleNo = dBMotor.ArticleNo;

                        motor.Brand = Brand.BSD;

                        motor.Description = dBMotor.Description;

                        if (dBMotor.Braked == "Yes")
                            motor.Braked = true;
                        else
                            motor.Braked = false;

                        switch (dBMotor.SubType)
                        {
                            case ("ServoMotor"):
                                motor.SubType = MotorType.ServoMotor;
                                break;

                            default:
                                motor.SubType = MotorType.Undefined;
                                break;
                        }
  
                        motor.DrivePrice = dBMotor.DrivePrice;

                        motor.Price = dBMotor.Price;
                        //Main data
                        motor.Resolution = dBMotor.Resolution;
                        motor.InertiaRatio = dBMotor.InertiaRatio;
                        motor.Inertia = dBMotor.Inertia / 10000;
                        motor.Eff = dBMotor.Eff;
                        motor.FldWeakSpd = dBMotor.FldWeakSpd;

                        //Mechanical output
                        switch (dBMotor.OutType)
                        {
                            case "Flange":
                                motor.OutType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                motor.OutType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                motor.OutType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                motor.OutType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                motor.OutType = MechInterface.SmoothShaft;
                                break;

                            default:
                                motor.OutType = MechInterface.Undefined;
                                break;
                        }

                        motor.OutMinDia = dBMotor.OutMinDia;
                        motor.OutMaxDia = dBMotor.OutMaxDia;
                        motor.OutRadFrc = dBMotor.OutRadFrc;
                        motor.OutAxFrc = dBMotor.OutAxFrc;

                        //Torque data
                        motor.MaxTrq = dBMotor.MaxTrq;
                        motor.MaxTrqDur = dBMotor.MaxTrqDur;
                        motor.TrqED = dBMotor.TrqED;
                        motor.NomTrq = dBMotor.NomTrq;

                        //Speed data
                        motor.MaxSpd = dBMotor.MaxSpd;
                        motor.MaxSpdDur = dBMotor.MaxSpdDur;
                        motor.SpdED = dBMotor.SpdED;

                        disableUpdate = false;

                        motor.NomSpd = dBMotor.NomSpd;
                    }
                }
            }
        }

        private void ReadGearBoxesFromdB(GearBox gearBox, string articleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb"; //Path.GetFullPath
            //(@"C:\DriveSystems.mdb");

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read the data from the GearBoxes table
                    var qryGearBox =
                        from tblGearBoxes myGearBoxes in dbDriveSystemsContext.tblGearBoxes
                        where myGearBoxes.ArticleNo == articleNo
                        select myGearBoxes;

                    disableUpdate = true;

                    //Load the motor object with its data
                    foreach (var dBGearBox in qryGearBox) //Should only be one item
                    {
                        //Get the values for the calculation

                        #region SetGearBoxData

                        //Component data
                        gearBox.ArticleNo = dBGearBox.ArticleNo;

                        switch (dBGearBox.Brand)
                        {
                            case ("Wittenstein"):
                                gearBox.Brand = GearBoxBrand.Wittenstein;
                                break;

                            case ("LSIS"):
                                gearBox.Brand = GearBoxBrand.LSIS;
                                break;

                            default:
                                gearBox.Brand = GearBoxBrand.Undefined;
                                break;
                        }

                        switch (dBGearBox.SubType)
                        {
                            case ("PlanetaryStraight"):
                                gearBox.SubType = GearBoxType.PlanetaryStraight;
                                break;

                            case ("PlanetaryRightAngle"):
                                gearBox.SubType = GearBoxType.PlanetaryRightAngle;
                                break;

                            case ("WormGear"):
                                gearBox.SubType = GearBoxType.WormGear;
                                break;

                            default:
                                gearBox.SubType = GearBoxType.Undefined;
                                break;
                        }

                        gearBox.Description = dBGearBox.Description;
                        gearBox.Price = dBGearBox.Price;

                        //Main data
                        gearBox.Gearing = dBGearBox.Gearing;
                        gearBox.IdleTrq = dBGearBox.IdleTrq;
                        gearBox.Inertia = dBGearBox.Inertia / 10000;
                        gearBox.Eff = dBGearBox.Eff;
                        gearBox.Backlash = dBGearBox.Backlash;

                        //Mechanical input
                        switch (dBGearBox.InpType)
                        {
                            case "Flange":
                                gearBox.InpType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                gearBox.InpType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                gearBox.InpType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                gearBox.InpType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                gearBox.InpType = MechInterface.SmoothShaft;
                                break;

                            default:
                                gearBox.InpType = MechInterface.Undefined;
                                break;
                        }

                        gearBox.InpMinDia = dBGearBox.InpMinDia;
                        gearBox.InpMaxDia = dBGearBox.InpMaxDia;
                        gearBox.InpRadFrc = dBGearBox.InpRadFrc;
                        gearBox.InpAxFrc = dBGearBox.InpAxFrc;

                        //Mechanical output
                        switch (dBGearBox.OutType)
                        {
                            case "Flange":
                                gearBox.OutType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                gearBox.OutType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                gearBox.OutType = MechInterface.KeyedHollow;
                                break;

                            case "KeyedShaft":
                                gearBox.OutType = MechInterface.KeyedShaft;
                                break;

                            case "SmoothShaft":
                                gearBox.OutType = MechInterface.SmoothShaft;
                                break;

                            default:
                                gearBox.OutType = MechInterface.Undefined;
                                break;
                        }

                        gearBox.OutMinDia = dBGearBox.OutMinDia;
                        gearBox.OutMaxDia = dBGearBox.OutMaxDia;
                        gearBox.OutRadFrc = dBGearBox.OutRadFrc;
                        gearBox.OutAxFrc = dBGearBox.OutAxFrc;

                        //Torque data
                        gearBox.MaxTrq = dBGearBox.MaxTrq;
                        gearBox.MaxTrqDur = dBGearBox.MaxTrqDur;
                        gearBox.TrqED = dBGearBox.TrqED;
                        gearBox.NomTrq = dBGearBox.NomTrq;

                        //Speed data
                        gearBox.MaxSpd = dBGearBox.MaxSpd;
                        gearBox.MaxSpdDur = dBGearBox.MaxSpdDur;
                        gearBox.SpdED = dBGearBox.SpdED;

                        disableUpdate = false;

                        gearBox.NomSpd = dBGearBox.NomSpd;

                        #endregion

                    }

                    disableUpdate = false;
                }
            }
        }

        private void CloseAll()
        {
            //Clear all application data
            applicationData.AppName = "";
            applicationData.Customer = "";
            applicationData.Date = "";
            applicationData.Description = "";
            applicationData.OurRef = "";
            applicationData.YourRef = "";

            //Clear all motor data
            ResetSystemMotor();

            //Remove all moves
            for (int moveNr = moves.MovesList.Count - 1; moveNr >= 0; moveNr-- )
            {
                //Find the form of the selected move
                FrmMove frmMove = (FrmMove)FindDocument(moves.MovesList[moveNr].Name);

                //Close the form if it exists
                if (frmMove != null)
                    frmMove.Close();

                //Remove the event handlers
                moves.MovesList[moveNr].PropertyChanged -= new PropertyChangedEventHandler(Move_PropertyChanged);
                moves.MovesList[moveNr].SectorsList.CollectionChanged -= new NotifyCollectionChangedEventHandler(SectorsList_CollectionChanged);
                moves.MovesList[moveNr].SectorPropertyChanged -= new PropertyChangedEventHandler(Sector_PropertyChanged);

                //Remove the load from the navigator
                frmNavigator.RemoveMoveNode(moves.MovesList[moveNr].Name);

                //Remove the move
                moves.MovesList[moveNr] = null;

                //Remove the move from the MovesList
                moves.MovesList.Remove(moveNr);
            }

            //moves.ActiveSectorElementsList.Clear();
            //moves.MovesList.Clear();
            //moves.MotorResults = null;
            //moves.LoadResultsList.Clear();

            //Remove all loads
            for (int loadNr = iLoads.Count - 1; loadNr >= 0; loadNr--)
            {
                //Remove the load from the navigator
                frmNavigator.RemoveLoadNode(iLoads[loadNr].Name);

                //Remove the event handlers
                switch (iLoads[loadNr].MainType)
                {
                    case (MainType.BallScrew):
                        BallScrew ballScrew = iLoads[loadNr] as BallScrew;
                        ballScrew.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        ballScrew.RefreshPropertyGridEvent -= new EventHandler<RefreshPropertyGridEventArgs>(ballScrew_RefreshPropertyGridEvent);   
                        break;

                    case (MainType.RackAndPinion):
                        RackAndPinion rackAndPinion = iLoads[loadNr] as RackAndPinion;
                        rackAndPinion.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        rackAndPinion.RefreshPropertyGridEvent -= new EventHandler<RefreshPropertyGridEventArgs>(rackAndPinion_RefreshPropertyGridEvent);
                        break;

                    case (MainType.ToothedBelt):
                        ToothedBelt toothedBelt = iLoads[loadNr] as ToothedBelt;
                        toothedBelt.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        toothedBelt.RefreshPropertyGridEvent -=new EventHandler<RefreshPropertyGridEventArgs>(toothedBelt_RefreshPropertyGridEvent);
                        break;

                    case (MainType.TurnTable):
                        TurnTable turnTable = iLoads[loadNr] as TurnTable;
                        turnTable.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        turnTable.RefreshPropertyGridEvent -= new EventHandler<RefreshPropertyGridEventArgs>(turnTable_RefreshPropertyGridEvent);
                        break;

                    case (MainType.Coupling):
                        Coupling coupling = iLoads[loadNr] as Coupling;
                        coupling.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        coupling.RefreshPropertyGridEvent -=new EventHandler<RefreshPropertyGridEventArgs>(coupling_RefreshPropertyGridEvent);
                        break;

                    case (MainType.GearBox):
                        GearBox gearBox = iLoads[loadNr] as GearBox;
                        gearBox.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        break;

                    case (MainType.Mass):
                        LoadMass mass = iLoads[loadNr] as LoadMass;
                        mass.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        //mass = null;
                        break;

                    case (MainType.Force):
                        LoadForce force = iLoads[loadNr] as LoadForce;
                        force.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        //mass = null;
                        break;

                    case (MainType.Torque):
                        LoadTorque torque = iLoads[loadNr] as LoadTorque;
                        torque.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        //torque = null;
                        break;

                    case (MainType.Inertia):
                        LoadInertia inertia = iLoads[loadNr] as LoadInertia;
                        inertia.PropertyChanged -= new PropertyChangedEventHandler(Load_PropertyChanged);
                        inertia.RefreshPropertyGridEvent -= new EventHandler<RefreshPropertyGridEventArgs>(inertia_RefreshPropertyGridEvent);
                        break;
                }

                iLoads.Remove(loadNr);
            }

            iLoads.Clear();

            //Clear all results if no moves exists
            frmGraph.ClearJerkAccAndVelDistGraph();
            frmGraph.ClearTorqueAndRegenPwrGraph();
            frmGraph.ClearTorquePerComponentsGraph();
            frmTable.ClearTable();

            frmResults.ClearTables();

            frmProperties.Clear();

            //File name
            mainFileName = null;

            this.Text = "BSD Drive Sizer - ";

            disableUpdate = false;
        }

        private void ResetSystemMotor()
        {
            //Clear all motor properties
            #region ClearMotorData
            motor.ArticleNo = "";

            motor.Brand = Brand.BSD;

            motor.Description = "Not selected";
            // ToDo: motor. = dBMotor.Type;
            motor.Price = 0;

            //Main data
            motor.Resolution = 0;
            motor.InertiaRatio = 0;
            motor.Inertia = 0;
            motor.Eff = 100;
            motor.FldWeakSpd = 0;

            //Mechanical output
            motor.OutType = MechInterface.Undefined;

            motor.OutMinDia = 0;
            motor.OutMaxDia = 0;
            motor.OutRadFrc = 0;
            motor.OutAxFrc = 0;

            //Torque data
            motor.MaxTrq = 0;
            motor.MaxTrqDur = 0;
            motor.TrqED = 0;
            motor.NomTrq = 0;

            //Speed data
            motor.MaxSpd = 0;
            motor.MaxSpdDur = 0;
            motor.SpdED = 0;
            motor.NomSpd = 0;
            
            #endregion
        }

        private DummyDoc CreateNewDocument()
        {
            DummyDoc dummyDoc = new DummyDoc();

            int count = 1;
            //string text = "C:\\MADFDKAJ\\ADAKFJASD\\ADFKDSAKFJASD\\ASDFKASDFJASDF\\ASDFIJADSFJ\\ASDFKDFDA" + count.ToString();
            string text = "Document" + count.ToString();
            while (FindDocument(text) != null)
            {
                count++;
                //text = "C:\\MADFDKAJ\\ADAKFJASD\\ADFKDSAKFJASD\\ASDFKASDFJASDF\\ASDFIJADSFJ\\ASDFKDFDA" + count.ToString();
                text = "Document" + count.ToString();
            }
            dummyDoc.Text = text;
            return dummyDoc;
        }

        private DummyDoc CreateNewHelpDocument(string text)
        {
            DummyDoc dummyDoc = new DummyDoc();
            dummyDoc.Text = text;
            return dummyDoc;
        }

        private IDockContent FindDocument(string text)
        {
            if (dockPanel.DocumentStyle == DocumentStyle.SystemMdi)
            {
                foreach (Form form in MdiChildren)
                    if (form.Text == text)
                        return form as IDockContent;
                return null;
            }
            else
            {
                foreach (IDockContent content in dockPanel.Documents)
                    if (content.DockHandler.TabText == text)
                        return content;

                return null;
            }
        }

    #endregion

        #region Methods

        private DummyDoc CreateNewDocument(string text)
        {
            DummyDoc dummyDoc = new DummyDoc();
            dummyDoc.Text = text;
            return dummyDoc;
        }

        private void CloseAllDocuments()
        {
            if (dockPanel.DocumentStyle == DocumentStyle.SystemMdi)
            {
                foreach (Form form in MdiChildren)
                    form.Close();
            }
            else
            {
                for (int index = dockPanel.Contents.Count - 1; index >= 0; index--)
                {
                    if (dockPanel.Contents[index] is IDockContent)
                    {
                        IDockContent content = (IDockContent)dockPanel.Contents[index];
                        content.DockHandler.Close();
                    }
                }
            }
        }

        private IDockContent GetContentFromPersistString(string persistString)
        {
            if (persistString == typeof(FrmNavigator).ToString())
                return frmNavigator;
            else if (persistString == typeof(FrmSettings).ToString())
                return frmSettings;
            else if (persistString == typeof(FrmOverview).ToString())
                return frmOverview;
            else if (persistString == typeof(FrmAccessories).ToString())
                return frmAccessories;
            else if (persistString == typeof(FrmResults).ToString())
                return frmResults;
            else if (persistString == typeof(FrmGraph).ToString())
                return frmGraph;
            else if (persistString == typeof(FrmTable).ToString())
                return frmTable;
            else if (persistString == typeof(FrmProperties).ToString())
                return frmProperties;
            else if (persistString == typeof(DummyToolbox).ToString())
                return frmToolbox;
            // else if (persistString == typeof(DummyOutputWindow).ToString())
            //   return m_outputWindow;
            //else if (persistString == typeof(DummyTaskList).ToString())
            // return m_taskList;
            else
            {
                // DummyDoc overrides GetPersistString to add extra information into persistString.
                // Any DockContent may override this value to add any needed information for deserialization.

                string[] parsedStrings = persistString.Split(new char[] { ',' });
                if (parsedStrings.Length != 3)
                    return null;

                if (parsedStrings[0] != typeof(DummyDoc).ToString())
                    return null;

                DummyDoc dummyDoc = new DummyDoc();
                if (parsedStrings[1] != string.Empty)
                    dummyDoc.FileName = parsedStrings[1];
                if (parsedStrings[2] != string.Empty)
                    dummyDoc.Text = parsedStrings[2];

                return dummyDoc;
            }
        }

        private void CloseAllContents()
        {
            // we don't want to create another instance of tool window, set DockPanel to null
            frmNavigator.DockPanel = null;
            frmSettings.DockPanel = null;
            frmProperties.DockPanel = null;

            frmResults.DockPanel = null;
            frmOverview.DockPanel = null;
            frmAccessories.DockPanel = null;

            frmTable.DockPanel = null;
            frmGraph.DockPanel = null;
            frmToolbox.DockPanel = null;

            frmOutput.DockPanel = null;
            frmUnitConversion.DockPanel = null;
            frmHelp.DockPanel = null;

            // Close all other document windows
            //CloseAllDocuments();
        }

        private void SetSchema(object sender, System.EventArgs e)
        {
            CloseAllContents();

            /* if (sender == menuItemSchemaVS2005)
                 Extender.SetSchema(dockPanel, Extender.Schema.VS2005);
             else if (sender == menuItemSchemaVS2003)
                 Extender.SetSchema(dockPanel, Extender.Schema.VS2003);

             menuItemSchemaVS2005.Checked = (sender == menuItemSchemaVS2005);
             menuItemSchemaVS2003.Checked = (sender == menuItemSchemaVS2003);*/
        }

        /*   private void SetDocumentStyle(object sender, System.EventArgs e)
           {
               DocumentStyle oldStyle = dockPanel.DocumentStyle;
               DocumentStyle newStyle;
               if (sender == menuItemDockingMdi)
                   newStyle = DocumentStyle.DockingMdi;
               else if (sender == menuItemDockingWindow)
                   newStyle = DocumentStyle.DockingWindow;
               else if (sender == menuItemDockingSdi)
                   newStyle = DocumentStyle.DockingSdi;
               else
                   newStyle = DocumentStyle.SystemMdi;

               if (oldStyle == newStyle)
                   return;

               if (oldStyle == DocumentStyle.SystemMdi || newStyle == DocumentStyle.SystemMdi)
                   CloseAllDocuments();

               dockPanel.DocumentStyle = newStyle;
               menuItemDockingMdi.Checked = (newStyle == DocumentStyle.DockingMdi);
               menuItemDockingWindow.Checked = (newStyle == DocumentStyle.DockingWindow);
               menuItemDockingSdi.Checked = (newStyle == DocumentStyle.DockingSdi);
               menuItemSystemMdi.Checked = (newStyle == DocumentStyle.SystemMdi);
               menuItemLayoutByCode.Enabled = (newStyle != DocumentStyle.SystemMdi);
               menuItemLayoutByXml.Enabled = (newStyle != DocumentStyle.SystemMdi);
               //toolBarButtonLayoutByCode.Enabled = (newStyle != DocumentStyle.SystemMdi);
               //toolBarButtonLayoutByXml.Enabled = (newStyle != DocumentStyle.SystemMdi);
           }*/

        private void SetDockPanelSkinOptions(bool isChecked)
        {
            if (isChecked)
            {
                // All of these options may be set in the designer.
                // This is not a complete list of possible options available in the skin.

                AutoHideStripSkin autoHideSkin = new AutoHideStripSkin();
                autoHideSkin.DockStripGradient.StartColor = Color.AliceBlue;
                autoHideSkin.DockStripGradient.EndColor = Color.Blue;
                autoHideSkin.DockStripGradient.LinearGradientMode = System.Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal;
                autoHideSkin.TabGradient.StartColor = SystemColors.Control;
                autoHideSkin.TabGradient.EndColor = SystemColors.ControlDark;
                autoHideSkin.TabGradient.TextColor = SystemColors.ControlText;
                autoHideSkin.TextFont = new Font("Showcard Gothic", 10);

                dockPanel.Skin.AutoHideStripSkin = autoHideSkin;

                DockPaneStripSkin dockPaneSkin = new DockPaneStripSkin();
                dockPaneSkin.DocumentGradient.DockStripGradient.StartColor = Color.Red;
                dockPaneSkin.DocumentGradient.DockStripGradient.EndColor = Color.Pink;

                dockPaneSkin.DocumentGradient.ActiveTabGradient.StartColor = Color.Green;
                dockPaneSkin.DocumentGradient.ActiveTabGradient.EndColor = Color.Green;
                dockPaneSkin.DocumentGradient.ActiveTabGradient.TextColor = Color.White;

                dockPaneSkin.DocumentGradient.InactiveTabGradient.StartColor = Color.Gray;
                dockPaneSkin.DocumentGradient.InactiveTabGradient.EndColor = Color.Gray;
                dockPaneSkin.DocumentGradient.InactiveTabGradient.TextColor = Color.Black;

                dockPaneSkin.TextFont = new Font("SketchFlow Print", 10);

                dockPanel.Skin.DockPaneStripSkin = dockPaneSkin;
            }
            else
            {
                dockPanel.Skin = new DockPanelSkin();
            }

            //menuItemLayoutByXml_Click(menuItemLayoutByXml, EventArgs.Empty);
        }

        #endregion

        #region Event Handlers

        private void menuItemExit_Click(object sender, System.EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult dlgResult = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (dlgResult == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (dlgResult == DialogResult.Cancel)
                        return;
            } 
            
            Close();
        }

        private void toolStripMenuItem_Navigator_Click(object sender, System.EventArgs e)
        {
            frmNavigator.Show(dockPanel);
        }

        private void toolStripMenuItem_Settings_Click(object sender, System.EventArgs e)
        {
            frmSettings.Show(dockPanel);
        }

        private void toolStripMenuItem_LoadProperties_Click(object sender, System.EventArgs e)
        {
            frmProperties.Show(dockPanel);
        }

        private void toolStripMenuItem_ApplicationData_Click(object sender, System.EventArgs e)
        {
            IDockContent frm = FindDocument("ApplicationData");

            if (frm != null)
                frmOverview.Show();
            else
            {
                frmOverview = null;
                frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
                frmOverview.Show(dockPanel, DockState.Document);
            }
        }

        private void toolStripMenuItem_Moves_Click(object sender, System.EventArgs e)
        {
            IDockContent frm = FindDocument("Results");

            if (frm != null)
                frmResults.Show();
            else
            {
                frmResults = null;
                frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                frmResults.Show(dockPanel, DockState.Document);
            }
        }

        private void toolStripMenuItem_Graph_Click(object sender, System.EventArgs e)
        {
            frmGraph.Show(dockPanel);
        }

        private void toolStripMenuItem_Table_Click(object sender, System.EventArgs e)
        {
            frmTable.Show(dockPanel);
        }

        private void toolStripMenuItem_Toolbox_Click(object sender, System.EventArgs e)
        {
            frmToolbox.Show(dockPanel);
        }

        private void toolStripMenuItem_Output_Click(object sender, System.EventArgs e)
        {
            frmOutput.Show(dockPanel);
        }

        private void toolStripMenuItem_UnitConversion_Click(object sender, System.EventArgs e)
        {
            frmUnitConversion.Show(dockPanel);
        }

        private void toolStripMenuItem_Help_Click(object sender, System.EventArgs e)
        {
            frmHelp.Show(dockPanel);
        }

        private void menuItemAbout_Click(object sender, System.EventArgs e)
        {
            AboutDialog aboutDialog = new AboutDialog();
            aboutDialog.ShowDialog(this);
        }

        private void menuItemNew_Click(object sender, System.EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult dlgResult = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (dlgResult == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();
                }
                else
                    if (dlgResult == DialogResult.Cancel)
                        return;
            }

            CloseAll();

            unsavedData = false;
        }

        private void menuItemCloseAll_Click(object sender, System.EventArgs e)
        {
            CloseAllDocuments();
        }

        private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            string configFile = Path.Combine(Path.GetDirectoryName(AppDomain.CurrentDomain.BaseDirectory), "DockPanel.config");
            if (m_bSaveLayout)
                dockPanel.SaveAsXml(configFile);
            else if (File.Exists(configFile))
                File.Delete(configFile);
        }

        private void menuItemToolBar_Click(object sender, System.EventArgs e)
        {
            toolBar.Visible = menuItemToolBar.Checked = !menuItemToolBar.Checked;
        }

        private void menuItemStatusBar_Click(object sender, System.EventArgs e)
        {
            statusBar.Visible = menuItemStatusBar.Checked = !menuItemStatusBar.Checked;
        }

        private void toolBar_ButtonClick(object sender, System.Windows.Forms.ToolStripItemClickedEventArgs e)
        {

            if (e.ClickedItem == toolStripButton_Open)
                menuItemOpen_Click(null, null);
            //else if (e.ClickedItem == toolBarButtonSolutionExplorer)
            //toolStripMenuItem_Navigator_Click(null, null);
            //else if (e.ClickedItem == toolBarButtonPropertyWindow)
            //toolStripMenuItem_Settings_Click(null, null);
            //else if (e.ClickedItem == toolBarButtonToolbox)
            //toolStripMenuItem_Toolbox_Click(null, null);
            //else if (e.ClickedItem == toolBarButtonLayoutByCode)
            //menuItemLayoutByCode_Click(null, null);
            //else if (e.ClickedItem == toolBarButtonLayoutByXml)
            //menuItemLayoutByXml_Click(null, null);
        }

        private void menuItemNewWindow_Click(object sender, System.EventArgs e)
        {
            MainForm newWindow = new MainForm();
            newWindow.Text = newWindow.Text + " - New";
            newWindow.Show();
        }

        private void menuItemTools_Popup(object sender, System.EventArgs e)
        {
            //menuItemLockLayout.Checked = !this.dockPanel.AllowEndUserDocking;
        }

        private void menuItemLockLayout_Click(object sender, System.EventArgs e)
        {
            dockPanel.AllowEndUserDocking = !dockPanel.AllowEndUserDocking;
        }

        private void menuItemLayoutByCode_Click(object sender, System.EventArgs e)
        {
            setDefaultLayout();
        }

        private void setDefaultLayout()
        {
            CloseAllContents();

            CloseAllDocuments();

            dockPanel.SuspendLayout(true);

            frmSettings.Show(dockPanel, DockState.DockLeft);
            frmNavigator.Show(frmSettings.Pane, frmSettings);
            frmProperties.Show(frmNavigator.Pane, DockAlignment.Bottom, 0.5);

            frmResults.Show(dockPanel, DockState.Document);
            frmOverview.Show(frmResults.Pane, frmResults);
            frmAccessories.Show(frmResults.Pane, frmResults);

            frmTable.Show(dockPanel, DockState.DockRight);
            frmGraph.Show(frmTable.Pane, frmTable);
            //frmToolbox.Show(frmGraph.Pane, DockAlignment.Bottom, 0.35);

            //frmOutput.Show(frmOverview.Pane, DockAlignment.Bottom, 0.2);
            //frmUnitConversion.Show(frmOutput.Pane, frmOutput);
            //frmHelp.Show(frmOutput.Pane, frmUnitConversion);

            dockPanel.ResumeLayout(true, true);
        }

        private void menuItemLayoutByXml_Click(object sender, System.EventArgs e)
        {
            string configFile = Path.Combine(Path.GetDirectoryName(AppDomain.CurrentDomain.BaseDirectory), "DockPanel.config");

            if (File.Exists(configFile) == true)
            {
                dockPanel.SuspendLayout(true);

                // In order to load layout from XML, we need to close all the DockContents
                CloseAllContents();

                dockPanel.LoadFromXml(configFile, m_deserializeDockContent);

                //Add the application data node to the treeview 
                frmNavigator.AddApplicationDataNode(applicationData);

                //Allow the user to drag or reorder windows
                dockPanel.AllowEndUserDocking = true;

                dockPanel.ResumeLayout(true, true);
            }
            else

                MessageBox.Show("File not found.", "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation,
                    MessageBoxDefaultButton.Button1);


            /*Assembly assembly = Assembly.GetAssembly(typeof(MainForm));
            Stream xmlStream = assembly.GetManifestResourceStream("DockSample.Resources.DockPanel.xml");
            dockPanel.LoadFromXml(xmlStream, m_deserializeDockContent);
            xmlStream.Close();*/
        }

        private void menuItemCloseAllButThisOne_Click(object sender, System.EventArgs e)
        {
            if (dockPanel.DocumentStyle == DocumentStyle.SystemMdi)
            {
                Form activeMdi = ActiveMdiChild;
                foreach (Form form in MdiChildren)
                {
                    if (form != activeMdi)
                        form.Close();
                }
            }
            else
            {
                foreach (IDockContent document in dockPanel.DocumentsToArray())
                {
                    if (!document.DockHandler.IsActivated)
                        document.DockHandler.Close();
                }
            }
        }

        private void menuItemShowDocumentIcon_Click(object sender, System.EventArgs e)
        {
           // dockPanel.ShowDocumentIcon = menuItemShowDocumentIcon.Checked = !menuItemShowDocumentIcon.Checked;
        }

        private void exitWithoutSavingLayout_Click(object sender, EventArgs e)
        {
            m_bSaveLayout = false;
            Close();
            m_bSaveLayout = true;
        }

        #endregion

        private void toolStripButton_Save_Click(object sender, EventArgs e)
        {
            if (mainFileName == null)
                SaveFileAs();
            else
                SaveFile();
        }

        private void toolStripMenuItem_SaveAs_Click(object sender, EventArgs e)
        {
            SaveFileAs();
        }

        private void SaveFileAs()
        {
            //Set focus on the first treenode. This triggers the other controls to update their values before data is saved
            frmNavigator.treeView_Navigator.SelectedNode = frmNavigator.treeView_Navigator.Nodes[0];
            frmNavigator.treeView_Navigator.Focus();

            SaveFileDialog saveFile = new SaveFileDialog();

            if (mainFileName == null)
                mainFileName = "BSD Drive Size doc";

            saveFile.InitialDirectory = mainPath; // Application.ExecutablePath;
            saveFile.FileName = mainFileName;
            saveFile.Filter = "Beijer Drive files (*.bdr)|*.bdr|All files (*.*)|*.*";
            saveFile.FilterIndex = 1;
            saveFile.RestoreDirectory = true;

            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                mainFileName = Path.GetFileName(saveFile.FileName);  //filename
                mainPath = Path.GetDirectoryName(saveFile.FileName); //Directory
                
                string fullName = Path.GetFullPath(saveFile.FileName);  //Directory + filename        //saveFile.FileName;

                xml.SaveAsXml(fullName, applicationData, moves, iLoads, motor, iAccessories);

                unsavedData = false;
            }

            this.Text = "BSD Drive Sizer - " + mainFileName;
        }

        private void SaveFile()
        {
            //Set focus on the first treenode. This triggers the other controls to update their values before data is saved
            frmNavigator.treeView_Navigator.SelectedNode = frmNavigator.treeView_Navigator.Nodes[0];
            frmNavigator.treeView_Navigator.Focus();

            SaveFileDialog saveFile = new SaveFileDialog();

            saveFile.InitialDirectory = mainPath;
            saveFile.RestoreDirectory = true;

            saveFile.FileName = mainPath + @"\" + mainFileName;

            string fullName = Path.GetFullPath(saveFile.FileName); 
            
            xml.SaveAsXml(fullName, applicationData, moves, iLoads, motor, iAccessories);

            unsavedData = false;
        }

        private void menuItemOpen_Click(object sender, System.EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult dlgResult = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (dlgResult == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (dlgResult == DialogResult.Cancel)
                        return;
            }

            disableUpdate = true;
            CloseAll();
            unsavedData = false;
            disableUpdate = false;

            OpenFileDialog openFile = new OpenFileDialog();

            openFile.InitialDirectory = mainPath; //Application.ExecutablePath;
            openFile.Filter = "Beijer Drive files (*.bdr)|*.bdr|All files (*.*)|*.*";
            openFile.FilterIndex = 1;
            openFile.RestoreDirectory = true;

            if (openFile.ShowDialog() == DialogResult.OK)
            {
                string fullName = openFile.FileName;
                string fileName = Path.GetDirectoryName(fullName);//Path.GetFileName(fullName);

                mainFileName = Path.GetFileName(openFile.FileName);  //filename
                mainPath = Path.GetDirectoryName(openFile.FileName); //Directory

                if (FindDocument(fileName) != null)
                {
                    MessageBox.Show("The document: " + fileName + " has already opened!");
                    return;
                }

                FileStream fs = new FileStream(fullName, FileMode.Open, FileAccess.Read);
                try
                {
                    xml.LoadFromXml(fs, applicationData, moves, iLoads, motor, iAccessories);
                }
                finally
                {
                    fs.Close();
                }

                //Attach load events and add the loads to the navigator
                foreach (var iload in iLoads)
                    InitializeLoad(iload.Value, iload.Key);

                //Update the ComboBox on the LoadProperties form
                frmProperties.ComboBox_ReLoad();

                //Select the last added load in the loads list
                frmProperties.ComboBox_SelectLoad(iLoads[iLoads.Count - 1].Name);

                //Initialize each move based on the new data
                foreach (var m in moves.MovesList)
                    //Attach the event handler, create treeview node...
                    initializeMove(m.Value, m.Key);

                disableUpdate = false;
                
                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm != null)
                    frmResults.Show();
                else
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                    frmResults.Show(dockPanel, DockState.Document);
                }

                this.Text = "BSD Drive Sizer - " + mainFileName;                
            }
        }

        private void InitializeLoad(ILoad iLoad, int loadNr)
        {
            //Add the new load to the navigator
            frmNavigator.AddLoadNode(loadNr, iLoad, iLoad.Name);

            //Attach the event handler
            switch (iLoad.MainType)
            {
                case (MainType.BallScrew):
                    BallScrew ballScrew = iLoad as BallScrew;
                    ballScrew.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    ballScrew.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(ballScrew_RefreshPropertyGridEvent);
                    break;

                case (MainType.RackAndPinion):
                    RackAndPinion rackAndPinion = iLoads[loadNr] as RackAndPinion;
                    rackAndPinion.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    rackAndPinion.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(rackAndPinion_RefreshPropertyGridEvent);
                    break;

                case (MainType.ToothedBelt):
                    ToothedBelt toothedBelt = iLoads[loadNr] as ToothedBelt;
                    toothedBelt.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    toothedBelt.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(toothedBelt_RefreshPropertyGridEvent);
                    break;

                case (MainType.TurnTable):
                    TurnTable turnTable = iLoads[loadNr] as TurnTable;
                    turnTable.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    turnTable.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(turnTable_RefreshPropertyGridEvent);
                    break;

                case (MainType.Coupling):
                    Coupling coupling = iLoad as Coupling;
                    coupling.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    coupling.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(coupling_RefreshPropertyGridEvent);
                    break;

                case (MainType.GearBox):
                    GearBox gearBox = iLoad as GearBox;
                    gearBox.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    break;

                case (MainType.Mass):
                    LoadMass mass = iLoad as LoadMass;
                    mass.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    break;

                case (MainType.Torque):
                    LoadTorque torque = iLoad as LoadTorque;
                    torque.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    break;

                case (MainType.Inertia):
                    LoadInertia inertia = iLoad as LoadInertia;
                    inertia.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    inertia.RefreshPropertyGridEvent += new EventHandler<RefreshPropertyGridEventArgs>(inertia_RefreshPropertyGridEvent);
                    break;

                case (MainType.Force):
                    LoadForce force = iLoad as LoadForce;
                    force.PropertyChanged += new PropertyChangedEventHandler(Load_PropertyChanged);
                    break;
            }
        }

        void turnTable_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }

        void rackAndPinion_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }
        
        void toothedBelt_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }

        void ballScrew_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }

        void coupling_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }

        void inertia_RefreshPropertyGridEvent(object sender, RefreshPropertyGridEventArgs e)
        {
            frmProperties.Refresh();
        }

        private void initializeMove(Move move, int moveNr)
        {
            move.PropertyChanged += new PropertyChangedEventHandler(Move_PropertyChanged);
            move.SectorsList.CollectionChanged += new NotifyCollectionChangedEventHandler(SectorsList_CollectionChanged);
            move.SectorPropertyChanged += new PropertyChangedEventHandler(Sector_PropertyChanged);

            //Add the new move to the application tree
            frmNavigator.AddMoveNode(moveNr, move, move.Name);

            //Add sectors
            if (move.MoveType == MoveType.PreDefined)
            {
                //Add three sectors
                Sector _sectorAcc = new Sector(move.Name + "_Acc", move.Name, 0);
                Sector _sectorConst = new Sector(move.Name + "_Const", move.Name, 1);
                Sector _sectorDec = new Sector(move.Name + "_Dec", move.Name, 2);

                move.SectorsList.Add(_sectorAcc);
                move.SectorsList.Add(_sectorConst);
                move.SectorsList.Add(_sectorDec);
            }

            if (move.MoveType == MoveType.UserDefined)
            {
                //Add one sector
                Sector _sector = new Sector(move.Name, move.Name, 0);
                move.SectorsList.Add(_sector);
            }

            //Set all existing loads to active as default
            //This should be based on the contents in the XML file
            if (iLoads.Count > 0)
                if (move.LoadResultsList.Count == 0)
                    move.ActivateLoadResults(iLoads);

            //Create and show the new move form
            FrmMove frmMove = new FrmMove(move, iLoads, motor);
            frmMove.Text = move.Name;
            frmMove.Show(dockPanel, DockState.Document);
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (unsavedData)
            {
                DialogResult dlgResult = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (dlgResult == DialogResult.Yes)
                {
                    if (mainPath == null || mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
            }
        }

        private void toolStripButton_Print_Click(object sender, EventArgs e)
        {
            //Set focus on the first tree node. This triggers the other controls to update their values before data is saved
            frmNavigator.treeView_Navigator.SelectedNode = frmNavigator.treeView_Navigator.Nodes[0];
            frmNavigator.treeView_Navigator.Focus();

            printIndex = 0;

            PrintDocument printDocument = new PrintDocument();

            printDocument.DocumentName = "BSD Drive Sizer";
            printDocument.DefaultPageSettings.Landscape = true;
            printDocument.OriginAtMargins = false;
            //printDocument.DefaultPageSettings.Landscape = false;
            printDocument.DefaultPageSettings.Margins.Top = 50;
            printDocument.DefaultPageSettings.Margins.Left = 75;
            printDocument.DefaultPageSettings.Margins.Right = 0;
            printDocument.DefaultPageSettings.Margins.Bottom = 50;

            printDocument.PrintPage += new PrintPageEventHandler(this.printDocument_PrintPage);

            CoolPrintPreview.CoolPrintPreviewDialog printPreviewDialog = new CoolPrintPreview.CoolPrintPreviewDialog();
            printPreviewDialog.Document = printDocument;
            printPreviewDialog.ShowDialog();
        }

        private void printDocument_PrintPage(object sender, PrintPageEventArgs e)
        {
            int y = e.MarginBounds.Y;
            int x = e.MarginBounds.X;

            //Draw header and footer

            Font myFont = new Font("Times New Roman", 16, FontStyle.Bold);
            e.Graphics.DrawString("Application: " + applicationData.AppName, myFont, Brushes.Black, e.MarginBounds.X, e.MarginBounds.Y + 25);

            //Draw upper and lower dividing line

            //Create pen.
            Pen blackPen = new Pen(Color.Black, 0);

            //Create points that define line.
            Point point1 = new Point(e.MarginBounds.X, e.MarginBounds.Y + 80);
            Point point2 = new Point(e.MarginBounds.Width, e.MarginBounds.Y + 80);

            //Draw upper line to screen.
            e.Graphics.DrawLine(blackPen, point1, point2);

            //Redefine line
            point1 = new Point(e.MarginBounds.X, e.MarginBounds.Bottom - 50);
            point2 = new Point(e.MarginBounds.Width, e.MarginBounds.Bottom - 50);

            //Draw lower line to screen.
            e.Graphics.DrawLine(blackPen, point1, point2);

            //Draw logo in upper right corner

            //Create a Bitmap object from an image file.
            Bitmap myBitmap = new Bitmap(Properties.Resources.Beijerlogo);

            //Make image transparent.
            myBitmap.MakeTransparent();

            //Draw the transparent bitmap to the screen.
            e.Graphics.DrawImage(myBitmap, e.MarginBounds.Width - 150, e.MarginBounds.Y, 150, 70);

            //Draw logo in upper right corner

            //Create a Bitmap object from an image file.
            //myBitmap = new Bitmap(Properties.Resources.AluFlex);

            //Make image transparent.
            //myBitmap.MakeTransparent();

            //Draw the transparent bitmap to the screen.
            //e.Graphics.DrawImage(myBitmap, e.MarginBounds.Width - 300, e.MarginBounds.Y+10, 100, 60);

            if (printIndex == 0) //Application data and table of contents
            {
                //Main page
                myFont = new Font("Times New Roman", 12, FontStyle.Bold);
                
                //StringFormat stringFormat = new StringFormat();
                //stringFormat.Alignment = StringAlignment.Center;
                //stringFormat.LineAlignment = StringAlignment.Center;

                //y += 150;
                //e.Graphics.DrawString("Application: " + applicationData.AppName, myFont, Brushes.Black, e.MarginBounds.Width / 2, y, stringFormat);
               
                //Application data
                                
                string appDate = "Date: " + applicationData.Date;
                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 100;
                e.Graphics.DrawString(appDate, myFont, Brushes.Black, e.MarginBounds.X, y);

                string appCust = "Customer: " + applicationData.Customer;
                e.Graphics.DrawString(appCust, myFont, Brushes.Black, e.MarginBounds.Width / 2, y);

                string appYourRef = "Your Reference: " + applicationData.YourRef;
                y += 20;
                e.Graphics.DrawString(appYourRef, myFont, Brushes.Black, e.MarginBounds.X, y);

                string appOurRef = "Our Reference: " + applicationData.OurRef;
                e.Graphics.DrawString(appOurRef, myFont, Brushes.Black, e.MarginBounds.Width / 2, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Description", myFont, Brushes.Black, e.MarginBounds.X, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                y += 20;
                e.Graphics.DrawString(applicationData.Description, myFont, Brushes.Black, e.MarginBounds.X, y);
                                
                //Table of Contents

                myFont = new Font("Times New Roman", 14, FontStyle.Bold);
                x = e.MarginBounds.Width / 3; 
                y += 60;
                e.Graphics.DrawString("Table of Contents", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 40;                
                e.Graphics.DrawString("Page 1 - Application data and Table of Contents (this page)", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("Application data, with a brief project description.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Page 2 - Bill of Material: List of components to be used as an ordering help.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("Material list including all components with a defined article number.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Page 3 - Main page: Calculation results for all components, including all moves.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("Summary of defined moves and mechanical components, with calculated speed and torque utilization in percent of rated values.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Page 4 - Graphs displaying all moves.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("Time graphs for jerk/acceleration, velocity/distance and torque/regenerative energy.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Page 5 - Detailed view of selected motor, drive and accessories", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("All properties of selected motor, motor cables, drive and drive accessories.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 20;
                e.Graphics.DrawString("Page 6 - Detailed view of selected load components.", myFont, Brushes.Black, x, y);

                myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                y += 20;
                e.Graphics.DrawString("All properties of selected load components, in mechanical order.", myFont, Brushes.Black, x, y);
                                
                foreach(var m in moves.MovesList)
                {
                    Move move = m.Value;
                    int count = m.Key + 7;

                    myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                    y += 20;
                    e.Graphics.DrawString("Page " + count + " - Detailed calculation results for move: " + move.Name, myFont, Brushes.Black, x, y);

                    myFont = new Font("Times New Roman", 8, FontStyle.Italic);
                    y += 20;
                    e.Graphics.DrawString("Detailed view of calculation results for each component including graphs, for this move.", myFont, Brushes.Black, x, y);
                }

                e.HasMorePages = true;
            }

            if (printIndex == 1) //Bill of material
            {
                //Set focus to the 'OverView'-form
                IDockContent frm = FindDocument("OverView");

                if (frm == null)
                {
                    frmOverview = null;
                    frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
                }                

                // Main page
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString("Bill of material including all components with a defined article number:", myFont, Brushes.Black, e.MarginBounds.X, y);

                //Check the number of defined articles
                int nrOfArticles = frmOverview.BillOfMaterialRowCount();

                y += 50;
                myDGVPrinter.EmbeddedPrint(frmOverview.GetDGVBillOfMaterial(), e.Graphics,
                    new Rectangle(e.MarginBounds.X, y, e.MarginBounds.Width, nrOfArticles*30));

                y = y + 20 * nrOfArticles;

                y += 30;
                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                e.Graphics.DrawString("Read this about gearboxes:", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                e.Graphics.DrawString("Gearboxes can normally not be mechanically fitted directly to a servo motor. Instead an adapter plate must be used with a matching drill pattern", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                e.Graphics.DrawString("for both components. The adapter plate is included in the gearbox delivery and the selected motor must be stated in the gearbox description.", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                e.Graphics.DrawString("Several motors has the same flange size and to simplify only one gearbox for each size has been assigned an article number.", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 30;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("The following list shows each group of motors with the same mechanical properties:", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 30;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 1 (Flange 40x40mm, 8mm shaft): BSD-FAL01(2), (ME-KF-KP13(B))", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 2 (Flange 60x60mm, 14mm shaft): , BSD-FB02-04(2), (ME-HF-KP23-43(B))", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 3 (Flange 80x80mm, 19mm shaft): BSD-FC08(2), (ME-HF-KP73(B))", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 4 (Flange 130x130mm, 19mm shaft): BSD-FEP09A-15A(2), BSD-FEP06D-11D(2)", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 5 (Flange 130x130mm, 22mm shaft): BSD-FEP22A(2), BSD-FEP16D(2)", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 6 (Flange 130x130mm, 24mm shaft): BSD-FEP30A(2), BSD-FEP22D(2), (ME-HF-SP524-1524(B))", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 7 (Flange 180x180mm, 35mm shaft): BSD-FFP50A(2), BSD-FFP35D-55D(2), (ME-HF-SP2024-7024(B))", myFont, Brushes.Black, e.MarginBounds.X, y);

                y += 20;
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Group 8 (Flange 180x180mm, 42mm shaft): BSD-FFP75D(2)", myFont, Brushes.Black, e.MarginBounds.X, y);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page 2 - Bill of material", myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);

                e.HasMorePages = true;
            }

            if (printIndex == 2) //Results
            {
                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm == null)
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                }

                if (iLoads.Count > 0)
                {
                    frmResults.CalculateMovesResults();
                    frmResults.DisplayMovesResults();                
                }

                frmGraph.CreateTorquePerComponentGraph(moves.MotorResults, moves.LoadResultsList);
             
                string messageDate = "Calculation results for all components including all moves. The utilization for each component parameter is shown in parenthesis.";
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString(messageDate, myFont, Brushes.Black, e.MarginBounds.X, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 50;
                e.Graphics.DrawString("All move results in time order:", myFont, Brushes.Black, e.MarginBounds.X, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("(     = locked movement parameter) (     = Initial and final speed mismatch )", myFont, Brushes.Black, e.MarginBounds.X + 190, y);

                Microsoft.VisualBasic.PowerPacks.RectangleShape colorIndicationRectangle = new Microsoft.VisualBasic.PowerPacks.RectangleShape();

                colorIndicationRectangle.Width = 12;
                colorIndicationRectangle.Height = 8;
                                
                colorIndicationRectangle.FillStyle = Microsoft.VisualBasic.PowerPacks.FillStyle.Solid;

                colorIndicationRectangle.BorderStyle = System.Drawing.Drawing2D.DashStyle.Solid;
                colorIndicationRectangle.BorderWidth = 1;
                colorIndicationRectangle.BorderColor = Color.Black;

                Bitmap colorIndicationBitMap = new Bitmap(12, 8);

                colorIndicationRectangle.FillColor = Color.BurlyWood; 
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 273, y + 5, 12, 8);

                colorIndicationRectangle.FillColor = Color.Salmon; 
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 479, y + 5, 12, 8);

                y += 20;
                myDGVPrinter.EmbeddedPrint(frmResults.GetDGVMoves(), e.Graphics,
                    new Rectangle(e.MarginBounds.X, y, e.MarginBounds.Width, 250));

                y = y + 20 * moves.MovesList.Count;

                y += 30;
                myFont = new Font("Times New Roman", 10, FontStyle.Bold); 
                e.Graphics.DrawString("All component results in mechanical order:", myFont, Brushes.Black, e.MarginBounds.X, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("(     = Auto Selected Component ) (     = Warning) (     = Error) (INF = Infinite).", myFont, Brushes.Black, e.MarginBounds.X + 270, y);

                colorIndicationBitMap = new Bitmap(12, 8);

                colorIndicationRectangle.FillColor = Color.PowderBlue;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 354, y + 5, 12, 8);

                colorIndicationRectangle.FillColor = Color.Gold;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 554, y + 5, 12, 8);

                colorIndicationRectangle.FillColor = Color.Salmon;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 648, y + 5, 12, 8);

                y += 20;
                myDGVPrinter.EmbeddedPrint(frmResults.GetDGVComponents(), e.Graphics,
                new Rectangle(e.MarginBounds.X, y, e.MarginBounds.Width, 250));

                y = y + 20 * iLoads.Count + 40;

                //Inertia ratio
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Inertia ratio:                                (between motor and total load inertia).", myFont, Brushes.Black, e.MarginBounds.X, y);

                TextBox textBoxInertia = frmResults.GetInertiaRatioTextBox();

                Bitmap inertiaBitMap = new Bitmap(textBoxInertia.Width, textBoxInertia.Height);

                textBoxInertia.DrawToBitmap(inertiaBitMap, textBoxInertia.ClientRectangle);

                e.Graphics.DrawImage(inertiaBitMap, e.MarginBounds.X + 90, y);

                y = y + 25;

                //Motor software torque limit
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Torque limit:                                  (to protect gear box).", myFont, Brushes.Black, e.MarginBounds.X, y);

                TextBox textBoxTorqueLimit = frmResults.GetTorqueLimitTextBox();

                Bitmap torqueLimitBitMap = new Bitmap(textBoxTorqueLimit.Width, textBoxTorqueLimit.Height);

                textBoxTorqueLimit.DrawToBitmap(torqueLimitBitMap, textBoxTorqueLimit.ClientRectangle);

                e.Graphics.DrawImage(torqueLimitBitMap, e.MarginBounds.X + 90, y);

                y = y + 25;

                //Regenerative brake resistor results
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Regenerative brake resistor results:", myFont, Brushes.Black, e.MarginBounds.X, y);

                GroupBox groupBoxRegenerativeResults = frmAccessories.GetRegenerativeGroupBox();

                Bitmap regenerativeBitMap = new Bitmap(groupBoxRegenerativeResults.Width, groupBoxRegenerativeResults.Height);

                groupBoxRegenerativeResults.DrawToBitmap(regenerativeBitMap, groupBoxRegenerativeResults.ClientRectangle);

                e.Graphics.DrawImage(regenerativeBitMap, e.MarginBounds.X, y);                
 
                //TorquePerComponent
                Chart chartTorquePerComp = frmGraph.GetTrqPerComponentChart();

                y = 600;
                Rectangle recTorquePerComp = new System.Drawing.Rectangle(e.MarginBounds.Width / 4, y, e.MarginBounds.Width / 2, e.MarginBounds.Height / 5);
                chartTorquePerComp.Printing.PrintPaint(e.Graphics, recTorquePerComp);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page 3 - Main results", myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);

                e.HasMorePages = true;
            }

            if (printIndex == 3) //Graphs
            {
                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm == null)
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                }

                if (iLoads.Count > 0)
                {                
                    frmGraph.CreateJerkAccAndVelDistGraphs(moves.ActiveSectorElementsList);

                    bool validTorqueResults = false;

                    if (moves.LoadResultsList[0].PeakTrq != 0) //ToDo
                        validTorqueResults = true;

                    if (validTorqueResults == true)
                    {
                        double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                        frmGraph.AddMotorSpeedToVelGraph(moves.ActiveSectorElementsList, scale, motor.NomSpd, motor.MaxSpd);
                    }
                }
                                
                frmGraph.CreateTorqueGraph(moves.ActiveSectorElementsList, motor.NomTrq, motor.MaxTrq);
                                                
                moves.AddAllMotionProfiles(moves.MovesList);

                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString("Time graph [" + Settings.MainSettings.MainTimeBase.ToString() + "]. Area between rated and maximum value =      . Area above maximum value =      ,(velocity & torque).", myFont, Brushes.Black, e.MarginBounds.X, y);

                Microsoft.VisualBasic.PowerPacks.RectangleShape colorIndicationRectangle = new Microsoft.VisualBasic.PowerPacks.RectangleShape();

                colorIndicationRectangle.Width = 12;
                colorIndicationRectangle.Height = 8;

                colorIndicationRectangle.FillStyle = Microsoft.VisualBasic.PowerPacks.FillStyle.Solid;

                colorIndicationRectangle.BorderStyle = System.Drawing.Drawing2D.DashStyle.Solid;
                colorIndicationRectangle.BorderWidth = 1;
                colorIndicationRectangle.BorderColor = Color.Black;

                Bitmap colorIndicationBitMap = new Bitmap(12, 8);

                colorIndicationRectangle.FillColor = Color.PapayaWhip;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 477, y + 3, 12, 8);

                colorIndicationRectangle.FillColor = Color.LightSalmon;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 675, y + 3, 12, 8);

                //Chart printing

                //Jerk & Acceleration
                Chart chartJerkAndAcc = frmGraph.GetJerkAndAccChart();

                //while (chartJerkAndAcc.Legends.Count > 0)
                  //  chartJerkAndAcc.Legends.RemoveAt(0);

                //chartJerkAndAcc.Titles.Add("Jerk and Acceleration");

                y += 50;
                Rectangle recJerkAndAcc = new System.Drawing.Rectangle(e.MarginBounds.Left, y, e.MarginBounds.Width, e.MarginBounds.Height / 4);
                chartJerkAndAcc.Printing.PrintPaint(e.Graphics, recJerkAndAcc);

                //Velocity & Distance
                Chart chartVelAndDist = frmGraph.GetVelAndDistChart();

                //while (chartVelAndDist.Legends.Count > 0)
                 //   chartVelAndDist.Legends.RemoveAt(0);

                //while (chartVelAndDist.Titles.Count > 0)
                 //   chartVelAndDist.Titles.RemoveAt(0);

                //chartVelAndDist.Titles.Add("Velocity and Distance");

                y += e.MarginBounds.Height / 4;
                Rectangle recVelAndDist = new System.Drawing.Rectangle(e.MarginBounds.Left, y, e.MarginBounds.Width, e.MarginBounds.Height / 4);
                chartVelAndDist.Printing.PrintPaint(e.Graphics, recVelAndDist);

                //Torque
                Chart chartTorque = frmGraph.GetTrqChart();

                //while (chartTorque.Legends.Count > 0)
                //    chartTorque.Legends.RemoveAt(0);

                //while (chartTorque.Titles.Count > 0)
                //    chartTorque.Titles.RemoveAt(0);

                ///chartTorque.Titles.Add("Torque [Nm] and Regenerative energy [W]");

                y += e.MarginBounds.Height / 4;
                Rectangle recTorque = new System.Drawing.Rectangle(e.MarginBounds.Left, y, e.MarginBounds.Width, e.MarginBounds.Height / 4);
                chartTorque.Printing.PrintPaint(e.Graphics, recTorque);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page 4 - Graphs", myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);
                                
                e.HasMorePages = true;
            }

            if (printIndex == 4) //Selected components
            {
                //Set focus to the 'OverView'-form
                IDockContent frm = FindDocument("OverView");

                if (frm == null)
                {
                    frmOverview = null;
                    frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
                }

                // Main page
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString("Detailed view of selected motor, cables, drive and drive accessories:", myFont, Brushes.Black, e.MarginBounds.X, y);

                TreeView myTreeView = frmOverview.GetTreeViewMotorSide();
                /*
                int height = myTreeView.Nodes[0].Bounds.Height;
                //this.nodeHeight = height;
                int width = myTreeView.Nodes[0].Bounds.Right;
                TreeNode node = myTreeView.Nodes[0].NextVisibleNode;
                while (node != null)
                {
                    height += node.Bounds.Height;
                    if (node.Bounds.Right > width)
                    {
                        width = node.Bounds.Right;
                    }
                    node = node.NextVisibleNode;
                }
                */

                PrintHelper myPrintHelper = new PrintHelper();

                Image treeViewImage = myPrintHelper.CreateTreeImage(myTreeView);

                //Bitmap myTreeViewBitMap = new Bitmap(width, height);

                //Rectangle myRect = new Rectangle(0, 0, width, height);

                //myTreeView.DrawToBitmap(myTreeViewBitMap, myRect);

                y += 30;
                e.Graphics.DrawImage(treeViewImage, e.MarginBounds.X, y, treeViewImage.Width, treeViewImage.Height);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page 5 - Selected components on drive side", myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);
                
                e.HasMorePages = true;
            }

            if (printIndex == 5) //Selected components
            {
                //Set focus to the 'OverView'-form
                IDockContent frm = FindDocument("OverView");

                if (frm == null)
                {
                    frmOverview = null;
                    frmOverview = new FrmOverview(applicationData, motor, iLoads, iAccessories);
                }

                // Main page
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString("Detailed view of selected load components, in mechanical order.", myFont, Brushes.Black, e.MarginBounds.X, y);

                TreeView myTreeView = frmOverview.GetTreeViewLoadSide();
                /*
                int height = myTreeView.Nodes[0].Bounds.Height;
                //this.nodeHeight = height;
                int width = myTreeView.Nodes[0].Bounds.Right;
                TreeNode node = myTreeView.Nodes[0].NextVisibleNode;
                while (node != null)
                {
                    height += node.Bounds.Height;
                    if (node.Bounds.Right > width)
                    {
                        width = node.Bounds.Right;
                    }
                    node = node.NextVisibleNode;
                }

                Bitmap myTreeViewBitMap = new Bitmap(width, height);

                Rectangle myRect = new Rectangle(0, 0, width, height);

                myTreeView.DrawToBitmap(myTreeViewBitMap, myRect);

                y += 60;
                e.Graphics.DrawImage(myTreeViewBitMap, e.MarginBounds.X, y, width, height);
                */

                PrintHelper myPrintHelper = new PrintHelper();

                Image treeViewImage = myPrintHelper.CreateTreeImage(myTreeView);

                //Bitmap myTreeViewBitMap = new Bitmap(width, height);

                //Rectangle myRect = new Rectangle(0, 0, width, height);

                //myTreeView.DrawToBitmap(myTreeViewBitMap, myRect);

                y += 30;
                e.Graphics.DrawImage(treeViewImage, e.MarginBounds.X, y, treeViewImage.Width, treeViewImage.Height);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page 6 - Selected components on load side", myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);

                e.HasMorePages = true;
            }           

            if (printIndex > 5 && printIndex < moves.MovesList.Count + 6)
            {
                Move move = moves.MovesList[printIndex - 6];

                IDockContent frm = FindDocument(move.Name);
                FrmMove frmMove = null;

                if (frm != null)
                    frmMove = (FrmMove)frm;

                else
                {
                    frmMove = new FrmMove(move, iLoads, motor);
                    frmMove.Text = move.Name;
                }

                move.CalcMoveResults();

                frmMove.UpdateMoveResults();

                frmGraph.CreateJerkAccAndVelDistGraphs(move.GetAllSectorElements());

                if (iLoads.Count > 0)
                {
                    move.CalcLoadResults(iLoads);
                    frmMove.CalcMotorResults();
                    frmMove.CreateLoadResultsTable();

                    bool validTorqueResults = false;

                    if (move.TotalLoadResults.PeakTrq != 0)
                        validTorqueResults = true;

                    if (validTorqueResults == true)
                    {
                        double scale = 1 / (moves.MovesList[0].TotalLoadResults.InpMovePerRev / 60);
                        frmGraph.AddMotorSpeedToVelGraph(move.GetAllSectorElements(), scale, motor.NomSpd, motor.MaxSpd);
                    }
                }

                frmGraph.CreateTorqueGraph(move.GetAllSectorElements(), motor.NomTrq, motor.MaxTrq);
                frmGraph.CreateTorquePerComponentGraph(move.MotorResults, move.LoadResultsList);

                frmTable.CreateTable(move.GetAllSectorElements());
                
                //Move pages
                myFont = new Font("Times New Roman", 10, FontStyle.Italic);
                y += 60;
                e.Graphics.DrawString("Detailed view of calculation results for each component including graphs, for move: " + move.Name, myFont, Brushes.Black, e.MarginBounds.X, y);

                GroupBox myGroupBox = frmMove.GetGroupBoxMove();

                myGroupBox.BackColor = Color.White;

                //Create a Bitmap object from an image file.
                Bitmap myMoveBitmap = new Bitmap(myGroupBox.Width, myGroupBox.Width);

                myMoveBitmap.MakeTransparent();

                 myGroupBox.DrawToBitmap(myMoveBitmap, myGroupBox.ClientRectangle);

                 y += 40;
                //Draw the transparent bitmap to the screen.
                 e.Graphics.DrawImage(myMoveBitmap, e.MarginBounds.X, y, 950, 900);                
                               
                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                y += 100;
                e.Graphics.DrawString("All sector results in time order", myFont, Brushes.Black, e.MarginBounds.X, y);
                
                y += 20;
                myDGVPrinter.EmbeddedPrint(frmMove.GetDGVSectors(), e.Graphics,
                    new Rectangle(e.MarginBounds.X, y, e.MarginBounds.Width, 200));

                y = y + 20 * move.SectorsList.Count;

                y += 30;
                myFont = new Font("Times New Roman", 10, FontStyle.Bold);
                e.Graphics.DrawString("All component results in mechanical order:", myFont, Brushes.Black, e.MarginBounds.X, y);

                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("(     = Auto Selected Component ) (     = Warning) (     = Error) (INF = Infinite).", myFont, Brushes.Black, e.MarginBounds.X + 270, y);

                Microsoft.VisualBasic.PowerPacks.RectangleShape colorIndicationRectangle = new Microsoft.VisualBasic.PowerPacks.RectangleShape();

                colorIndicationRectangle.Width = 12;
                colorIndicationRectangle.Height = 8;

                colorIndicationRectangle.FillStyle = Microsoft.VisualBasic.PowerPacks.FillStyle.Solid;
                
                colorIndicationRectangle.BorderStyle = System.Drawing.Drawing2D.DashStyle.Solid;
                colorIndicationRectangle.BorderWidth = 1;
                colorIndicationRectangle.BorderColor = Color.Black;

                Bitmap colorIndicationBitMap = new Bitmap(12, 8);

                colorIndicationRectangle.FillColor = Color.PowderBlue;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 354, y + 5, 12, 8);

                colorIndicationRectangle.FillColor = Color.Gold;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 554, y + 5, 12, 8);

                colorIndicationRectangle.FillColor = Color.Salmon;
                colorIndicationRectangle.DrawToBitmap(colorIndicationBitMap, colorIndicationRectangle.ClientRectangle);
                e.Graphics.DrawImage(colorIndicationBitMap, 648, y + 5, 12, 8);

                y += 20;
                myDGVPrinter.EmbeddedPrint(frmMove.GetDGVComponents(), e.Graphics,
                    new Rectangle(e.MarginBounds.X, y, 1200, 200));
                
                //Chart printing

                //Jerk & Acceleration
                Chart chartJerkAndAcc = frmGraph.GetJerkAndAccChart();

                while (chartJerkAndAcc.Legends.Count > 0)
                    chartJerkAndAcc.Legends.RemoveAt(0);

                if (chartJerkAndAcc.Titles.Count == 0)
                chartJerkAndAcc.Titles.Add("Jerk and Acceleration");

                Rectangle recJerkAndAcc = new System.Drawing.Rectangle(e.MarginBounds.Left, 600, e.MarginBounds.Width / 4, 150);
                chartJerkAndAcc.Printing.PrintPaint(e.Graphics, recJerkAndAcc);

                //Velocity & Distance
                Chart chartVelAndDist = frmGraph.GetVelAndDistChart();

                while (chartVelAndDist.Legends.Count > 0)
                    chartVelAndDist.Legends.RemoveAt(0);

                while (chartVelAndDist.Titles.Count > 0)
                    chartVelAndDist.Titles.RemoveAt(0);

                chartVelAndDist.Titles.Add("Velocity and Distance");

                Rectangle recVelAndDist = new System.Drawing.Rectangle(e.MarginBounds.Left + e.MarginBounds.Width / 4, 600, e.MarginBounds.Width / 4, 150);
                chartVelAndDist.Printing.PrintPaint(e.Graphics, recVelAndDist);

                //Torque
                Chart chartTorque = frmGraph.GetTrqChart();

                while (chartTorque.Legends.Count > 0)
                    chartTorque.Legends.RemoveAt(0);

                while (chartTorque.Titles.Count > 0)
                    chartTorque.Titles.RemoveAt(0);

                chartTorque.Titles.Add("Torque [Nm]");

                Rectangle recTorque = new System.Drawing.Rectangle(e.MarginBounds.Left + e.MarginBounds.Width / 2, 600, e.MarginBounds.Width / 4, 150);
                chartTorque.Printing.PrintPaint(e.Graphics, recTorque);

                //TorquePerComponent
                Chart chartTorquePerComp = frmGraph.GetTrqPerComponentChart();

                if (chartTorquePerComp.Legends.Count > 0)
                    chartTorquePerComp.Legends[0].Name = "";

                Rectangle recTorquePerComp = new System.Drawing.Rectangle(e.MarginBounds.Left + 3 * e.MarginBounds.Width / 4, 600, e.MarginBounds.Width / 4, 150);
                chartTorquePerComp.Printing.PrintPaint(e.Graphics, recTorquePerComp);

                //page number
                myFont = new Font("Times New Roman", 10, FontStyle.Regular);
                e.Graphics.DrawString("Page " + (printIndex + 1).ToString() + " - " + move.Name, myFont, Brushes.Black, (e.MarginBounds.Width / 2) - 50, e.MarginBounds.Bottom - 20);

                e.HasMorePages = true;
            }
            
            if (printIndex >= moves.MovesList.Count + 5)
                e.HasMorePages = false;

            printIndex++;
        }

        private void toolStripButton_DefineBallScrew_Click(object sender, EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult result3 = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (result3 == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (result3 == DialogResult.Cancel)
                        return;
            }

            //Clear all existing data
            CloseAll();

            DlgCreateBallScrew dlgCreateBallScrew = new DlgCreateBallScrew();

            dlgCreateBallScrew.StartPosition = FormStartPosition.CenterParent;

            // Show dialog as a modal dialog and determine if DialogResult = OK. 
            if (dlgCreateBallScrew.ShowDialog(this) == DialogResult.OK)
            {
                applicationData.AppName = dlgCreateBallScrew.ApplicationData.AppName;
                applicationData.Customer = dlgCreateBallScrew.ApplicationData.Customer;
                applicationData.Date = dlgCreateBallScrew.ApplicationData.Date;
                applicationData.Description = dlgCreateBallScrew.ApplicationData.Description;
                applicationData.OurRef = dlgCreateBallScrew.ApplicationData.OurRef;
                applicationData.YourRef = dlgCreateBallScrew.ApplicationData.YourRef;

                motor.AutoSelect = true;

                //Create Moves
                int moveNr = 0;
                moves.MovesList.Add(0, dlgCreateBallScrew.MoveToDest);
                
                if (dlgCreateBallScrew._addReturnMove)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateBallScrew.MoveReturn);
                    moves.MovesList[moveNr].Time = dlgCreateBallScrew.MoveToDest.Time;
                    moves.MovesList[moveNr].Distance = -dlgCreateBallScrew.MoveToDest.Distance;
                    moveNr++;
                }

                if (dlgCreateBallScrew.MoveWait.Time > 0)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateBallScrew.MoveWait);
                    moves.MovesList[moveNr].Time = dlgCreateBallScrew.MoveWait.Time;
                }

                //Create loads
                if (dlgCreateBallScrew._addGearBox)
                {
                    GearBox gearBox = new GearBox("MyGearBox");
                    gearBox.Description = "Not selected";
                    gearBox.AutoSelect = true;

                    iLoads.Add(iLoads.Count(), (ILoad)gearBox);
                }

                if (dlgCreateBallScrew._addCoupling)
                {
                    Coupling coupling = new Coupling("MyCoupling");

                    coupling.CouplingGeometry.CylinderHeight = 80;
                    coupling.CouplingGeometry.OuterDiameter = 40;
                    coupling.CouplingGeometry.InnerDiameter = 10;
                    coupling.CouplingGeometry.Material = Material.Aluminum;

                    coupling.CouplingGeometry.ManualInertiaCalc = true;
                    
                    iLoads.Add(iLoads.Count(), (ILoad)coupling);
                }

                BallScrew ballScrew = dlgCreateBallScrew.BallScrew;
                iLoads.Add(iLoads.Count(), (ILoad)ballScrew);

                LoadMass mass = dlgCreateBallScrew.Mass;
                iLoads.Add(iLoads.Count(), (ILoad)mass);

                if (dlgCreateBallScrew.Force.Force != 0)
                {
                    LoadForce force = dlgCreateBallScrew.Force;
                    iLoads.Add(iLoads.Count(), (ILoad)force);
                }

                //Attach load events and add the loads to the navigator
                foreach (var iload in iLoads)
                    InitializeLoad(iload.Value, iload.Key);

                //Update the ComboBox on the LoadProperties form
                frmProperties.ComboBox_ReLoad();

                //Select the last added load in the loads list
                frmProperties.ComboBox_SelectLoad(iLoads[iLoads.Count - 1].Name);

                //Attach move events, add the moves to the navigator and display the move forms
                foreach (var m in moves.MovesList)
                    initializeMove(m.Value, m.Key);
                
                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm != null)
                    frmResults.Show();
                else
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                    frmResults.Show(dockPanel, DockState.Document);
                }
            }
        }

        private void toolStripButton_DefineToothedBelt_Click(object sender, EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult result3 = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (result3 == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (result3 == DialogResult.Cancel)
                        return;
            }

            //Clear all existing data
            CloseAll();

            DlgCreateToothedBelt dlgCreateToothedBelt = new DlgCreateToothedBelt();

            dlgCreateToothedBelt.StartPosition = FormStartPosition.CenterParent;

            // Show dialog as a modal dialog and determine if DialogResult = OK. 
            if (dlgCreateToothedBelt.ShowDialog(this) == DialogResult.OK)
            {
                applicationData.AppName = dlgCreateToothedBelt.ApplicationData.AppName;
                applicationData.Customer = dlgCreateToothedBelt.ApplicationData.Customer;
                applicationData.Date = dlgCreateToothedBelt.ApplicationData.Date;
                applicationData.Description = dlgCreateToothedBelt.ApplicationData.Description;
                applicationData.OurRef = dlgCreateToothedBelt.ApplicationData.OurRef;
                applicationData.YourRef = dlgCreateToothedBelt.ApplicationData.YourRef;

                motor.AutoSelect = true;

                //Create Moves
                int moveNr = 0;
                moves.MovesList.Add(0, dlgCreateToothedBelt.MoveToDest);

                if (dlgCreateToothedBelt._addReturnMove)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateToothedBelt.MoveReturn);
                    moves.MovesList[moveNr].Time = dlgCreateToothedBelt.MoveToDest.Time;
                    moves.MovesList[moveNr].Distance = -dlgCreateToothedBelt.MoveToDest.Distance;
                    moveNr++;
                }

                if (dlgCreateToothedBelt.MoveWait.Time > 0)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateToothedBelt.MoveWait);
                    moves.MovesList[moveNr].Time = dlgCreateToothedBelt.MoveWait.Time;
                }

                //Create loads
                if (dlgCreateToothedBelt._addGearBox)
                {
                    GearBox gearBox = new GearBox("MyGearBox");
                    gearBox.Description = "Not selected";
                    gearBox.AutoSelect = true;

                    iLoads.Add(iLoads.Count(), (ILoad)gearBox);
                }

                if (dlgCreateToothedBelt._addCoupling)
                {
                    Coupling coupling = new Coupling("MyCoupling");

                    coupling.CouplingGeometry.CylinderHeight = 80;
                    coupling.CouplingGeometry.OuterDiameter = 40;
                    coupling.CouplingGeometry.InnerDiameter = 10;
                    coupling.CouplingGeometry.Material = Material.Aluminum;

                    coupling.CouplingGeometry.ManualInertiaCalc = true;

                    iLoads.Add(iLoads.Count(), (ILoad)coupling);
                }

                ToothedBelt toothedBelt = dlgCreateToothedBelt.ToothedBelt;
                iLoads.Add(iLoads.Count(), (ILoad)toothedBelt);

                LoadMass mass = dlgCreateToothedBelt.Mass;
                iLoads.Add(iLoads.Count(), (ILoad)mass);

                //Attach load events and add the loads to the navigator
                foreach (var iload in iLoads)
                    InitializeLoad(iload.Value, iload.Key);

                //Update the ComboBox on the LoadProperties form
                frmProperties.ComboBox_ReLoad();

                //Select the last added load in the loads list
                frmProperties.ComboBox_SelectLoad(iLoads[iLoads.Count - 1].Name);

                //Attach move events, add the moves to the navigator and display the move forms
                foreach (var m in moves.MovesList)
                    initializeMove(m.Value, m.Key);

                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm != null)
                    frmResults.Show();
                else
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                    frmResults.Show(dockPanel, DockState.Document);
                }
            }
        }

        private void toolStripButton_DefineTurnTable_Click(object sender, EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult result3 = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (result3 == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (result3 == DialogResult.Cancel)
                        return;
            }

            //Clear all existing data
            CloseAll();

            DlgCreateTurnTable dlgCreateTurnTable = new DlgCreateTurnTable();

            dlgCreateTurnTable.StartPosition = FormStartPosition.CenterParent;

            // Show dialog as a modal dialog and determine if DialogResult = OK. 
            if (dlgCreateTurnTable.ShowDialog(this) == DialogResult.OK)
            {
                applicationData.AppName = dlgCreateTurnTable.ApplicationData.AppName;
                applicationData.Customer = dlgCreateTurnTable.ApplicationData.Customer;
                applicationData.Date = dlgCreateTurnTable.ApplicationData.Date;
                applicationData.Description = dlgCreateTurnTable.ApplicationData.Description;
                applicationData.OurRef = dlgCreateTurnTable.ApplicationData.OurRef;
                applicationData.YourRef = dlgCreateTurnTable.ApplicationData.YourRef;

                motor.AutoSelect = true;

                //Create Moves
                int moveNr = 0;
                moves.MovesList.Add(0, dlgCreateTurnTable.MoveToDest);

                if (dlgCreateTurnTable._addReturnMove)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateTurnTable.MoveReturn);
                    moves.MovesList[moveNr].Time = dlgCreateTurnTable.MoveToDest.Time;
                    moves.MovesList[moveNr].Distance = -dlgCreateTurnTable.MoveToDest.Distance;
                    moveNr++;
                }

                if (dlgCreateTurnTable.MoveWait.Time > 0)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateTurnTable.MoveWait);
                    moves.MovesList[moveNr].Time = dlgCreateTurnTable.MoveWait.Time;
                }

                //Create loads
                if (dlgCreateTurnTable._addGearBox)
                {
                    GearBox gearBox = new GearBox("MyGearBox");
                    gearBox.Description = "Not selected";
                    gearBox.AutoSelect = true;

                    iLoads.Add(iLoads.Count(), (ILoad)gearBox);
                }

                if (dlgCreateTurnTable._addCoupling)
                {
                    Coupling coupling = new Coupling("MyCoupling");

                    coupling.CouplingGeometry.CylinderHeight = 80;
                    coupling.CouplingGeometry.OuterDiameter = 40;
                    coupling.CouplingGeometry.InnerDiameter = 10;
                    coupling.CouplingGeometry.Material = Material.Aluminum;

                    coupling.CouplingGeometry.ManualInertiaCalc = true;

                    iLoads.Add(iLoads.Count(), (ILoad)coupling);
                }

                TurnTable turnTable = dlgCreateTurnTable.TurnTable;
                iLoads.Add(iLoads.Count(), (ILoad)turnTable);
                                
                LoadMass mass = dlgCreateTurnTable.Mass;
                
                if (dlgCreateTurnTable.Mass.Mass != null && dlgCreateTurnTable.Mass.Mass != 0)
                    iLoads.Add(iLoads.Count(), (ILoad)mass);

                //Attach load events and add the loads to the navigator
                foreach (var iload in iLoads)
                    InitializeLoad(iload.Value, iload.Key);

                //Update the ComboBox on the LoadProperties form
                frmProperties.ComboBox_ReLoad();

                //Select the last added load in the loads list
                frmProperties.ComboBox_SelectLoad(iLoads[iLoads.Count - 1].Name);

                //Attach move events, add the moves to the navigator and display the move forms
                foreach (var m in moves.MovesList)
                    initializeMove(m.Value, m.Key);

                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm != null)
                    frmResults.Show();
                else
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                    frmResults.Show(dockPanel, DockState.Document);
                }
            }
        }

        private void toolStripButton_DefineRackAndPinion_Click(object sender, EventArgs e)
        {
            if (unsavedData)
            {
                DialogResult result3 = MessageBox.Show("Do you want to save changes?",
                "FileSave",
                MessageBoxButtons.YesNoCancel,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button1);

                if (result3 == DialogResult.Yes)
                {
                    if (mainFileName == null)
                        SaveFileAs();
                    else
                        SaveFile();

                    unsavedData = false;
                }
                else
                    if (result3 == DialogResult.Cancel)
                        return;
            }

            //Clear all existing data
            CloseAll();

            DlgCreateRackAndPinion dlgCreateRackAndPinion = new DlgCreateRackAndPinion();

            dlgCreateRackAndPinion.StartPosition = FormStartPosition.CenterParent;

            // Show dialog as a modal dialog and determine if DialogResult = OK. 
            if (dlgCreateRackAndPinion.ShowDialog(this) == DialogResult.OK)
            {
                applicationData.AppName = dlgCreateRackAndPinion.ApplicationData.AppName;
                applicationData.Customer = dlgCreateRackAndPinion.ApplicationData.Customer;
                applicationData.Date = dlgCreateRackAndPinion.ApplicationData.Date;
                applicationData.Description = dlgCreateRackAndPinion.ApplicationData.Description;
                applicationData.OurRef = dlgCreateRackAndPinion.ApplicationData.OurRef;
                applicationData.YourRef = dlgCreateRackAndPinion.ApplicationData.YourRef;

                motor.AutoSelect = true;

                //Create Moves
                int moveNr = 0;
                moves.MovesList.Add(0, dlgCreateRackAndPinion.MoveToDest);

                if (dlgCreateRackAndPinion._addReturnMove)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateRackAndPinion.MoveReturn);
                    moves.MovesList[moveNr].Time = dlgCreateRackAndPinion.MoveToDest.Time;
                    moves.MovesList[moveNr].Distance = -dlgCreateRackAndPinion.MoveToDest.Distance;
                    moveNr++;
                }

                if (dlgCreateRackAndPinion.MoveWait.Time > 0)
                {
                    moveNr = moves.MovesList.Count();
                    moves.MovesList.Add(moveNr, dlgCreateRackAndPinion.MoveWait);
                    moves.MovesList[moveNr].Time = dlgCreateRackAndPinion.MoveWait.Time;
                }

                //Create loads
                if (dlgCreateRackAndPinion._addGearBox)
                {
                    GearBox gearBox = new GearBox("MyGearBox");
                    gearBox.Description = "Not selected";
                    gearBox.AutoSelect = true;

                    iLoads.Add(iLoads.Count(), (ILoad)gearBox);
                }

                if (dlgCreateRackAndPinion._addCoupling)
                {
                    Coupling coupling = new Coupling("MyCoupling");

                    coupling.CouplingGeometry.CylinderHeight = 80;
                    coupling.CouplingGeometry.OuterDiameter = 40;
                    coupling.CouplingGeometry.InnerDiameter = 10;
                    coupling.CouplingGeometry.Material = Material.Aluminum;

                    coupling.CouplingGeometry.ManualInertiaCalc = true;

                    iLoads.Add(iLoads.Count(), (ILoad)coupling);
                }

                RackAndPinion rackAndPinion = dlgCreateRackAndPinion.RackAndPinion;
                iLoads.Add(iLoads.Count(), (ILoad)rackAndPinion);

                LoadMass mass = dlgCreateRackAndPinion.Mass;
                iLoads.Add(iLoads.Count(), (ILoad)mass);

                //Attach load events and add the loads to the navigator
                foreach (var iload in iLoads)
                    InitializeLoad(iload.Value, iload.Key);

                //Update the ComboBox on the LoadProperties form
                frmProperties.ComboBox_ReLoad();

                //Select the last added load in the loads list
                frmProperties.ComboBox_SelectLoad(iLoads[iLoads.Count - 1].Name);

                //Attach move events, add the moves to the navigator and display the move forms
                foreach (var m in moves.MovesList)
                    initializeMove(m.Value, m.Key);

                //Set focus to the 'Results'-form
                IDockContent frm = FindDocument("Results");

                if (frm != null)
                    frmResults.Show();
                else
                {
                    frmResults = null;
                    frmResults = new FrmResults(moves, iLoads, motor, applicationData);
                    frmResults.Show(dockPanel, DockState.Document);
                }
            }
        }
    }
}